------------------------------------------------------------------------------
-- opb_m_wrapper.vhd - entity/architecture pair
------------------------------------------------------------------------------
-- IMPORTANT:
-- DO NOT MODIFY THIS FILE EXCEPT IN THE DESIGNATED SECTIONS.
--
-- SEARCH FOR --USER TO DETERMINE WHERE CHANGES ARE ALLOWED.
--
-- TYPICALLY, THE ONLY ACCEPTABLE CHANGES INVOLVE ADDING NEW
-- PORTS AND GENERICS THAT GET PASSED THROUGH TO THE INSTANTIATION
-- OF THE USER_LOGIC ENTITY.
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2004 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ** YOU MAY COPY AND MODIFY THESE FILES FOR YOUR OWN INTERNAL USE SOLELY  **
-- ** WITH XILINX PROGRAMMABLE LOGIC DEVICES AND XILINX EDK SYSTEM OR       **
-- ** CREATE IP MODULES SOLELY FOR XILINX PROGRAMMABLE LOGIC DEVICES AND    **
-- ** XILINX EDK SYSTEM. NO RIGHTS ARE GRANTED TO DISTRIBUTE ANY FILES      **
-- ** UNLESS THEY ARE DISTRIBUTED IN XILINX PROGRAMMABLE LOGIC DEVICES.     **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          opb_m_wrapper.vhd
-- Version:           1.00.a
-- Description:       Top level design, instantiates IPIF and user logic.
-- Date:              Wed Aug 23 12:14:28 2006 (by Create and Import Peripheral Wizard)
-- VHDL-Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
-- 	active low signals:                    "*_n"
-- 	clock signals:                         "clk", "clk_div#", "clk_#x"
-- 	reset signals:                         "rst", "rst_n"
-- 	generics:                              "C_*"
-- 	user defined types:                    "*_TYPE"
-- 	state machine next state:              "*_ns"
-- 	state machine current state:           "*_cs"
-- 	combinatorial signals:                 "*_com"
-- 	pipelined or register delay signals:   "*_d#"
-- 	counter signals:                       "*cnt*"
-- 	clock enable signals:                  "*_ce"
-- 	internal version of output port:       "*_i"
-- 	device pins:                           "*_pin"
-- 	ports:                                 "- Names begin with Uppercase"
-- 	processes:                             "*_PROCESS"
-- 	component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library opb_m_wrapper_v1_00_a;
use opb_m_wrapper_v1_00_a.all;

------------------------------------------------------------------------------
-- Definition of Generics:
--   C_BASEADDR                   -- User logic base address
--   C_HIGHADDR                   -- User logic high address
--   C_OPB_AWIDTH                 -- OPB address bus width
--   C_OPB_DWIDTH                 -- OPB data bus width
--   C_FAMILY                     -- Target FPGA architecture
--
-- Definition of Ports:
--   OPB_Clk                      -- OPB Clock
--   OPB_Rst                      -- OPB Reset
--   Sl_DBus                      -- Slave data bus
--   Sl_errAck                    -- Slave error acknowledge
--   Sl_retry                     -- Slave retry
--   Sl_toutSup                   -- Slave timeout suppress
--   Sl_xferAck                   -- Slave transfer acknowledge
--   OPB_ABus                     -- OPB address bus
--   OPB_BE                       -- OPB byte enable
--   OPB_DBus                     -- OPB data bus
--   OPB_RNW                      -- OPB read/not write
--   OPB_select                   -- OPB select
--   OPB_seqAddr                  -- OPB sequential address
--   M_ABus                       -- Master address bus
--   M_BE                         -- Master byte enables
--   M_busLock                    -- Master buslock
--   M_request                    -- Master bus request
--   M_RNW                        -- Master read, not write
--   M_select                     -- Master select
--   M_seqAddr                    -- Master sequential address
--   OPB_errAck                   -- OPB error acknowledge
--   OPB_MGrant                   -- OPB bus grant
--   OPB_retry                    -- OPB bus cycle retry
--   OPB_timeout                  -- OPB timeout error
--   OPB_xferAck                  -- OPB transfer acknowledge
--
--------------------------------------------------------------------------------
-- Entity section
--------------------------------------------------------------------------------

entity opb_m_wrapper is
	generic
	(
		-- ADD USER GENERICS BELOW THIS LINE ---------------
		--USER generics added here
		-- ADD USER GENERICS ABOVE THIS LINE ---------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol parameters, do not add to or delete
		C_BASEADDR	: std_logic_vector	:= X"00000000";
		C_HIGHADDR	: std_logic_vector	:= X"0000FFFF";
		C_OPB_AWIDTH	: integer	:= 32;
		C_OPB_DWIDTH	: integer	:= 32;
		C_FAMILY	: string	:= "virtex2p"
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
	port
	(
		-- ADD USER PORTS BELOW THIS LINE ------------------
		u_OPB_Clk	: out	std_logic;
		u_OPB_Rst	: out	std_logic;
		u_M_DBus	: in	std_logic_vector(0 to C_OPB_DWIDTH-1);
		u_OPB_DBus	: out	std_logic_vector(0 to C_OPB_DWIDTH-1);
		u_M_ABus	: in	std_logic_vector(0 to C_OPB_AWIDTH-1);
		u_M_BE	: in	std_logic_vector(0 to C_OPB_DWIDTH/8-1);
		u_M_busLock	: in	std_logic;
		u_M_request	: in	std_logic;
		u_M_RNW	: in	std_logic;
		u_M_select	: in	std_logic;
		u_M_seqAddr	: in	std_logic;
		u_OPB_errAck	: out	std_logic;
		u_OPB_MGrant	: out	std_logic;
		u_OPB_retry	: out	std_logic;
		u_OPB_timeout	: out	std_logic;
		u_OPB_xferAck	: out	std_logic;
		-- ADD USER PORTS ABOVE THIS LINE ------------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol ports, do not add to or delete
		OPB_Clk	: in	std_logic;
		OPB_Rst	: in	std_logic;
		M_DBus	: out	std_logic_vector(0 to C_OPB_DWIDTH-1);
		OPB_DBus	: in	std_logic_vector(0 to C_OPB_DWIDTH-1);
		M_ABus	: out	std_logic_vector(0 to C_OPB_AWIDTH-1);
		M_BE	: out	std_logic_vector(0 to C_OPB_DWIDTH/8-1);
		M_busLock	: out	std_logic;
		M_request	: out	std_logic;
		M_RNW	: out	std_logic;
		M_select	: out	std_logic;
		M_seqAddr	: out	std_logic;
		OPB_errAck	: in	std_logic;
		OPB_MGrant	: in	std_logic;
		OPB_retry	: in	std_logic;
		OPB_timeout	: in	std_logic;
		OPB_xferAck	: in	std_logic
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);

	attribute SIGIS : string;
	attribute SIGIS of OPB_Clk : signal is "Clk";
	attribute SIGIS of OPB_Rst : signal is "Rst";

end entity opb_m_wrapper;

--------------------------------------------------------------------------------
-- Architecture section
--------------------------------------------------------------------------------

architecture IMP of opb_m_wrapper is

begin
  
  u_OPB_Clk <= OPB_Clk;
  u_OPB_Rst <= OPB_Rst;
  M_DBus <= u_M_DBus;
  u_OPB_DBus <= OPB_DBus;
  M_ABus <= u_M_ABus;
  M_BE <= u_M_BE;
  M_busLock <= u_M_busLock;
  M_request <= u_M_request;
  M_RNW <= u_M_RNW;
  M_select <= u_M_select;
  M_seqAddr <= u_M_seqAddr;
  u_OPB_errAck <= OPB_errAck;
  u_OPB_MGrant <= OPB_MGrant;
  u_OPB_retry <= OPB_retry;
  u_OPB_timeout <= OPB_timeout;
  u_OPB_xferAck <= OPB_xferAck;

end IMP;
