---------------------------------------------------------------------------
--
--  File        : ac_input.vhd
--
--  Title:      : AC-input divider and phase shifter
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Change Log  :
--
---------------------------------------------------------------------------
 
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library unisim;
use unisim.vcomponents.all;

entity ac_input is
  port (
    clk         : in std_logic;
    usec_enable : in std_logic;
    reset       : in std_logic;
    ac_divider  : in std_logic_vector(7 downto 0);
    shift       : in std_logic_vector(7 downto 0);
    ac_in       : in std_logic;
    bypass      : in std_logic;
    trigger     : out std_logic);
end ac_input;

architecture structure of ac_input is

begin

  ac_input_divider : process (clk, usec_enable, reset, ac_divider,
			      ac_in, bypass)
    variable phase_counter : std_logic_vector(8 downto 0) := "000000000";
    variable counter_100us : std_logic_vector(7 downto 0) := X"00";
    variable divider       : std_logic_vector(8 downto 0) := "000000000";
    variable ac_ena        : std_logic;
    variable ac_div        : std_logic;
    variable ac_in_1       : std_logic;
    variable ac_in_2       : std_logic;
    variable ac_in_3       : std_logic;
    variable ena_100us     : std_logic;
  begin
    if rising_edge(clk) then
      -- Phase shifter
      trigger <= '0';
      if ena_100us = '1' and bypass = '0' then
	if phase_counter(8) = '0' then
	  phase_counter := phase_counter - 1;
	  if phase_counter(8) = '1' then
	    trigger <= '1';
	  end if;
	end if;
      end if;
      if bypass = '1' then
	trigger <= ac_ena;
      end if;
      
      if ac_div = '1' then
	phase_counter := '0' & shift;
      end if;
      if reset = '1' then
	phase_counter := "111111111";
      end if;
      
      ena_100us := '0';
      if usec_enable = '1' then
	if counter_100us(7) = '1' then
	  ena_100us := '1';
	  counter_100us := X"62";
	else
	  counter_100us := counter_100us - 1;
	end if;
      end if;
	
      if ac_div = '1' or reset = '1' then
	counter_100us := X"62";
      end if;
      
      -- AC divider
      ac_div := '0';
      if ac_ena = '1' then
	ac_div := divider(8);
	if divider(8) = '1' then
	  divider := '0' & (ac_divider - 1);
	end if;
	divider := divider - 1;
      end if;
      
      if reset = '1' then
	ac_div := '0';
	divider := "000000000";
      end if;

      -- AC input synchronisation
      ac_ena := '0';
      if ac_in_2 = '1' and ac_in_3 = '0' then
	ac_ena := '1';
      end if;
      ac_in_3 := ac_in_2;
      ac_in_2 := ac_in_1;
      ac_in_1 := ac_in;
      
    end if;

  end process;

end structure;
