---------------------------------------------------------------------------
--
--  File        : cpcievg220_pkg.vhd
--
--  Title       : Event Generator cPCI/PXI-EVG-220 board specific package
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use work.evg_pkg.all;

package evg_bs_pkg is
  -- Local OPB address definitions
  constant LOCAL_BASE_MASK  : std_logic_vector := X"80000000";
  constant EVG_LOCAL_BASE   : std_logic_vector := X"00000000";

  -- Event Generator configuration parameters
  constant C_EVG_SEQUENCE_RAMS : integer := 2;
  constant C_EVG_EVENT_TRIGGERS : integer := 8;
  constant C_EVG_MULTIPLEXED_COUNTERS : integer := 8;
  -- ACIN is connected to TTLIN(0)
  constant C_EVG_TTL_INPUTS    : integer := 1;
  -- C_EVG_TTL_OUTPUTS defines the number of front panel TTL outputs
  constant C_EVG_TTL_OUTPUTS   : integer := 0;
  -- C_EVG_UNIV_INPUTS defines the number of Universal outputs
  -- = twice the number of Universal I/O slots
  constant C_EVG_UNIV_INPUTS  : integer := 10;
  -- C_EVG_UNIV_OUTPUTS defines the number of Universal outputs
  -- = twice the number of Universal I/O slots
  constant C_EVG_UNIV_OUTPUTS  : integer := 10;
  -- C_EVG_TB_INPUTS defines the number of Transition Board/Rear I/O
  -- PXI star trigger/trigger bus inputs
  constant C_EVG_TB_INPUTS    : integer := 0;
  -- C_EVG_TB_OUTPUTS defines the number of Transition Board/Rear I/O
  -- PXI star trigger/trigger bus outputs
  constant C_EVG_TB_OUTPUTS    : integer := 0;
  -- C_EVG_USE_RESYNC defines if the board supports the event clock
  -- resynchronizing circuit to reduce jitter (this is a requirement
  -- for CML outputs)
  constant C_EVG_USE_RESYNC    : boolean := FALSE;
  -- C_EVG_REF_BREFCLK defines if reference clock from fractional
  -- synthesizer is on BREFCLKN/BREFCLKP (TRUE) or
  -- BREFCLK2N/BREFCLK2P (FALSE)
  constant C_EVG_REF_BREFCLK   : boolean := FALSE;
  -- C_EVG_RXPOLARITY defines the event link receiver polarity
  -- '0' - normal polarity, '1' - inverted polarity
  constant C_EVG_RXPOLARITY    : std_logic := '1';
  -- C_EVG_TXPOLARITY defines the event link receiver polarity
  -- '0' - normal polarity, '1' - inverted polarity
  constant C_EVG_TXPOLARITY    : std_logic := '1';
--  constant C_EVG_MICREL_WORD   : std_logic_vector := X"0C9282A6";
  constant C_EVG_MICREL_WORD   : std_logic_vector := X"0C928166";
  constant C_FORM_FACTOR       : std_logic_vector(3 downto 0) := X"0";
end evg_bs_pkg;
