---------------------------------------------------------------------------
--                                                                       --
--  File        : refclock.vhd                                          --
--                                                                       --
--  Title       : Reference clock input circuit
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--                                                                       --
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
LIBRARY UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity refclock is
  port (
    BREFCLKN    : in std_logic; -- Reference clock
    BREFCLKP    : in std_logic;
    reset       : in std_logic;
    brefclk     : out std_logic;
    brefclk_dcm : out std_logic);
end refclock;

architecture structure of refclock is

  signal brefclk_in : std_logic;
  signal clkfb_in : std_logic;
  signal gnd : std_logic;
  
  component bufgmux is
    port (
      O : out std_ulogic;
      I0 : in std_ulogic;
      I1 : in std_ulogic;
      S : in std_ulogic);
  end component;

  component ibufgds
  port (O  : out std_ulogic;
        I  : in std_ulogic;
        IB : in std_ulogic);
  end component;

  component bufg
    port (
      O : out std_logic;
      I : in  std_logic
      );
  end component;
  
begin

  gnd <= '0';
  
  i_ref_ibufgds2 : ibufgds
    port map (
      O => brefclk_in,
      I => BREFCLKP,
      IB => BREFCLKN);

  CLK0_BUFG_INST : BUFGMUX
     port map (
      O => clkfb_in,
      I0 => brefclk_in,
      I1 => gnd,
      S => gnd);

  brefclk <= brefclk_in;
  brefclk_dcm <= clkfb_in;

end structure;
  
