---------------------------------------------------------------------------
--                                                                       --
--  File        : trigger.vhd                                            --
--                                                                       --
--  Title       : Event generator trigger event control                  --
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--                                                                       --
--  Description :                                                        --
--                                                                       --
--                                                                       --
---------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY trigger IS
  PORT (
    trigger_input  : IN    std_logic;
    trigger_enable : IN    std_logic;
    event_pending  : OUT   std_logic;
    event_ack      : IN    std_logic;
    event_clk      : IN    std_logic);
END trigger;

ARCHITECTURE rtl OF trigger IS

BEGIN

  process (event_clk, trigger_enable, event_ack, trigger_input)
    variable sync_to_clk : std_logic_vector(3 downto 1);
  begin
    if rising_edge(event_clk) then
      if sync_to_clk(3 downto 2) = "01" then
	event_pending <= '1';
      end if;
      if event_ack = '1' or trigger_enable = '0' then
	event_pending <= '0';
      end if;
      sync_to_clk := sync_to_clk(2 downto 1) & trigger_input;
    end if;
  end process;
  
END rtl;
