TARGET_FAMILY = virtex2p
XST_OPTIONS = -arch $(TARGET_FAMILY)
PART = xc2vp20-ff896-6

CORE_DIR = core
TOP = vmeevg230
BUILD_DIR = build/vmeevg230
DESIGN = $(BUILD_DIR)/$(TOP)
EDK_DIR = edk/vmeevg230
EDK_PRJ = system
VHDL_SRC = vhdl/ac_input.vhd \
        vhdl/event_analyser.vhd \
        vhdl/event_dcm.vhd \
        vhdl/evg_pkg.vhd \
        vhdl/vmeevg230_pkg.vhd \
        vhdl/memory_buffer.vhd \
        vhdl/mgt_rx.vhd \
        vhdl/mgt_tx.vhd \
        vhdl/mx_counter.vhd \
        vhdl/seq_ramb.vhd \
        vhdl/micrel_load.vhd \
        vhdl/opb_evg.vhd \
        vhdl/recclk_out.vhd \
        vhdl/refclock.vhd \
        vhdl/sync_domains.vhd \
        vhdl/transceiver.vhd \
        vhdl/trigger.vhd \
        vhdl/vmeevg230.vhd \
        vhdl/vmeevg230_CR.vhd \
        opb_ubicom/opb_ubicom_pkg.vhd \
        opb_ubicom/opb_ubicom.vhd \
        opb_vme/opb_vme_pkg.vhd \
        opb_vme/opb_vme.vhd \
        opb_vme/opb_cr_csr.vhd \
        $(EDK_DIR)/hdl/$(EDK_PRJ).vhd
UCF_SRC = vhdl/vmeevg230.ucf
UT_SRC = vhdl/vmeevg230.ut
# This project is not using the PowerPC, no software project
# EXECUTABLE_ELF = $(EDK_DIR)/ppc405_1/code/executable.elf
# EDK_BMMFILE = $(EDK_DIR)/implementation/$(EDK_PRJ)_stub.bmm
# BMMFILE = $(BUILD_DIR)/edkBmmFile.bmm
# BMMFILE_BD = $(BUILD_DIR)/edkBmmFile_bd.bmm
# EDK_SW_SOURCES = $(EDK_DIR)/sw/linker_script \
#         $(EDK_DIR)/sw/sseq_control.c \
#         $(EDK_DIR)/sw/sseq_irq.S
FS_UPGRADE_HEAD = ip2k/vmeevg230.upgrade.head.bin
FS_IMAGE = build/vmeevg230/vmeevg230.bin

XST_SCRIPT = vmeevg230.xst_script

all: $(FS_IMAGE)

$(FS_IMAGE): $(FS_UPGRADE_HEAD) $(DESIGN).bit
	cat $(FS_UPGRADE_HEAD) $(DESIGN).bit > $@

$(EDK_DIR)/hdl/$(EDK_PRJ).vhd: $(EDK_DIR)/$(EDK_PRJ).xmp \
        $(EDK_DIR)/$(EDK_PRJ).mhs $(EDK_DIR)/$(EDK_PRJ).mss
	make -f $(EDK_PRJ).make -C $(EDK_DIR) netlist

$(DESIGN).ngc : $(VHDL_SRC) $(XST_SCRIPT)
	xst -ifn $(XST_SCRIPT) -intstyle xflow

$(DESIGN).ngd : $(DESIGN).ngc $(UCF_SRC)
	ngdbuild -intstyle xflow -dd _ngo -uc $(UCF_SRC) \
        -sd $(CORE_DIR) -sd $(EDK_DIR)/implementation \
        -p $(PART) $(DESIGN).ngc $(DESIGN).ngd

$(DESIGN)_map.ncd : $(DESIGN).ngd
	map -intstyle xflow -p $(PART) -cm area -pr b -k 4 -c 100 -tx off \
        -o $(DESIGN)_map.ncd $(DESIGN).ngd $(DESIGN).pcf

$(DESIGN).ncd : $(DESIGN)_map.ncd
	par -w -intstyle xflow -ol std -t 1 $(DESIGN)_map.ncd $(DESIGN).ncd \
        $(DESIGN).pcf

$(DESIGN).bit : $(DESIGN).ncd $(UT_SRC)
	bitgen -intstyle xflow -f $(UT_SRC) $(DESIGN).ncd

clean:
	-rm -rf $(BUILD_DIR)/* xst/* vhdl/*~ *~ *.lso *.srp _ngo/* \
        xilinx_device_details.xml
	make -f $(EDK_PRJ).make -C $(EDK_DIR) clean
