-- mico top wrapper

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_bs_pkg.all;
use work.evr_pkg.all;

entity crioevr is
  port (
    ddr2_sdramem_ddr_ba    : out std_logic_vector(1 downto 0);
    ddr2_sdramem_ddr_we_n  : out std_logic;
    ddr2_sdramem_ddr_cke   : out std_logic_vector(0 downto 0);
    ddr2_sdramem_ddr_cs_n  : out std_logic_vector(0 downto 0);
    ddr2_sdramem_ddr_cas_n : out std_logic;
    ddr2_sdramem_ddr_addr  : out std_logic_vector(12 downto 0);
    ddr2_sdramem_ddr_clk   : out std_logic_vector(0 downto 0);
    ddr2_sdramem_ddr_odt   : out std_logic_vector(0 downto 0);
    ddr2_sdramem_ddr_ras_n : out std_logic;
    ddr2_sdramem_ddr_dm    : out std_logic_vector(1 downto 0);
    ddr2_sdramem_ddr_dqs   : inout std_logic_vector(1 downto 0);
    ddr2_sdramem_ddr_data  : inout std_logic_vector(15 downto 0);

    MDIO      : inout std_logic;
    RX_ER     : in  std_logic;
    COL       : in  std_logic;
    RX_DV     : in  std_logic;
    CRS       : in  std_logic;
    MDC       : out std_logic;
    TX_CLK    : in  std_logic;
    RX_CLK    : in  std_logic;
    PWR_DOWN  : out std_logic;
    TXD       : out std_logic_vector(3 downto 0);
    TX_EN     : out std_logic;
    RXD       : in  std_logic_vector(3 downto 0);
    PHYRESETn : out std_logic;

    EECS      : out std_logic;
    EESK      : out std_logic;
    EEDI      : out std_logic;
    EEDO      : in  std_logic;
    FLASHD    : out std_logic_vector(1 downto 0);
    FLASHC    : out std_logic;
    FLASHQ    : in  std_logic_vector(1 downto 0);
    FLASHS_N  : out std_logic_vector(1 downto 0);
    CCLK      : out std_logic;
    SYNCVL    : out std_logic;
    SYNC1V8   : out std_logic;
    SYNC1V2   : out std_logic;
    CLK25     : in  std_logic;

    USER      : inout std_logic_vector(8 downto 0);
    IDSEL_IN  : in  std_logic;
    IDSEL_PU  : out std_logic;
    SLEEP     : in  std_logic;

    PROGSK    : out std_logic;
    PROGCS    : out std_logic;
    PROGDI    : out std_logic;
    LOCKED    : in  std_logic;

    MOD_TXF   : in    std_logic;
    MOD_SDA   : in    std_logic;
    MOD_SCL   : out   std_logic;
    MOD_TXD   : out   std_logic;
    MOD_DEF0  : in    std_logic;
    MOD_LOS   : in    std_logic;

    refclkp         : in std_logic;
    refclkn         : in std_logic;
    hdinp0          : in std_logic;
    hdinn0          : in std_logic;
    hdinp1          : in std_logic;
    hdinn1          : in std_logic;
    hdinp2          : in std_logic;
    hdinn2          : in std_logic;
    hdinp3          : in std_logic;
    hdinn3          : in std_logic;

    hdoutp0         : out std_logic;
    hdoutn0         : out std_logic;
    hdoutp1         : out std_logic;
    hdoutn1         : out std_logic;
    hdoutp2         : out std_logic;
    hdoutn2         : out std_logic;
    hdoutp3         : out std_logic;
    hdoutn3         : out std_logic);
end crioevr;

architecture structure of crioevr is

  attribute nopad : string;
  attribute nopad of
    refclkp, refclkn,
    hdinp0, hdinn0, hdinp1, hdinn1,
    hdinp2, hdinn2, hdinp3, hdinn3,
    hdoutp0, hdoutn0, hdoutp1, hdoutn1,
    hdoutp2, hdoutn2, hdoutp3, hdoutn3 : signal is "true";

  signal gnd : std_logic;
  signal vcc : std_logic;
  signal gnd_vec : std_logic_vector(7 downto 0);

  signal clk_i : std_logic;
  signal ts_coresys_clk : std_logic;
  signal reset_n : std_logic;
  signal PLL_LOCK : std_logic;

  signal spi_MOSI : std_logic;
  signal spi_MISO : std_logic;
  signal spi_SCLK : std_logic;
  signal spi_SS_N : std_logic_vector(2 downto 0);

  signal slave_passthruclk      : std_logic;
  signal slave_passthrurst      : std_logic;
  signal slave_passthruslv_adr  : std_logic_vector(31 downto 0);
  signal slave_passthruslv_master_data : std_logic_vector(31 downto 0);
  signal slave_passthruslv_slave_data : std_logic_vector(31 downto 0);
  signal slave_passthruslv_strb : std_logic;
  signal slave_passthruslv_cyc  : std_logic;
  signal slave_passthruslv_ack  : std_logic;
  signal slave_passthruslv_err  : std_logic;
  signal slave_passthruslv_rty  : std_logic;
  signal slave_passthruslv_sel  : std_logic_vector(3 downto 0);
  signal slave_passthruslv_we   : std_logic;
  signal slave_passthruslv_bte  : std_logic_vector(1 downto 0);
  signal slave_passthruslv_cti  : std_logic_vector(2 downto 0);
  signal slave_passthruslv_lock : std_logic;
  signal slave_passthruintr_active_high : std_logic;

  signal TTLIN  : std_logic_vector(0 to C_EVR_TTL_INPUTS-1);
  signal GPIO   : std_logic_vector(C_EVR_GPIOS-1 downto 0);
  signal TBIN   : std_logic_vector(0 to C_EVR_TB_INPUTS-1);
  signal BREFCLKN : std_logic;
  signal BREFCLKP : std_logic;
  signal BREFCLK2N : std_logic;
  signal BREFCLK2P : std_logic;

  signal RECCLKN   : std_logic;
  signal RECCLKP   : std_logic;
  
  signal EVRXN  : std_logic;
  signal EVRXP  : std_logic;
  signal EVTXN  : std_logic;
  signal EVTXP  : std_logic;
  signal FBRXN  : std_logic;
  signal FBRXP  : std_logic;
  signal FBTXN  : std_logic;
  signal FBTXP  : std_logic;
  
  signal CMLRXN : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
  signal CMLRXP : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
  signal CMLTXN : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
  signal CMLTXP : std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);

  signal TXLOOPBACK : std_logic;
  signal RXLOOPBACK : std_logic;

  signal clk                 : std_logic;
  signal evr_OPB_Rst         : std_logic;
  signal evr_Sl_DBus         : std_logic_vector(0 to 31);
  signal evr_Sl_errAck       : std_logic;
  signal evr_Sl_retry        : std_logic;
  signal evr_Sl_toutSup      : std_logic;
  signal evr_Sl_xferAck      : std_logic;
  signal evr_OPB_ABus        : std_logic_vector(0 to 31);
  signal evr_OPB_BE          : std_logic_vector(0 to 3);
  signal evr_OPB_DBus        : std_logic_vector(0 to 31);
  signal evr_OPB_RNW         : std_logic;
  signal evr_OPB_select      : std_logic;
  signal evr_OPB_seqAddr     : std_logic;
  signal evr_irq             : std_logic;

  signal univin              : std_logic_vector(0 to C_EVR_UNIV_INPUTS-1);
  signal univout             : std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);

  signal lm32_gpio           : std_logic_vector(3 downto 0);
  signal IDSEL_pullup        : std_logic;

  signal uartSIN             : std_logic;
  signal uartSOUT            : std_logic;
  
  component pll25
    port (
      CLK   : in std_logic;
      CLKOP : out std_logic;
      CLKOK : out std_logic;
      LOCK  : out std_logic);
  end component;
  
  component gpll125
    port (
      CLK   : in std_logic;
      CLKOP : out std_logic;
      CLKOK : out std_logic;
      LOCK  : out std_logic);
  end component;

  component platform
    port (
      clk_i                  : in std_logic;
      reset_n                : in std_logic;
      gpioPIO_OUT            : out std_logic_vector(3 downto 0);
      uartSIN                : in std_logic;
      uartSOUT               : out std_logic;
      ddr2_sdramem_ddr_data  : inout std_logic_vector(15 downto 0);
      ddr2_sdramem_ddr_dqs   : inout std_logic_vector(1 downto 0);
      ddr2_sdramem_ddr_clk   : out std_logic_vector(0 downto 0);
      ddr2_sdramem_ddr_cke   : out std_logic_vector(0 downto 0);
      ddr2_sdramem_ddr_ras_n : out std_logic;
      ddr2_sdramem_ddr_cas_n : out std_logic;
      ddr2_sdramem_ddr_we_n  : out std_logic;
      ddr2_sdramem_ddr_cs_n  : out std_logic_vector(0 downto 0);
      ddr2_sdramem_ddr_odt   : out std_logic_vector(0 downto 0);
      ddr2_sdramem_ddr_dm    : out std_logic_vector(1 downto 0);
      ddr2_sdramem_ddr_ba    : out std_logic_vector(1 downto 0);
      ddr2_sdramem_ddr_addr  : out std_logic_vector(12 downto 0);
      ts_mac_coregtx_clk     : out std_logic;
      ts_mac_corerxmac_clk   : out std_logic;
      ts_mac_coretxmac_clk   : out std_logic;
      ts_mac_coretx_clk      : in std_logic;
      ts_mac_coremdc         : out std_logic;
      ts_mac_coresys_clk     : in std_logic;
      ts_mac_corerx_clk      : in std_logic;
      ts_mac_corerxd         : in std_logic_vector(7 downto 0);
      ts_mac_corerx_dv       : in std_logic;
      ts_mac_corerx_er       : in std_logic;
      ts_mac_corecol         : in std_logic;
      ts_mac_corecrs         : in std_logic;
      ts_mac_coretxd         : out std_logic_vector(7 downto 0);
      ts_mac_coretx_en       : out std_logic;
      ts_mac_coretx_er       : out std_logic;
      ts_mac_coremdio        : inout std_logic;
      spiMISO_MASTER         : in std_logic;
      spiMOSI_MASTER         : out std_logic;
      spiSS_N_MASTER         : out std_logic_vector(2 downto 0);
      spiSCLK_MASTER         : out std_logic;
      slave_passthruclk      : out std_logic;
      slave_passthrurst      : out std_logic;
      slave_passthruslv_adr  : out std_logic_vector(31 downto 0);
      slave_passthruslv_master_data : out std_logic_vector(31 downto 0);
      slave_passthruslv_slave_data : in std_logic_vector(31 downto 0);
      slave_passthruslv_strb : out std_logic;
      slave_passthruslv_cyc  : out std_logic;
      slave_passthruslv_ack  : in std_logic;
      slave_passthruslv_err  : in std_logic;
      slave_passthruslv_rty  : in std_logic;
      slave_passthruslv_sel  : out std_logic_vector(3 downto 0);
      slave_passthruslv_we   : out std_logic;
      slave_passthruslv_bte  : out std_logic_vector(1 downto 0);
      slave_passthruslv_cti  : out std_logic_vector(2 downto 0);
      slave_passthruslv_lock : out std_logic;
      slave_passthruintr_active_high : in std_logic);
  end component;

  attribute black_box_pad_pin : string;
  attribute black_box_pad_pin of platform : component is "ddr2_sdramem_ddr_data(15:0),ddr2_sdramem_ddr_dqs(1:0)";

  component opb_evr is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_BASEADDR         : std_logic_vector;
      C_BASEMASK         : std_logic_vector;
      C_OPB_AWIDTH       : integer := 32;
      C_OPB_DWIDTH       : integer := 32;
      C_FAMILY           : string  := "virtex2p");
    port (
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      Sl_DBus         : out   std_logic_vector(0 to 31);
      Sl_errAck       : out   std_logic;
      Sl_retry        : out   std_logic;
      Sl_toutSup      : out   std_logic;
      Sl_xferAck      : out   std_logic;
      OPB_ABus        : in    std_logic_vector(0 to 31);
      OPB_BE          : in    std_logic_vector(0 to 3);
      OPB_DBus        : in    std_logic_vector(0 to 31);
      OPB_RNW         : in    std_logic;
      OPB_select      : in    std_logic;
      OPB_seqAddr     : in    std_logic;
      
      ip_irq_out      : out std_logic;
      
      TTLIN           : in std_logic_vector(0 to C_EVR_TTL_INPUTS-1);
      TTLOUT          : out std_logic_vector(0 to C_EVR_TTL_OUTPUTS-1);
      UNIVIN          : in std_logic_vector(0 to C_EVR_UNIV_INPUTS-1);
      UNIVOUT         : out std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);

      GPIO            : inout std_logic_vector(C_EVR_GPIOS-1 downto 0);

      TBIN            : in std_logic_vector(0 to C_EVR_TB_INPUTS-1);
      TBOUT           : out std_logic_vector(0 to C_EVR_TB_OUTPUTS-1);
      
      -- Differential clocks from special brefclkx and brefclk2x pins
      -- Either pair may provide reference from fractional synthesizer
      BREFCLKN        : in std_logic; -- Reference clock input from
      BREFCLKP        : in std_logic; -- fractional synthesizer
      BREFCLK2N       : in std_logic; -- Resynchronised clock input
      BREFCLK2P       : in std_logic; -- from FPGA external FF

      refclk_out      : out std_logic; -- reference clock output
      brefclk_out     : out std_logic; -- reference clock output for MGT
      refclk2_out     : out std_logic; -- reference clock output
      brefclk2_out    : out std_logic; -- reference clock output for MGT
      
      RECCLKN         : out std_logic; -- Recovered clock output
      RECCLKP         : out std_logic;
      
      -- Front panel LEDs
      LED_fail        : out std_logic;
      LED_rx_link     : out std_logic;
      LED_ena         : out std_logic;
      LED_event_in    : out std_logic;
      LED_event_out   : out std_logic;
      LED_rx_fail     : out std_logic;
      LED_error       : out std_logic;
      
      -- MGT I/O pins
      EVRXN           : in std_logic;  -- Event Link RXN
      EVRXP           : in std_logic;  -- Event Link RXN
      EVTXN           : out std_logic; -- Event Link TXN
      EVTXP           : out std_logic; -- Event Link TXN
      FBRXN           : in std_logic;  -- Event Link RXN
      FBRXP           : in std_logic;  -- Event Link RXN
      FBTXN           : out std_logic; -- Event Link TXN
      FBTXP           : out std_logic; -- Event Link TXN
      CMLRXN          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLRXP          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXN          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXP          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);

      -- Micrel SY87729L serial bus signals
      CGSK        : out std_logic;
      CGDI        : out std_logic;
      CGCS        : out std_logic;
      CGLOCK      : in std_logic;

      TXLOOPBACK  : out std_logic; -- when '1' loop back FPGA TX to FPGA RX
      RXLOOPBACK  : out std_logic; -- when '1' loop back optical RX signal
                                   -- to transceiver TX port
    
      debug       : out std_logic_vector(39 downto 0);

      -- Lattice ECP2M dedicated transceiver pins
      refclkp         : in std_logic;
      refclkn         : in std_logic;
      hdinp0          : in std_logic;
      hdinn0          : in std_logic;
      hdinp1          : in std_logic;
      hdinn1          : in std_logic;
      hdinp2          : in std_logic;
      hdinn2          : in std_logic;
      hdinp3          : in std_logic;
      hdinn3          : in std_logic;
      
      hdoutp0         : out std_logic;
      hdoutn0         : out std_logic;
      hdoutp1         : out std_logic;
      hdoutn1         : out std_logic;
      hdoutp2         : out std_logic;
      hdoutn2         : out std_logic;
      hdoutp3         : out std_logic;
      hdoutn3         : out std_logic);
  end component;

begin

--  i_pll : pll25
--    port map (
--      CLK => CLK25,
--      CLKOK => clk_i,
--      LOCK => PLL_LOCK);

  i_gpll : gpll125
    port map (
      CLK => CLK25,
      CLKOP => ts_coresys_clk,
--      CLKOK => clk_i,
      LOCK => PLL_LOCK);

  clk_i <= CLK25;
      
  i_pf : platform
    port map (
      clk_i => clk_i,
      reset_n => reset_n,
      gpioPIO_OUT => lm32_gpio,
      uartSIN => uartSIN, -- IDSEL_IN,
      uartSOUT => uartSOUT, -- USER(8),
      ddr2_sdramem_ddr_data  => ddr2_sdramem_ddr_data,
      ddr2_sdramem_ddr_dqs   => ddr2_sdramem_ddr_dqs,
      ddr2_sdramem_ddr_clk   => ddr2_sdramem_ddr_clk,
      ddr2_sdramem_ddr_cke   => ddr2_sdramem_ddr_cke,
      ddr2_sdramem_ddr_ras_n => ddr2_sdramem_ddr_ras_n,
      ddr2_sdramem_ddr_cas_n => ddr2_sdramem_ddr_cas_n,
      ddr2_sdramem_ddr_we_n  => ddr2_sdramem_ddr_we_n,
      ddr2_sdramem_ddr_cs_n  => ddr2_sdramem_ddr_cs_n,
      ddr2_sdramem_ddr_odt   => ddr2_sdramem_ddr_odt,
      ddr2_sdramem_ddr_dm    => ddr2_sdramem_ddr_dm,
      ddr2_sdramem_ddr_ba    => ddr2_sdramem_ddr_ba,
      ddr2_sdramem_ddr_addr  => ddr2_sdramem_ddr_addr,
      ts_mac_coregtx_clk     => open,
      ts_mac_corerxmac_clk   => open,
      ts_mac_coretxmac_clk   => open,
      ts_mac_coretx_clk      => TX_CLK,
      ts_mac_coremdc         => MDC,
      ts_mac_coresys_clk     => ts_coresys_clk,
      ts_mac_corerx_clk      => RX_CLK,
      ts_mac_corerxd(7 downto 4) => gnd_vec(3 downto 0),
      ts_mac_corerxd(3 downto 0) => RXD,
      ts_mac_corerx_dv       => RX_DV,
      ts_mac_corerx_er       => RX_ER,
      ts_mac_corecol         => COL,
      ts_mac_corecrs         => CRS,
      ts_mac_coretxd(7 downto 4) => open, 
      ts_mac_coretxd(3 downto 0) => TXD, 
      ts_mac_coretx_en       => TX_EN,
      ts_mac_coretx_er       => open,
--      ts_mac_coremdio        => MDIO
      spiMISO_MASTER         => spi_MISO,
      spiMOSI_MASTER         => spi_MOSI,
      spiSS_N_MASTER         => spi_SS_N,
      spiSCLK_MASTER         => spi_SCLK,
      slave_passthruclk  => slave_passthruclk,
      slave_passthrurst  => slave_passthrurst,
      slave_passthruslv_adr  => slave_passthruslv_adr,
      slave_passthruslv_master_data  => slave_passthruslv_master_data,
      slave_passthruslv_slave_data  => slave_passthruslv_slave_data,
      slave_passthruslv_strb  => slave_passthruslv_strb,
      slave_passthruslv_cyc  => slave_passthruslv_cyc,
      slave_passthruslv_ack  => slave_passthruslv_ack,
      slave_passthruslv_err  => slave_passthruslv_err,
      slave_passthruslv_rty  => slave_passthruslv_rty,
      slave_passthruslv_sel  => slave_passthruslv_sel,
      slave_passthruslv_we  => slave_passthruslv_we,
      slave_passthruslv_bte  => slave_passthruslv_bte,
      slave_passthruslv_cti  => slave_passthruslv_cti,
      slave_passthruslv_lock  => slave_passthruslv_lock,
      slave_passthruintr_active_high  => slave_passthruintr_active_high
      );

  i_opb_evr : opb_evr
    generic map (
      C_BASEADDR => EVR_LOCAL_BASE,
      C_BASEMASK => LOCAL_BASE_MASK)
    port map (
      OPB_Clk => clk,
      OPB_Rst => evr_OPB_Rst,
      Sl_DBus => evr_Sl_DBus,
      Sl_errAck => evr_Sl_errAck,
      Sl_retry => evr_Sl_retry,
      Sl_toutSup => evr_Sl_toutSup,
      Sl_xferAck => evr_Sl_xferAck,
      OPB_ABus => evr_OPB_ABus,
      OPB_BE => evr_OPB_BE,
      OPB_DBus => evr_OPB_DBus,
      OPB_RNW => evr_OPB_RNW,
      OPB_select => evr_OPB_select,
      OPB_seqAddr => evr_OPB_seqAddr,

      ip_irq_out => evr_irq,

      TTLIN => TTLIN,
      TTLOUT => open,      
      UNIVIN => UNIVIN,
      UNIVOUT => UNIVOUT,

      GPIO => GPIO,

      TBIN => TBIN,
      TBOUT => open,
      
      BREFCLKN => BREFCLKN,
      BREFCLKP => BREFCLKP,
      BREFCLK2N => BREFCLK2N,
      BREFCLK2P => BREFCLK2P,

      refclk_out => open,
      brefclk_out => open,
      refclk2_out => open,
      brefclk2_out => open,
      
      RECCLKN => RECCLKN,
      RECCLKP => RECCLKP,

      LED_fail => open,
      LED_rx_link => open,
      LED_ena => open,
      LED_event_in => open,
      LED_event_out => open,
      LED_rx_fail => open,
      LED_error => open,
      
      EVRXN => EVRXN,
      EVRXP => EVRXP,
      EVTXN => EVTXN,
      EVTXP => EVTXP,
      FBRXN => FBRXN,
      FBRXP => FBRXP,
      FBTXN => FBTXN,
      FBTXP => FBTXP,
      CMLRXN => CMLRXN,
      CMLRXP => CMLRXP,
      CMLTXN => CMLTXN,
      CMLTXP => CMLTXP,

      CGSK => PROGSK,
      CGDI => PROGDI,
      CGCS => PROGCS,
      CGLOCK => LOCKED,

      TXLOOPBACK => TXLOOPBACK,
      RXLOOPBACK => RXLOOPBACK,

      debug => open,
      refclkp => refclkp,
      refclkn => refclkn,
      hdinp0 => hdinp0,
      hdinn0 => hdinn0,
      hdoutp0 => hdoutp0,
      hdoutn0 => hdoutn0,
      hdinp1 => hdinp1,
      hdinn1 => hdinn1,
      hdoutp1 => hdoutp1,
      hdoutn1 => hdoutn1,
      hdinp2 => hdinp2,
      hdinn2 => hdinn2,
      hdoutp2 => hdoutp2,
      hdoutn2 => hdoutn2,
      hdinp3 => hdinp3,
      hdinn3 => hdinn3,
      hdoutp3 => hdoutp3,
      hdoutn3 => hdoutn3
      );

  gnd <= '0';
  vcc <= '1';
  gnd_vec <= (others => '0');
  
  PWR_DOWN <= '1';
--  PHYRESETn <= reset_n;

  SYNCVL <= GND;
  SYNC1V2 <= GND;
  SYNC1V8 <= GND;

  USER(3 downto 0) <= UNIVOUT;
  -- IDSEL_PU controls the pull-up resistor on the ID_Select* pin of
  -- the cRIO backplane connector.
  -- When IDSEL_PU is low a cRIO backplane does not see the module
  -- in the slot.
  IDSEL_pullup <= lm32_gpio(0);
  IDSEL_PU <= IDSEL_pullup;

  MOD_SCL <= '0';
  MOD_TXD <= '0';

  reset_n <= PLL_LOCK;

  process (CLK25, PLL_LOCK)
    variable por : unsigned(15 downto 0) := X"7FFF";
  begin
    if rising_edge(CLK25) then
      PHYRESETn <= por(por'high);
      if por(por'high) = '0' then
        por := por - 1;
      end if;
      if PLL_LOCK = '0' then
        por := X"7FFF";
      end if;
    end if;
  end process;

  clk <= slave_passthruclk;
  evr_OPB_Rst <= slave_passthrurst;
  evr_OPB_ABus <= slave_passthruslv_adr;
  evr_OPB_BE <= slave_passthruslv_sel;
  evr_OPB_DBus <= slave_passthruslv_master_data;
  evr_OPB_RNW <= not slave_passthruslv_we;
  evr_OPB_select <= slave_passthruslv_strb;
  evr_OPB_seqAddr <= '0';
    
  slave_passthruslv_slave_data <= evr_Sl_DBus;
  slave_passthruslv_ack <= evr_Sl_xferAck;
  slave_passthruslv_err <= evr_Sl_errAck;
  slave_passthruslv_rty <= evr_Sl_retry;
  slave_passthruintr_active_high <= evr_irq;

  -- Uart is available when IDSEL_pullup = '0' or
  -- in cRIO IDSEL mode with SPI_FUNC = '0'
  process (IDSEL_pullup, IDSEL_IN, USER)
  begin
    USER(8) <= 'Z';
    uartSIN <= '1';
    if IDSEL_pullup = '0' then
      USER(8) <= uartSOUT;
      uartSIN <= IDSEL_IN;
    elsif IDSEL_IN = '0' and USER(4) = '0' then
      uartSIN <= USER(7);
    end if;
  end process;

  -- SPI signals
  process (IDSEL_pullup, IDSEL_IN)
  begin
    USER(7 downto 4) <= "ZZZZ";
    EECS <= spi_SS_N(2);
    EESK <= spi_SCLK;
    EEDI <= spi_MOSI;
    UNIVIN <= (others => '0');
    spi_MISO <= EEDO and FLASHQ(1) and FLASHQ(0);
    FLASHD <= spi_MOSI & spi_MOSI;
    FLASHC <= spi_SCLK;
    CCLK <= spi_SCLK;
    FLASHS_N <= spi_SS_N(1 downto 0);
    if IDSEL_pullup = '1' and IDSEL_IN = '0' then
      -- ID mode, check if EEPROM or smart sensor/microLAN interface
      if USER(4) = '1' then
	-- EEPROM
	USER(6) <= EEDO;
	EECS <= USER(5);
	EESK <= USER(8);
	EEDI <= USER(7);
	spi_MISO <= FLASHQ(1) and FLASHQ(0);
      else
	-- smart sensor/microLAN interface
	USER(6) <= uartSOUT;
      end if;
    else
      UNIVIN <= USER(7 downto 4);
    end if;
  end process;
  
end structure;
