library IEEE;
use IEEE.std_logic_1164.all;
-- synopsys translate_off
library ecp2m;
use ecp2m.components.all;
-- synopsys translate_on

entity membuf_bsram is
    port (
      addra: IN std_logic_VECTOR(10 downto 0);
      addrb: IN std_logic_VECTOR(8 downto 0);
      clka: IN std_logic;
      clkb: IN std_logic;
      douta: OUT std_logic_VECTOR(7 downto 0);
      dinb: IN std_logic_VECTOR(31 downto 0);
      doutb: OUT std_logic_VECTOR(31 downto 0);
      web: IN std_logic);
end membuf_bsram;

architecture Structure of membuf_bsram is

  signal DataInA: std_logic_vector(7 downto 0); 
  signal DataInB: std_logic_vector(31 downto 0); 
  signal AddressA: std_logic_vector(10 downto 0); 
  signal AddressB: std_logic_vector(8 downto 0); 
  signal ClockA: std_logic; 
  signal ClockB: std_logic; 
  signal ClockEnA: std_logic; 
  signal ClockEnB: std_logic; 
  signal WrA: std_logic; 
  signal WrB: std_logic; 
  signal ResetA: std_logic; 
  signal ResetB: std_logic; 
  signal QA: std_logic_vector(7 downto 0); 
  signal QB: std_logic_vector(31 downto 0));

    -- internal signal declarations
  signal scuba_vhi: std_logic;
  signal scuba_vlo: std_logic;

    -- local component declarations
    component VHI
        port (Z: out  std_logic);
    end component;
    component VLO
        port (Z: out  std_logic);
    end component;
    component DP16KB
    -- synopsys translate_off
        generic (GSR : in String; WRITEMODE_B : in String; 
                CSDECODE_B : in std_logic_vector(2 downto 0); 
                CSDECODE_A : in std_logic_vector(2 downto 0); 
                WRITEMODE_A : in String; RESETMODE : in String; 
                REGMODE_B : in String; REGMODE_A : in String; 
                DATA_WIDTH_B : in Integer; DATA_WIDTH_A : in Integer);
    -- synopsys translate_on
        port (DIA0: in  std_logic; DIA1: in  std_logic; 
            DIA2: in  std_logic; DIA3: in  std_logic; 
            DIA4: in  std_logic; DIA5: in  std_logic; 
            DIA6: in  std_logic; DIA7: in  std_logic; 
            DIA8: in  std_logic; DIA9: in  std_logic; 
            DIA10: in  std_logic; DIA11: in  std_logic; 
            DIA12: in  std_logic; DIA13: in  std_logic; 
            DIA14: in  std_logic; DIA15: in  std_logic; 
            DIA16: in  std_logic; DIA17: in  std_logic; 
            ADA0: in  std_logic; ADA1: in  std_logic; 
            ADA2: in  std_logic; ADA3: in  std_logic; 
            ADA4: in  std_logic; ADA5: in  std_logic; 
            ADA6: in  std_logic; ADA7: in  std_logic; 
            ADA8: in  std_logic; ADA9: in  std_logic; 
            ADA10: in  std_logic; ADA11: in  std_logic; 
            ADA12: in  std_logic; ADA13: in  std_logic; 
            CEA: in  std_logic; CLKA: in  std_logic; WEA: in  std_logic; 
            CSA0: in  std_logic; CSA1: in  std_logic; 
            CSA2: in  std_logic; RSTA: in  std_logic; 
            DIB0: in  std_logic; DIB1: in  std_logic; 
            DIB2: in  std_logic; DIB3: in  std_logic; 
            DIB4: in  std_logic; DIB5: in  std_logic; 
            DIB6: in  std_logic; DIB7: in  std_logic; 
            DIB8: in  std_logic; DIB9: in  std_logic; 
            DIB10: in  std_logic; DIB11: in  std_logic; 
            DIB12: in  std_logic; DIB13: in  std_logic; 
            DIB14: in  std_logic; DIB15: in  std_logic; 
            DIB16: in  std_logic; DIB17: in  std_logic; 
            ADB0: in  std_logic; ADB1: in  std_logic; 
            ADB2: in  std_logic; ADB3: in  std_logic; 
            ADB4: in  std_logic; ADB5: in  std_logic; 
            ADB6: in  std_logic; ADB7: in  std_logic; 
            ADB8: in  std_logic; ADB9: in  std_logic; 
            ADB10: in  std_logic; ADB11: in  std_logic; 
            ADB12: in  std_logic; ADB13: in  std_logic; 
            CEB: in  std_logic; CLKB: in  std_logic; WEB: in  std_logic; 
            CSB0: in  std_logic; CSB1: in  std_logic; 
            CSB2: in  std_logic; RSTB: in  std_logic; 
            DOA0: out  std_logic; DOA1: out  std_logic; 
            DOA2: out  std_logic; DOA3: out  std_logic; 
            DOA4: out  std_logic; DOA5: out  std_logic; 
            DOA6: out  std_logic; DOA7: out  std_logic; 
            DOA8: out  std_logic; DOA9: out  std_logic; 
            DOA10: out  std_logic; DOA11: out  std_logic; 
            DOA12: out  std_logic; DOA13: out  std_logic; 
            DOA14: out  std_logic; DOA15: out  std_logic; 
            DOA16: out  std_logic; DOA17: out  std_logic; 
            DOB0: out  std_logic; DOB1: out  std_logic; 
            DOB2: out  std_logic; DOB3: out  std_logic; 
            DOB4: out  std_logic; DOB5: out  std_logic; 
            DOB6: out  std_logic; DOB7: out  std_logic; 
            DOB8: out  std_logic; DOB9: out  std_logic; 
            DOB10: out  std_logic; DOB11: out  std_logic; 
            DOB12: out  std_logic; DOB13: out  std_logic; 
            DOB14: out  std_logic; DOB15: out  std_logic; 
            DOB16: out  std_logic; DOB17: out  std_logic);
    end component;
    attribute MEM_LPC_FILE : string; 
    attribute MEM_INIT_FILE : string; 
    attribute CSDECODE_B : string; 
    attribute CSDECODE_A : string; 
    attribute WRITEMODE_B : string; 
    attribute WRITEMODE_A : string; 
    attribute GSR : string; 
    attribute RESETMODE : string; 
    attribute REGMODE_B : string; 
    attribute REGMODE_A : string; 
    attribute DATA_WIDTH_B : string; 
    attribute DATA_WIDTH_A : string; 
    attribute MEM_LPC_FILE of membuf_0_0_1 : label is "membuf.lpc";
    attribute MEM_INIT_FILE of membuf_0_0_1 : label is "";
    attribute CSDECODE_B of membuf_0_0_1 : label is "0b000";
    attribute CSDECODE_A of membuf_0_0_1 : label is "0b000";
    attribute WRITEMODE_B of membuf_0_0_1 : label is "NORMAL";
    attribute WRITEMODE_A of membuf_0_0_1 : label is "NORMAL";
    attribute GSR of membuf_0_0_1 : label is "DISABLED";
    attribute RESETMODE of membuf_0_0_1 : label is "SYNC";
    attribute REGMODE_B of membuf_0_0_1 : label is "NOREG";
    attribute REGMODE_A of membuf_0_0_1 : label is "NOREG";
    attribute DATA_WIDTH_B of membuf_0_0_1 : label is "18";
    attribute DATA_WIDTH_A of membuf_0_0_1 : label is "4";
    attribute MEM_LPC_FILE of membuf_0_1_0 : label is "membuf.lpc";
    attribute MEM_INIT_FILE of membuf_0_1_0 : label is "";
    attribute CSDECODE_B of membuf_0_1_0 : label is "0b000";
    attribute CSDECODE_A of membuf_0_1_0 : label is "0b000";
    attribute WRITEMODE_B of membuf_0_1_0 : label is "NORMAL";
    attribute WRITEMODE_A of membuf_0_1_0 : label is "NORMAL";
    attribute GSR of membuf_0_1_0 : label is "DISABLED";
    attribute RESETMODE of membuf_0_1_0 : label is "SYNC";
    attribute REGMODE_B of membuf_0_1_0 : label is "NOREG";
    attribute REGMODE_A of membuf_0_1_0 : label is "NOREG";
    attribute DATA_WIDTH_B of membuf_0_1_0 : label is "18";
    attribute DATA_WIDTH_A of membuf_0_1_0 : label is "4";

begin
    -- component instantiation statements
    membuf_0_0_1: DP16KB
        -- synopsys translate_off
        generic map (CSDECODE_B=> "000", CSDECODE_A=> "000", WRITEMODE_B=> "NORMAL", 
        WRITEMODE_A=> "NORMAL", GSR=> "DISABLED", RESETMODE=> "SYNC", 
        REGMODE_B=> "NOREG", REGMODE_A=> "NOREG", DATA_WIDTH_B=>  18, 
        DATA_WIDTH_A=>  4)
        -- synopsys translate_on
        port map (DIA0=>DataInA(0), DIA1=>DataInA(1), DIA2=>DataInA(2), 
            DIA3=>DataInA(3), DIA4=>scuba_vlo, DIA5=>scuba_vlo, 
            DIA6=>scuba_vlo, DIA7=>scuba_vlo, DIA8=>scuba_vlo, 
            DIA9=>scuba_vlo, DIA10=>scuba_vlo, DIA11=>scuba_vlo, 
            DIA12=>scuba_vlo, DIA13=>scuba_vlo, DIA14=>scuba_vlo, 
            DIA15=>scuba_vlo, DIA16=>scuba_vlo, DIA17=>scuba_vlo, 
            ADA0=>scuba_vlo, ADA1=>scuba_vlo, ADA2=>AddressA(0), 
            ADA3=>AddressA(1), ADA4=>AddressA(2), ADA5=>AddressA(3), 
            ADA6=>AddressA(4), ADA7=>AddressA(5), ADA8=>AddressA(6), 
            ADA9=>AddressA(7), ADA10=>AddressA(8), ADA11=>AddressA(9), 
            ADA12=>AddressA(10), ADA13=>scuba_vlo, CEA=>ClockEnA, 
            CLKA=>ClockA, WEA=>WrA, CSA0=>scuba_vlo, CSA1=>scuba_vlo, 
            CSA2=>scuba_vlo, RSTA=>ResetA, DIB0=>DataInB(0), 
            DIB1=>DataInB(1), DIB2=>DataInB(2), DIB3=>DataInB(3), 
            DIB4=>DataInB(8), DIB5=>DataInB(9), DIB6=>DataInB(10), 
            DIB7=>DataInB(11), DIB8=>scuba_vlo, DIB9=>DataInB(16), 
            DIB10=>DataInB(17), DIB11=>DataInB(18), DIB12=>DataInB(19), 
            DIB13=>DataInB(24), DIB14=>DataInB(25), DIB15=>DataInB(26), 
            DIB16=>DataInB(27), DIB17=>scuba_vlo, ADB0=>scuba_vhi, 
            ADB1=>scuba_vhi, ADB2=>scuba_vlo, ADB3=>scuba_vlo, 
            ADB4=>AddressB(0), ADB5=>AddressB(1), ADB6=>AddressB(2), 
            ADB7=>AddressB(3), ADB8=>AddressB(4), ADB9=>AddressB(5), 
            ADB10=>AddressB(6), ADB11=>AddressB(7), ADB12=>AddressB(8), 
            ADB13=>scuba_vlo, CEB=>ClockEnB, CLKB=>ClockB, WEB=>WrB, 
            CSB0=>scuba_vlo, CSB1=>scuba_vlo, CSB2=>scuba_vlo, 
            RSTB=>ResetB, DOA0=>QA(0), DOA1=>QA(1), DOA2=>QA(2), 
            DOA3=>QA(3), DOA4=>open, DOA5=>open, DOA6=>open, DOA7=>open, 
            DOA8=>open, DOA9=>open, DOA10=>open, DOA11=>open, 
            DOA12=>open, DOA13=>open, DOA14=>open, DOA15=>open, 
            DOA16=>open, DOA17=>open, DOB0=>QB(0), DOB1=>QB(1), 
            DOB2=>QB(2), DOB3=>QB(3), DOB4=>QB(8), DOB5=>QB(9), 
            DOB6=>QB(10), DOB7=>QB(11), DOB8=>open, DOB9=>QB(16), 
            DOB10=>QB(17), DOB11=>QB(18), DOB12=>QB(19), DOB13=>QB(24), 
            DOB14=>QB(25), DOB15=>QB(26), DOB16=>QB(27), DOB17=>open);

    scuba_vhi_inst: VHI
        port map (Z=>scuba_vhi);

    scuba_vlo_inst: VLO
        port map (Z=>scuba_vlo);

    membuf_0_1_0: DP16KB
        -- synopsys translate_off
        generic map (CSDECODE_B=> "000", CSDECODE_A=> "000", WRITEMODE_B=> "NORMAL", 
        WRITEMODE_A=> "NORMAL", GSR=> "DISABLED", RESETMODE=> "SYNC", 
        REGMODE_B=> "NOREG", REGMODE_A=> "NOREG", DATA_WIDTH_B=>  18, 
        DATA_WIDTH_A=>  4)
        -- synopsys translate_on
        port map (DIA0=>DataInA(4), DIA1=>DataInA(5), DIA2=>DataInA(6), 
            DIA3=>DataInA(7), DIA4=>scuba_vlo, DIA5=>scuba_vlo, 
            DIA6=>scuba_vlo, DIA7=>scuba_vlo, DIA8=>scuba_vlo, 
            DIA9=>scuba_vlo, DIA10=>scuba_vlo, DIA11=>scuba_vlo, 
            DIA12=>scuba_vlo, DIA13=>scuba_vlo, DIA14=>scuba_vlo, 
            DIA15=>scuba_vlo, DIA16=>scuba_vlo, DIA17=>scuba_vlo, 
            ADA0=>scuba_vlo, ADA1=>scuba_vlo, ADA2=>AddressA(0), 
            ADA3=>AddressA(1), ADA4=>AddressA(2), ADA5=>AddressA(3), 
            ADA6=>AddressA(4), ADA7=>AddressA(5), ADA8=>AddressA(6), 
            ADA9=>AddressA(7), ADA10=>AddressA(8), ADA11=>AddressA(9), 
            ADA12=>AddressA(10), ADA13=>scuba_vlo, CEA=>ClockEnA, 
            CLKA=>ClockA, WEA=>WrA, CSA0=>scuba_vlo, CSA1=>scuba_vlo, 
            CSA2=>scuba_vlo, RSTA=>ResetA, DIB0=>DataInB(4), 
            DIB1=>DataInB(5), DIB2=>DataInB(6), DIB3=>DataInB(7), 
            DIB4=>DataInB(12), DIB5=>DataInB(13), DIB6=>DataInB(14), 
            DIB7=>DataInB(15), DIB8=>scuba_vlo, DIB9=>DataInB(20), 
            DIB10=>DataInB(21), DIB11=>DataInB(22), DIB12=>DataInB(23), 
            DIB13=>DataInB(28), DIB14=>DataInB(29), DIB15=>DataInB(30), 
            DIB16=>DataInB(31), DIB17=>scuba_vlo, ADB0=>scuba_vhi, 
            ADB1=>scuba_vhi, ADB2=>scuba_vlo, ADB3=>scuba_vlo, 
            ADB4=>AddressB(0), ADB5=>AddressB(1), ADB6=>AddressB(2), 
            ADB7=>AddressB(3), ADB8=>AddressB(4), ADB9=>AddressB(5), 
            ADB10=>AddressB(6), ADB11=>AddressB(7), ADB12=>AddressB(8), 
            ADB13=>scuba_vlo, CEB=>ClockEnB, CLKB=>ClockB, WEB=>WrB, 
            CSB0=>scuba_vlo, CSB1=>scuba_vlo, CSB2=>scuba_vlo, 
            RSTB=>ResetB, DOA0=>QA(4), DOA1=>QA(5), DOA2=>QA(6), 
            DOA3=>QA(7), DOA4=>open, DOA5=>open, DOA6=>open, DOA7=>open, 
            DOA8=>open, DOA9=>open, DOA10=>open, DOA11=>open, 
            DOA12=>open, DOA13=>open, DOA14=>open, DOA15=>open, 
            DOA16=>open, DOA17=>open, DOB0=>QB(4), DOB1=>QB(5), 
            DOB2=>QB(6), DOB3=>QB(7), DOB4=>QB(12), DOB5=>QB(13), 
            DOB6=>QB(14), DOB7=>QB(15), DOB8=>open, DOB9=>QB(20), 
            DOB10=>QB(21), DOB11=>QB(22), DOB12=>QB(23), DOB13=>QB(28), 
            DOB14=>QB(29), DOB15=>QB(30), DOB16=>QB(31), DOB17=>open);

    DataInA <= (others => '0');
    DataInB <= dinb;
    AddressA <= addra;
    AddressB <= addrb;
    ClockA <= clka;
    ClockB <= clkb;
    ClockEnA <= '1';
    ClockEnB <= '1';
    WrA <= '0';
    WrB <= web;
    ResetA <= '0';
    ResetB <= '0';
    douta <= QA;
    doutb <= QB;
    
end Structure;

-- synopsys translate_off
library ecp2m;
configuration Structure_CON of membuf is
    for Structure
        for all:VHI use entity ecp2m.VHI(V); end for;
        for all:VLO use entity ecp2m.VLO(V); end for;
        for all:DP16KB use entity ecp2m.DP16KB(V); end for;
    end for;
end Structure_CON;

-- synopsys translate_on
