---------------------------------------------------------------------------
--
--  File        : cpcievrtg.vhd
--
--  Title       : cPCI EVRTG Top Level
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_bs_pkg.all;
use work.evr_pkg.all;
library UNISIM;
use UNISIM.Vcomponents.ALL;

entity cpcievrtg is
  port (
    -- ADN2812ACPZ signals
    ADNLOL : in std_logic;
    ADNLOS : in std_logic;
    ADNSCK : out std_logic;
    ADNSDA : inout std_logic;

    -- SY89230 signals
    DIV_SEL   : out std_logic; -- 3/5 divider select
    EN        : out std_logic; -- divider enable
    nMR       : out std_logic; -- divider reset
    RXREFCLKN : in std_logic;
    RXREFCLKP : in std_logic;
    
    -- SY87739 Fractional Synthesizer signals
    LOCKED  : in std_logic;
    PROGCS  : out std_logic;
    PROGDI  : out std_logic;
    PROGSK  : out std_logic;
    REFCLKN : in std_logic;
    REFCLKP : in std_logic;
    
    -- PCI9056 local bus signals
    LCLK : in std_logic; -- 66 MHz local clock
    BBn : in std_logic;
    BDIPn : in std_logic;
    BGn : in std_logic;
    BRn : in std_logic;
    BURSTn : in std_logic;
    CCSn : in std_logic;
    DACKn : in std_logic_vector(1 downto 0);
    DREQn : in std_logic_vector(1 downto 0);
    DP : in std_logic_vector(3 downto 0);
    LA : inout std_logic_vector(31 downto 0);
    LD : inout std_logic_vector(31 downto 0);
    LINTIn : in std_logic;
    LINTOn : in std_logic;
    MDREQn_DMPAF_EOTn : in std_logic;
    PMEREQn : in std_logic;
    RD_WRn : in std_logic;
    RETRYn : in std_logic;
    TAn : in std_logic;
    TEAn : in std_logic;
    TSn : in std_logic;
    TSIZ : in std_logic_vector(1 downto 0);
    nLRESET : in std_logic;
    nSYSEN : in std_logic;
    nWAIT : inout std_logic;
    nBI : out std_logic;
    
    -- IP2022 interface signals
    nUACK : out std_logic;
    nUBERR : out std_logic;
    nURD : in std_logic;
    nUWR : in std_logic;
    UA : in std_logic_vector(16 downto 0);
    UD : inout std_logic_vector(7 downto 0);
    ULE : in std_logic;
    
    -- DDR2 memory interface signals
    DDRAA : out std_logic_vector(12 downto 0);
    DDRABA : out std_logic_vector(1 downto 0);
    DDRACASn : out std_logic;
    DDRARASn : out std_logic;
    DDRAWEn : out std_logic;
    DDRACK : out std_logic;
    DDRACKn : out std_logic;
    DDRACKE : out std_logic;
    DDRACSn : out std_logic;
--    DDRADQ : inout std_logic_vector(15 downto 0);
    DDRADQ : out std_logic_vector(15 downto 0);
    DDRALDM : out std_logic;
    DDRAUDM : out std_logic;
    DDRALDQS : inout std_logic;
    DDRALDQSn : inout std_logic;
    DDRAUDQS : inout std_logic;
    DDRAUDQSn : inout std_logic;
    DDRAODT : out std_logic;
    
    DDRBA : out std_logic_vector(12 downto 0);
    DDRBBA : out std_logic_vector(1 downto 0);
    DDRBCASn : out std_logic;
    DDRBRASn : out std_logic;
    DDRBWEn : out std_logic;
    DDRBCK : out std_logic;
    DDRBCKn : out std_logic;
    DDRBCKE : out std_logic;
    DDRBCSn : out std_logic;
--    DDRBDQ : inout std_logic_vector(15 downto 0);
    DDRBDQ : out std_logic_vector(15 downto 0);
    DDRBLDM : out std_logic;
    DDRBUDM : out std_logic;
    DDRBLDQS : inout std_logic;
    DDRBLDQSn : inout std_logic;
    DDRBUDQS : inout std_logic;
    DDRBUDQSn : inout std_logic;
    DDRBODT : out std_logic;

    -- GigE PHY signals
    PHY_CLK_TO_MAC : in std_logic;
    PHY_COL : in std_logic;
    PHY_CRS : in std_logic;
    PHY_GTXCLK : out std_logic;
    PHY_MDC : out std_logic;
    PHY_MDIO : inout std_logic;
    PHY_RXD : in std_logic_vector(7 downto 0);
    PHY_RX_CLK : in std_logic;
    PHY_RX_DV : in std_logic;
    PHY_RX_ER : in std_logic;
    PHY_TXD : out std_logic_vector(7 downto 0);
    PHY_TX_CLK : in std_logic;
    PHY_TX_EN : out std_logic;
    PHY_TX_ER : out std_logic;
    nPHY_IRQ : in std_logic;
    
    -- LED controller signals
    LEDBLANK : out std_logic;
    LEDMODE  : out std_logic;
    LEDSCLK  : out std_logic;
    LEDSIN   : out std_logic;
    LEDSOUT  : in std_logic;
    LEDXERR  : in std_logic;
    LEDXLAT  : out std_logic;

    -- Serial Port
    RXD : in std_logic;
    TXD : out std_logic;

    -- Power supply synchronization signals
    SYNC_1V0 : out std_logic;
    SYNC_1V8 : out std_logic;
    
    -- Transceiver control signals
    MOD_TXD  : out std_logic_vector(2 downto 0); -- Transmitter disable
    nMOD_SEL : out std_logic_vector(2 downto 0);
-- The following signals are not connected!
--    MOD_LOS : in std_logic;
--    MOD_TXF : in std_logic;
--    MOD_SDA : inout std_logic;
--    MOD_SCL : out std_logic;

    -- Delay chip control signals
    LEN      : out std_logic_vector(8 downto 1); -- latch when '1'
    DLY      : out std_logic_vector(9 downto 0);
    OUTEN    : out std_logic;

    UNIVIN   : in std_logic_vector(3 downto 0);
    
    GPIO     : inout std_logic_vector(7 downto 0);

    CLK_OUT  : out std_logic;
    
    RD1N : in std_logic;
    RD1P : in std_logic;
    RD2N : in std_logic;
    RD2P : in std_logic;
    RD3N : in std_logic;
    RD3P : in std_logic;
    RD4N : in std_logic;
    RD4P : in std_logic;
    RD5N : in std_logic;
    RD5P : in std_logic;
    RD6N : in std_logic;
    RD6P : in std_logic;
    RD7N : in std_logic;
    RD7P : in std_logic;
    RD8N : in std_logic;
    RD8P : in std_logic;
    RDUPN : in std_logic;
    RDUPP : in std_logic;
    RDRESN : in std_logic;
    RDRESP : in std_logic;

    TD1N : out std_logic;
    TD1P : out std_logic;
    TD2N : out std_logic;
    TD2P : out std_logic;
    TD3N : out std_logic;
    TD3P : out std_logic;
    TD4N : out std_logic;
    TD4P : out std_logic;
    TD5N : out std_logic;
    TD5P : out std_logic;
    TD6N : out std_logic;
    TD6P : out std_logic;
    TD7N : out std_logic;
    TD7P : out std_logic;
    TD8N : out std_logic;
    TD8P : out std_logic;
    TDUPN : out std_logic;
    TDUPP : out std_logic;
    TDRESN : out std_logic;
    TDRESP : out std_logic);
end entity cpcievrtg;

architecture struct of cpcievrtg is

  signal gnd : std_logic;
  signal vcc : std_logic;
  
  signal rclk : std_logic;
  signal clk : std_logic;
  signal reset : std_logic;
  signal usec_ena : std_logic;

  signal event_clk : std_logic;
  
  signal evr_LED_fail : std_logic;
  signal evr_LED_rx_link : std_logic;
  signal evr_LED_ena : std_logic;
  signal evr_LED_event_in : std_logic;
  signal evr_LED_event_out : std_logic;
  signal evr_LED_rx_fail : std_logic;
  signal evr_LED_error : std_logic;
    
  signal led_on : std_logic_vector(63 downto 0);
  -- LED colors in vector are RGB
  signal led_link_rx : std_logic_vector(2 downto 0) := "000";
  signal led_link_tx : std_logic_vector(2 downto 0) := "000";
  signal led_ch2_rx  : std_logic_vector(2 downto 0) := "000";
  signal led_ch2_tx  : std_logic_vector(2 downto 0) := "000";
  signal led_ch1_rx  : std_logic_vector(2 downto 0) := "000";
  signal led_ch1_tx  : std_logic_vector(2 downto 0) := "000";
  signal led_d       : std_logic_vector(2 downto 0) := "000";
  signal led_c       : std_logic_vector(2 downto 0) := "000";
  signal led_b       : std_logic_vector(2 downto 0) := "000";
  signal led_a       : std_logic_vector(2 downto 0) := "000";
  signal led_com_tx  : std_logic_vector(2 downto 0) := "000";
  signal led_com_rx  : std_logic_vector(2 downto 0) := "000";
  signal TXD_int : std_logic;
  
  signal recclk : std_logic_vector(8 downto 0);

  signal rx_link_ok : std_logic_vector(8 downto 0);
  signal rx_violation : std_logic_vector(8 downto 0);
  signal rx_event_led : std_logic_vector(9 downto 0);
  signal tx_event_led : std_logic;

  signal CMLRXN : std_logic_vector(0 to 7);
  signal CMLRXP : std_logic_vector(0 to 7);
  signal CMLTXN : std_logic_vector(0 to 7);
  signal CMLTXP : std_logic_vector(0 to 7);

  signal plbv46_ubicom_PLB_MWrBTerm :  std_logic;
  signal plbv46_ubicom_PLB_MWrDAck :  std_logic;
  signal plbv46_ubicom_PLB_MRdBTerm :  std_logic;
  signal plbv46_ubicom_PLB_MRdDAck :  std_logic;
  signal plbv46_ubicom_PLB_MRdWdAddr :  std_logic_vector(0 to 3);
  signal plbv46_ubicom_PLB_MRdDBus :  std_logic_vector(0 to 127);
  signal plbv46_ubicom_PLB_MIRQ :  std_logic;
  signal plbv46_ubicom_PLB_MWrErr :  std_logic;
  signal plbv46_ubicom_PLB_MRdErr :  std_logic;
  signal plbv46_ubicom_PLB_MBusy :  std_logic;
  signal plbv46_ubicom_PLB_MTimeout :  std_logic;
  signal plbv46_ubicom_PLB_MRearbitrate :  std_logic;
  signal plbv46_ubicom_PLB_MSSize :  std_logic_vector(0 to 1);
  signal plbv46_ubicom_PLB_MAddrAck :  std_logic;
  signal plbv46_ubicom_M_rdBurst : std_logic;
  signal plbv46_ubicom_M_wrBurst : std_logic;
  signal plbv46_ubicom_M_wrDBus : std_logic_vector(0 to 127);
  signal plbv46_ubicom_M_ABus : std_logic_vector(0 to 31);
  signal plbv46_ubicom_M_UABus : std_logic_vector(0 to 31);
  signal plbv46_ubicom_M_abort : std_logic;
  signal plbv46_ubicom_M_lockErr : std_logic;
  signal plbv46_ubicom_M_TAttribute : std_logic_vector(0 to 15);
  signal plbv46_ubicom_M_type : std_logic_vector(0 to 2);
  signal plbv46_ubicom_M_size : std_logic_vector(0 to 3);
  signal plbv46_ubicom_M_MSize : std_logic_vector(0 to 1);
  signal plbv46_ubicom_M_BE : std_logic_vector(0 to 15);
  signal plbv46_ubicom_M_RNW : std_logic;
  signal plbv46_ubicom_M_busLock : std_logic;
  signal plbv46_ubicom_M_priority : std_logic_vector(0 to 1);
  signal plbv46_ubicom_M_request : std_logic;
  signal plbv46_ubicom_MPLB_Rst :  std_logic;
  signal plbv46_ubicom_MPLB_Clk :  std_logic;

--  signal plbv46_pci9056_PLB_MWrBTerm :  std_logic;
--  signal plbv46_pci9056_PLB_MWrDAck :  std_logic;
--  signal plbv46_pci9056_PLB_MRdBTerm :  std_logic;
--  signal plbv46_pci9056_PLB_MRdDAck :  std_logic;
--  signal plbv46_pci9056_PLB_MRdWdAddr :  std_logic_vector(0 to 3);
--  signal plbv46_pci9056_PLB_MRdDBus :  std_logic_vector(0 to 127);
--  signal plbv46_pci9056_PLB_MIRQ :  std_logic;
--  signal plbv46_pci9056_PLB_MWrErr :  std_logic;
--  signal plbv46_pci9056_PLB_MRdErr :  std_logic;
--  signal plbv46_pci9056_PLB_MBusy :  std_logic;
--  signal plbv46_pci9056_PLB_MTimeout :  std_logic;
--  signal plbv46_pci9056_PLB_MRearbitrate :  std_logic;
--  signal plbv46_pci9056_PLB_MSSize :  std_logic_vector(0 to 1);
--  signal plbv46_pci9056_PLB_MAddrAck :  std_logic;
--  signal plbv46_pci9056_M_rdBurst : std_logic;
--  signal plbv46_pci9056_M_wrBurst : std_logic;
--  signal plbv46_pci9056_M_wrDBus : std_logic_vector(0 to 127);
--  signal plbv46_pci9056_M_ABus : std_logic_vector(0 to 31);
--  signal plbv46_pci9056_M_UABus : std_logic_vector(0 to 31);
--  signal plbv46_pci9056_M_abort : std_logic;
--  signal plbv46_pci9056_M_lockErr : std_logic;
--  signal plbv46_pci9056_M_TAttribute : std_logic_vector(0 to 15);
--  signal plbv46_pci9056_M_type : std_logic_vector(0 to 2);
--  signal plbv46_pci9056_M_size : std_logic_vector(0 to 3);
--  signal plbv46_pci9056_M_MSize : std_logic_vector(0 to 1);
--  signal plbv46_pci9056_M_BE : std_logic_vector(0 to 15);
--  signal plbv46_pci9056_M_RNW : std_logic;
--  signal plbv46_pci9056_M_busLock : std_logic;
--  signal plbv46_pci9056_M_priority : std_logic_vector(0 to 1);
--  signal plbv46_pci9056_M_request : std_logic;
--  signal plbv46_pci9056_MPLB_Rst :  std_logic;
--  signal plbv46_pci9056_MPLB_Clk :  std_logic;

  signal plbv46_test_IP2Bus_Error : std_logic;
  signal plbv46_test_IP2Bus_WrAck: std_logic;
  signal plbv46_test_IP2Bus_RdAck : std_logic;
  signal plbv46_test_IP2Bus_Data : std_logic_vector(0 to 31);
  signal plbv46_test_Bus2IP_BE : std_logic_vector(0 to 3);
  signal plbv46_test_Bus2IP_Data : std_logic_vector(0 to 31);
  signal plbv46_test_Bus2IP_RNW : std_logic;
  signal plbv46_test_Bus2IP_CS : std_logic_vector(0 to 0);
  signal plbv46_test_Bus2IP_Addr : std_logic_vector(0 to 31);
  signal plbv46_test_Bus2IP_Reset : std_logic;
  signal plbv46_test_Bus2IP_Clk : std_logic;
  
  signal evr_OPB_Clk : std_logic;
  signal evr_OPB_Rst : std_logic;
  signal evr_Sl_DBus : std_logic_vector(0 to 31);
  signal evr_Sl_errAck : std_logic;
  signal evr_Sl_retry : std_logic;
  signal evr_Sl_toutSup : std_logic;
  signal evr_Sl_xferAck : std_logic;
  signal evr_OPB_ABus : std_logic_vector(0 to 31);
  signal evr_OPB_BE : std_logic_vector(0 to 3);
  signal evr_OPB_DBus : std_logic_vector(0 to 31);
  signal evr_OPB_RNW : std_logic;
  signal evr_OPB_select : std_logic;
  signal evr_OPB_seqAddr : std_logic;
  signal evr_irq : std_logic;
  
  signal control0   : std_logic_vector(35 downto 0);
  signal control1   : std_logic_vector(35 downto 0);
  signal control2   : std_logic_vector(35 downto 0);
  signal control3   : std_logic_vector(35 downto 0);
  signal control4   : std_logic_vector(35 downto 0);

  component adi_config is
    port (
      clk      : in std_logic;
      usec_ena : in std_logic;
      sck      : out std_logic;
      sda      : inout std_logic;
      status   : out std_logic
    );
  end component;

  component plbv46_ubicom_master is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_FAMILY                       : string               := "virtex5";
    C_MPLB_AWIDTH                  : integer              := 32;
    C_MPLB_DWIDTH                  : integer              := 128;
    C_MPLB_NATIVE_DWIDTH           : integer              := 32;
    C_MPLB_P2P                     : integer              := 0;
    C_MPLB_SMALLEST_SLAVE          : integer              := 32;
    C_MPLB_CLK_PERIOD_PS           : integer              := 10000
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    -- Ubicom interface signals
    ULE          : in std_logic;
    nUBERR       : out std_logic;
    nUACK        : out std_logic;
    nUIRQ        : out std_logic;
    nURD         : in std_logic;
    nUWR         : in std_logic;
    UA           : in std_logic_vector(16 downto 0);
                                  -- Ubicom address bits A16-A9 and A0,
                                  -- A8-A1 are multiplexed with D7-D0
    UD           : inout std_logic_vector(7 downto 0);
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    MPLB_Clk                       : in  std_logic;
    MPLB_Rst                       : in  std_logic;
    MD_error                       : out std_logic;
    M_request                      : out std_logic;
    M_priority                     : out std_logic_vector(0 to 1);
    M_busLock                      : out std_logic;
    M_RNW                          : out std_logic;
    M_BE                           : out std_logic_vector(0 to C_MPLB_DWIDTH/8-1);
    M_MSize                        : out std_logic_vector(0 to 1);
    M_size                         : out std_logic_vector(0 to 3);
    M_type                         : out std_logic_vector(0 to 2);
    M_TAttribute                   : out std_logic_vector(0 to 15);
    M_lockErr                      : out std_logic;
    M_abort                        : out std_logic;
    M_UABus                        : out std_logic_vector(0 to 31);
    M_ABus                         : out std_logic_vector(0 to 31);
    M_wrDBus                       : out std_logic_vector(0 to C_MPLB_DWIDTH-1);
    M_wrBurst                      : out std_logic;
    M_rdBurst                      : out std_logic;
    PLB_MAddrAck                   : in  std_logic;
    PLB_MSSize                     : in  std_logic_vector(0 to 1);
    PLB_MRearbitrate               : in  std_logic;
    PLB_MTimeout                   : in  std_logic;
    PLB_MBusy                      : in  std_logic;
    PLB_MRdErr                     : in  std_logic;
    PLB_MWrErr                     : in  std_logic;
    PLB_MIRQ                       : in  std_logic;
    PLB_MRdDBus                    : in  std_logic_vector(0 to (C_MPLB_DWIDTH-1));
    PLB_MRdWdAddr                  : in  std_logic_vector(0 to 3);
    PLB_MRdDAck                    : in  std_logic;
    PLB_MRdBTerm                   : in  std_logic;
    PLB_MWrDAck                    : in  std_logic;
    PLB_MWrBTerm                   : in  std_logic;
    control0    :   in std_logic_vector(35 downto 0)
  );
  end component;

  component plbv46_pci9056_master is
    generic
      (
	-- ADD USER GENERICS BELOW THIS LINE ---------------
	--USER generics added here
	-- ADD USER GENERICS ABOVE THIS LINE ---------------
	
	-- DO NOT EDIT BELOW THIS LINE ---------------------
	-- Bus protocol parameters, do not add to or delete
	C_FAMILY                       : string               := "virtex5";
	C_MPLB_AWIDTH                  : integer              := 32;
	C_MPLB_DWIDTH                  : integer              := 128;
	C_MPLB_NATIVE_DWIDTH           : integer              := 32;
	C_MPLB_P2P                     : integer              := 0;
	C_MPLB_SMALLEST_SLAVE          : integer              := 32;
	C_MPLB_CLK_PERIOD_PS           : integer              := 10000
	-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
    port
      (
	-- ADD USER PORTS BELOW THIS LINE ------------------
	-- PCI9056 local bus signals
	BBn : in std_logic;
	BDIPn : in std_logic;
	BGn : in std_logic;
	BRn : in std_logic;
	BURSTn : in std_logic;
	CCSn : in std_logic;
	DACKn : in std_logic_vector(1 downto 0);
	DREQn : in std_logic_vector(1 downto 0);
	DP : in std_logic_vector(3 downto 0);
	LA : inout std_logic_vector(31 downto 0);
	LD : inout std_logic_vector(31 downto 0);
	LINTIn : in std_logic;
	LINTOn : in std_logic;
	MDREQn_DMPAF_EOTn : in std_logic;
	PMEREQn : in std_logic;
	RD_WRn : in std_logic;
	RETRYn : in std_logic;
	TAn : in std_logic;
	TEAn : in std_logic;
	TSn : in std_logic;
	TSIZ : in std_logic_vector(1 downto 0);
	nLRESET : in std_logic;
	nSYSEN : in std_logic;
	nWAIT : inout std_logic;
	nBI : out std_logic;
	-- ADD USER PORTS ABOVE THIS LINE ------------------
	
	-- DO NOT EDIT BELOW THIS LINE ---------------------
	-- Bus protocol ports, do not add to or delete
	MPLB_Clk                       : in  std_logic;
	MPLB_Rst                       : in  std_logic;
	MD_error                       : out std_logic;
	M_request                      : out std_logic;
	M_priority                     : out std_logic_vector(0 to 1);
	M_busLock                      : out std_logic;
	M_RNW                          : out std_logic;
	M_BE                           : out std_logic_vector(0 to C_MPLB_DWIDTH/8-1);
	M_MSize                        : out std_logic_vector(0 to 1);
	M_size                         : out std_logic_vector(0 to 3);
	M_type                         : out std_logic_vector(0 to 2);
	M_TAttribute                   : out std_logic_vector(0 to 15);
	M_lockErr                      : out std_logic;
	M_abort                        : out std_logic;
	M_UABus                        : out std_logic_vector(0 to 31);
	M_ABus                         : out std_logic_vector(0 to 31);
	M_wrDBus                       : out std_logic_vector(0 to C_MPLB_DWIDTH-1);
	M_wrBurst                      : out std_logic;
	M_rdBurst                      : out std_logic;
	PLB_MAddrAck                   : in  std_logic;
	PLB_MSSize                     : in  std_logic_vector(0 to 1);
	PLB_MRearbitrate               : in  std_logic;
	PLB_MTimeout                   : in  std_logic;
	PLB_MBusy                      : in  std_logic;
	PLB_MRdErr                     : in  std_logic;
	PLB_MWrErr                     : in  std_logic;
	PLB_MIRQ                       : in  std_logic;
	PLB_MRdDBus                    : in  std_logic_vector(0 to (C_MPLB_DWIDTH-1));
	PLB_MRdWdAddr                  : in  std_logic_vector(0 to 3);
	PLB_MRdDAck                    : in  std_logic;
	PLB_MRdBTerm                   : in  std_logic;
	PLB_MWrDAck                    : in  std_logic;
	PLB_MWrBTerm                   : in  std_logic;
	-- DO NOT EDIT ABOVE THIS LINE ---------------------
	control0    :   in std_logic_vector(35 downto 0));
  end component;
    
  component system is
    port (
      sys_clk_pin : in std_logic;
      sys_rst_pin : in std_logic;
      dcm_locked_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MWrBTerm_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MWrDAck_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MRdBTerm_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MRdDAck_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MRdWdAddr_pin : out std_logic_vector(0 to 3);
      plbv46_master_wrapper_0_wrapped_PLB_MRdDBus_pin : out std_logic_vector(0 to 127);
      plbv46_master_wrapper_0_wrapped_PLB_MIRQ_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MWrErr_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MRdErr_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MBusy_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MTimeout_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MRearbitrate_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_PLB_MSSize_pin : out std_logic_vector(0 to 1);
      plbv46_master_wrapper_0_wrapped_PLB_MAddrAck_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_M_rdBurst_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_M_wrBurst_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_M_wrDBus_pin : in std_logic_vector(0 to 127);
      plbv46_master_wrapper_0_wrapped_M_ABus_pin : in std_logic_vector(0 to 31);
      plbv46_master_wrapper_0_wrapped_M_UABus_pin : in std_logic_vector(0 to 31);
      plbv46_master_wrapper_0_wrapped_M_abort_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_M_lockErr_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_M_TAttribute_pin : in std_logic_vector(0 to 15);
      plbv46_master_wrapper_0_wrapped_M_type_pin : in std_logic_vector(0 to 2);
      plbv46_master_wrapper_0_wrapped_M_size_pin : in std_logic_vector(0 to 3);
      plbv46_master_wrapper_0_wrapped_M_MSize_pin : in std_logic_vector(0 to 1);
      plbv46_master_wrapper_0_wrapped_M_BE_pin : in std_logic_vector(0 to 15);
      plbv46_master_wrapper_0_wrapped_M_RNW_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_M_busLock_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_M_priority_pin : in std_logic_vector(0 to 1);
      plbv46_master_wrapper_0_wrapped_M_request_pin : in std_logic;
      plbv46_master_wrapper_0_wrapped_MPLB_Rst_pin : out std_logic;
      plbv46_master_wrapper_0_wrapped_MPLB_Clk_pin : out std_logic;

--       plbv46_master_wrapper_1_wrapped_PLB_MWrBTerm_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MWrDAck_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MRdBTerm_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MRdDAck_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MRdWdAddr_pin : out std_logic_vector(0 to 3);
--       plbv46_master_wrapper_1_wrapped_PLB_MRdDBus_pin : out std_logic_vector(0 to 127);
--       plbv46_master_wrapper_1_wrapped_PLB_MIRQ_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MWrErr_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MRdErr_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MBusy_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MTimeout_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MRearbitrate_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_PLB_MSSize_pin : out std_logic_vector(0 to 1);
--       plbv46_master_wrapper_1_wrapped_PLB_MAddrAck_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_M_rdBurst_pin : in std_logic;
--       plbv46_master_wrapper_1_wrapped_M_wrBurst_pin : in std_logic;
--       plbv46_master_wrapper_1_wrapped_M_wrDBus_pin : in std_logic_vector(0 to 127);
--       plbv46_master_wrapper_1_wrapped_M_ABus_pin : in std_logic_vector(0 to 31);
--       plbv46_master_wrapper_1_wrapped_M_UABus_pin : in std_logic_vector(0 to 31);
--       plbv46_master_wrapper_1_wrapped_M_abort_pin : in std_logic;
--       plbv46_master_wrapper_1_wrapped_M_lockErr_pin : in std_logic;
--       plbv46_master_wrapper_1_wrapped_M_TAttribute_pin : in std_logic_vector(0 to 15);
--       plbv46_master_wrapper_1_wrapped_M_type_pin : in std_logic_vector(0 to 2);
--       plbv46_master_wrapper_1_wrapped_M_size_pin : in std_logic_vector(0 to 3);
--       plbv46_master_wrapper_1_wrapped_M_MSize_pin : in std_logic_vector(0 to 1);
--       plbv46_master_wrapper_1_wrapped_M_BE_pin : in std_logic_vector(0 to 15);
--       plbv46_master_wrapper_1_wrapped_M_RNW_pin : in std_logic;
--       plbv46_master_wrapper_1_wrapped_M_busLock_pin : in std_logic;
--       plbv46_master_wrapper_1_wrapped_M_priority_pin : in std_logic_vector(0 to 1);
--       plbv46_master_wrapper_1_wrapped_M_request_pin : in std_logic;
--       plbv46_master_wrapper_1_wrapped_MPLB_Rst_pin : out std_logic;
--       plbv46_master_wrapper_1_wrapped_MPLB_Clk_pin : out std_logic;

      plbv46_ipif_slave_0_IP2Bus_Error_pin : in std_logic;
      plbv46_ipif_slave_0_IP2Bus_WrAck_pin : in std_logic;
      plbv46_ipif_slave_0_IP2Bus_RdAck_pin : in std_logic;
      plbv46_ipif_slave_0_IP2Bus_Data_pin : in std_logic_vector(0 to 31);
      plbv46_ipif_slave_0_Bus2IP_BE_pin : out std_logic_vector(0 to 3);
      plbv46_ipif_slave_0_Bus2IP_Data_pin : out std_logic_vector(0 to 31);
      plbv46_ipif_slave_0_Bus2IP_RNW_pin : out std_logic;
      plbv46_ipif_slave_0_Bus2IP_CS_pin : out std_logic_vector(0 to 0);
      plbv46_ipif_slave_0_Bus2IP_Addr_pin : out std_logic_vector(0 to 31);
      plbv46_ipif_slave_0_Bus2IP_Reset_pin : out std_logic;
      plbv46_ipif_slave_0_Bus2IP_Clk_pin : out std_logic
    );
  end component;

  component opb_evr is
    generic (
      -- Bus protocol parameters, do not add to or delete
      C_BASEADDR         : std_logic_vector;
      C_BASEMASK         : std_logic_vector;
      C_OPB_AWIDTH       : integer := 32;
      C_OPB_DWIDTH       : integer := 32;
      C_FAMILY           : string  := "virtex2p");
    port (
      -- Bus protocol ports, do not add to or delete
      OPB_Clk         : in    std_logic;
      OPB_Rst         : in    std_logic;
      Sl_DBus         : out   std_logic_vector(0 to 31);
      Sl_errAck       : out   std_logic;
      Sl_retry        : out   std_logic;
      Sl_toutSup      : out   std_logic;
      Sl_xferAck      : out   std_logic;
      OPB_ABus        : in    std_logic_vector(0 to 31);
      OPB_BE          : in    std_logic_vector(0 to 3);
      OPB_DBus        : in    std_logic_vector(0 to 31);
      OPB_RNW         : in    std_logic;
      OPB_select      : in    std_logic;
      OPB_seqAddr     : in    std_logic;
      
      ip_irq_out      : out std_logic;
      
      TTLIN           : in std_logic_vector(0 to C_EVR_TTL_INPUTS-1);
      TTLOUT          : out std_logic_vector(0 to C_EVR_TTL_OUTPUTS-1);
      UNIVIN          : in std_logic_vector(0 to C_EVR_UNIV_INPUTS-1);
      UNIVOUT         : out std_logic_vector(0 to C_EVR_UNIV_OUTPUTS-1);

      GPIO            : inout std_logic_vector(C_EVR_GPIOS-1 downto 0);

      TBIN            : in std_logic_vector(0 to C_EVR_TB_INPUTS-1);
      TBOUT           : out std_logic_vector(0 to C_EVR_TB_OUTPUTS-1);
      
      -- Differential clocks from special brefclkx and brefclk2x pins
      -- Either pair may provide reference from fractional synthesizer
      BREFCLKN        : in std_logic; -- Reference clock input from
      BREFCLKP        : in std_logic; -- fractional synthesizer
      BREFCLK2N       : in std_logic; -- Resynchronised clock input
      BREFCLK2P       : in std_logic; -- from FPGA external FF

      RECCLKN         : out std_logic; -- Recovered clock output
      RECCLKP         : out std_logic;

      event_clk_out   : out std_logic;
     
      -- Front panel LEDs
      LED_fail        : out std_logic;
      LED_rx_link     : out std_logic;
      LED_ena         : out std_logic;
      LED_event_in    : out std_logic;
      LED_event_out   : out std_logic;
      LED_rx_fail     : out std_logic;
      LED_error       : out std_logic;
      
      -- MGT I/O pins
      EVRXN           : in std_logic;  -- Event Link RXN
      EVRXP           : in std_logic;  -- Event Link RXN
      EVTXN           : out std_logic; -- Event Link TXN
      EVTXP           : out std_logic; -- Event Link TXN
      FBRXN           : in std_logic;
      FBRXP           : in std_logic;
      FBTXN           : out std_logic;
      FBTXP           : out std_logic;
      CMLRXN          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLRXP          : in std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXN          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);
      CMLTXP          : out std_logic_vector(0 to C_EVR_CML_OUTPUTS-1);

      -- Micrel SY87729L serial bus signals
      CGSK        : out std_logic;
      CGDI        : out std_logic;
      CGCS        : out std_logic;
      CGLOCK      : in std_logic;
      TXLOOPBACK  : out std_logic; -- when '1' loop back FPGA TX to FPGA RX
      RXLOOPBACK  : out std_logic; -- when '1' loop back optical RX signal
                                   -- to transceiver TX port
    
      debug       : out std_logic_vector(39 downto 0);
      CONTROL0    : inout std_logic_vector(35 downto 0)
      );
  end component;

  component icon1
    PORT (
      CONTROL0 : OUT STD_LOGIC_VECTOR(35 DOWNTO 0));
  end component;

                                
begin

  i_icon : icon1
    port map (
      CONTROL0 => control0);

  i_bufr : bufr
    port map (
      O => rclk,
      CE => vcc,
      CLR => gnd,
      I => LCLK);

  i_bufg : bufg
    port map (
      O => clk,
      I => rclk);
  
  i_adi : adi_config
    port map (
      clk => clk,
      usec_ena => usec_ena,
      sck => ADNSCK,
      sda => ADNSDA);

  i_plb_ubicom : plbv46_ubicom_master
    port map (
      ULE => ULE,
      nUBERR => nUBERR,
      nUACK => nUACK,
--      nUIRQ => nUIRQ,
      nURD => nURD,
      nUWR => nUWR,
      UA => UA,
      UD => UD,
      MPLB_Clk => plbv46_ubicom_MPLB_Clk,
      MPLB_Rst => plbv46_ubicom_MPLB_Rst,
      M_request => plbv46_ubicom_M_request,
      M_priority => plbv46_ubicom_M_priority,
      M_busLock => plbv46_ubicom_M_busLock,
      M_RNW => plbv46_ubicom_M_RNW,
      M_BE => plbv46_ubicom_M_BE,
      M_MSize => plbv46_ubicom_M_MSize,
      M_size => plbv46_ubicom_M_size,
      M_type => plbv46_ubicom_M_type,
      M_TAttribute => plbv46_ubicom_M_TAttribute,
      M_lockErr => plbv46_ubicom_M_lockErr,
      M_abort => plbv46_ubicom_M_abort,
      M_UABus => plbv46_ubicom_M_UABus,
      M_ABus => plbv46_ubicom_M_ABus,
      M_wrDBus => plbv46_ubicom_M_wrDBus,
      M_wrBurst => plbv46_ubicom_M_wrBurst,
      M_rdBurst => plbv46_ubicom_M_rdBurst,
      PLB_MAddrAck => plbv46_ubicom_PLB_MAddrAck,
      PLB_MSSize => plbv46_ubicom_PLB_MSSize,
      PLB_MRearbitrate => plbv46_ubicom_PLB_MRearbitrate,
      PLB_MTimeout => plbv46_ubicom_PLB_MTimeout,
      PLB_MBusy => plbv46_ubicom_PLB_MBusy,
      PLB_MRdErr => plbv46_ubicom_PLB_MRdErr,
      PLB_MWrErr => plbv46_ubicom_PLB_MWrErr,
      PLB_MIRQ => plbv46_ubicom_PLB_MIRQ,
      PLB_MRdDBus => plbv46_ubicom_PLB_MRdDBus,
      PLB_MRdWdAddr => plbv46_ubicom_PLB_MRdWdAddr,
      PLB_MRdDAck => plbv46_ubicom_PLB_MRdDAck,
      PLB_MRdBTerm => plbv46_ubicom_PLB_MRdBTerm,
      PLB_MWrDAck => plbv46_ubicom_PLB_MWrDAck,
      PLB_MWrBTerm =>plbv46_ubicom_PLB_MWrBTerm,
      control0 => control3);

--   i_plb_pci9056 : plbv46_pci9056_master
--     port map (
--       BBn => BBn,
--       BDIPn => BDIPn,
--       BGn => BGn,
--       BRn => BRn,
--       BURSTn => BURSTn,
--       CCSn => CCSn,
--       DACKn => DACKn,
--       DREQn => DREQn,
--       DP => DP,
--       LA => LA,
--       LD => LD,
--       LINTIn => LINTIn,
--       LINTOn => LINTOn,
--       MDREQn_DMPAF_EOTn => MDREQn_DMPAF_EOTn,
--       PMEREQn => PMEREQn,
--       RD_WRn => RD_WRn,
--       RETRYn => RETRYn,
--       TAn => TAn,
--       TEAn => TEAn,
--       TSn => TSn,
--       TSIZ => TSIZ,
--       nLRESET => nLRESET,
--       nSYSEN => nSYSEN,
--       nWAIT => nWAIT,
--       nBI => nBI,
--       MPLB_Clk => plbv46_pci9056_MPLB_Clk,
--       MPLB_Rst => plbv46_pci9056_MPLB_Rst,
--       M_request => plbv46_pci9056_M_request,
--       M_priority => plbv46_pci9056_M_priority,
--       M_busLock => plbv46_pci9056_M_busLock,
--       M_RNW => plbv46_pci9056_M_RNW,
--       M_BE => plbv46_pci9056_M_BE,
--       M_MSize => plbv46_pci9056_M_MSize,
--       M_size => plbv46_pci9056_M_size,
--       M_type => plbv46_pci9056_M_type,
--       M_TAttribute => plbv46_pci9056_M_TAttribute,
--       M_lockErr => plbv46_pci9056_M_lockErr,
--       M_abort => plbv46_pci9056_M_abort,
--       M_UABus => plbv46_pci9056_M_UABus,
--       M_ABus => plbv46_pci9056_M_ABus,
--       M_wrDBus => plbv46_pci9056_M_wrDBus,
--       M_wrBurst => plbv46_pci9056_M_wrBurst,
--       M_rdBurst => plbv46_pci9056_M_rdBurst,
--       PLB_MAddrAck => plbv46_pci9056_PLB_MAddrAck,
--       PLB_MSSize => plbv46_pci9056_PLB_MSSize,
--       PLB_MRearbitrate => plbv46_pci9056_PLB_MRearbitrate,
--       PLB_MTimeout => plbv46_pci9056_PLB_MTimeout,
--       PLB_MBusy => plbv46_pci9056_PLB_MBusy,
--       PLB_MRdErr => plbv46_pci9056_PLB_MRdErr,
--       PLB_MWrErr => plbv46_pci9056_PLB_MWrErr,
--       PLB_MIRQ => plbv46_pci9056_PLB_MIRQ,
--       PLB_MRdDBus => plbv46_pci9056_PLB_MRdDBus,
--       PLB_MRdWdAddr => plbv46_pci9056_PLB_MRdWdAddr,
--       PLB_MRdDAck => plbv46_pci9056_PLB_MRdDAck,
--       PLB_MRdBTerm => plbv46_pci9056_PLB_MRdBTerm,
--       PLB_MWrDAck => plbv46_pci9056_PLB_MWrDAck,
--       PLB_MWrBTerm =>plbv46_pci9056_PLB_MWrBTerm,
--       control0 => control3);
  
  i_system : system
    port map (
      sys_clk_pin => clk,
      sys_rst_pin => reset,
      dcm_locked_pin => vcc,
      plbv46_master_wrapper_0_wrapped_PLB_MWrBTerm_pin => plbv46_ubicom_PLB_MWrBTerm,
      plbv46_master_wrapper_0_wrapped_PLB_MWrDAck_pin => plbv46_ubicom_PLB_MWrDAck,
      plbv46_master_wrapper_0_wrapped_PLB_MRdBTerm_pin => plbv46_ubicom_PLB_MRdBTerm,
      plbv46_master_wrapper_0_wrapped_PLB_MRdDAck_pin => plbv46_ubicom_PLB_MRdDAck,
      plbv46_master_wrapper_0_wrapped_PLB_MRdWdAddr_pin => plbv46_ubicom_PLB_MRdWdAddr,
      plbv46_master_wrapper_0_wrapped_PLB_MRdDBus_pin => plbv46_ubicom_PLB_MRdDBus,
      plbv46_master_wrapper_0_wrapped_PLB_MIRQ_pin => plbv46_ubicom_PLB_MIRQ,
      plbv46_master_wrapper_0_wrapped_PLB_MWrErr_pin => plbv46_ubicom_PLB_MWrErr,
      plbv46_master_wrapper_0_wrapped_PLB_MRdErr_pin => plbv46_ubicom_PLB_MRdErr,
      plbv46_master_wrapper_0_wrapped_PLB_MBusy_pin => plbv46_ubicom_PLB_MBusy,
      plbv46_master_wrapper_0_wrapped_PLB_MTimeout_pin => plbv46_ubicom_PLB_MTimeout,
      plbv46_master_wrapper_0_wrapped_PLB_MRearbitrate_pin => plbv46_ubicom_PLB_MRearbitrate,
      plbv46_master_wrapper_0_wrapped_PLB_MSSize_pin => plbv46_ubicom_PLB_MSSize,
      plbv46_master_wrapper_0_wrapped_PLB_MAddrAck_pin => plbv46_ubicom_PLB_MAddrAck,
      plbv46_master_wrapper_0_wrapped_M_rdBurst_pin => plbv46_ubicom_M_rdBurst,
      plbv46_master_wrapper_0_wrapped_M_wrBurst_pin => plbv46_ubicom_M_wrBurst,
      plbv46_master_wrapper_0_wrapped_M_wrDBus_pin => plbv46_ubicom_M_wrDBus,
      plbv46_master_wrapper_0_wrapped_M_ABus_pin => plbv46_ubicom_M_ABus,
      plbv46_master_wrapper_0_wrapped_M_UABus_pin => plbv46_ubicom_M_UABus,
      plbv46_master_wrapper_0_wrapped_M_abort_pin => plbv46_ubicom_M_abort,
      plbv46_master_wrapper_0_wrapped_M_lockErr_pin => plbv46_ubicom_M_lockErr,
      plbv46_master_wrapper_0_wrapped_M_TAttribute_pin => plbv46_ubicom_M_TAttribute,
      plbv46_master_wrapper_0_wrapped_M_type_pin => plbv46_ubicom_M_type,
      plbv46_master_wrapper_0_wrapped_M_size_pin => plbv46_ubicom_M_size,
      plbv46_master_wrapper_0_wrapped_M_MSize_pin => plbv46_ubicom_M_MSize,
      plbv46_master_wrapper_0_wrapped_M_BE_pin => plbv46_ubicom_M_BE,
      plbv46_master_wrapper_0_wrapped_M_RNW_pin => plbv46_ubicom_M_RNW,
      plbv46_master_wrapper_0_wrapped_M_busLock_pin => plbv46_ubicom_M_busLock,
      plbv46_master_wrapper_0_wrapped_M_priority_pin => plbv46_ubicom_M_priority,
      plbv46_master_wrapper_0_wrapped_M_request_pin => plbv46_ubicom_M_request,
      plbv46_master_wrapper_0_wrapped_MPLB_Rst_pin => plbv46_ubicom_MPLB_Rst,
      plbv46_master_wrapper_0_wrapped_MPLB_Clk_pin => plbv46_ubicom_MPLB_Clk,

--       plbv46_master_wrapper_1_wrapped_PLB_MWrBTerm_pin => plbv46_pci9056_PLB_MWrBTerm,
--       plbv46_master_wrapper_1_wrapped_PLB_MWrDAck_pin => plbv46_pci9056_PLB_MWrDAck,
--       plbv46_master_wrapper_1_wrapped_PLB_MRdBTerm_pin => plbv46_pci9056_PLB_MRdBTerm,
--       plbv46_master_wrapper_1_wrapped_PLB_MRdDAck_pin => plbv46_pci9056_PLB_MRdDAck,
--       plbv46_master_wrapper_1_wrapped_PLB_MRdWdAddr_pin => plbv46_pci9056_PLB_MRdWdAddr,
--       plbv46_master_wrapper_1_wrapped_PLB_MRdDBus_pin => plbv46_pci9056_PLB_MRdDBus,
--       plbv46_master_wrapper_1_wrapped_PLB_MIRQ_pin => plbv46_pci9056_PLB_MIRQ,
--       plbv46_master_wrapper_1_wrapped_PLB_MWrErr_pin => plbv46_pci9056_PLB_MWrErr,
--       plbv46_master_wrapper_1_wrapped_PLB_MRdErr_pin => plbv46_pci9056_PLB_MRdErr,
--       plbv46_master_wrapper_1_wrapped_PLB_MBusy_pin => plbv46_pci9056_PLB_MBusy,
--       plbv46_master_wrapper_1_wrapped_PLB_MTimeout_pin => plbv46_pci9056_PLB_MTimeout,
--       plbv46_master_wrapper_1_wrapped_PLB_MRearbitrate_pin => plbv46_pci9056_PLB_MRearbitrate,
--       plbv46_master_wrapper_1_wrapped_PLB_MSSize_pin => plbv46_pci9056_PLB_MSSize,
--       plbv46_master_wrapper_1_wrapped_PLB_MAddrAck_pin => plbv46_pci9056_PLB_MAddrAck,
--       plbv46_master_wrapper_1_wrapped_M_rdBurst_pin => plbv46_pci9056_M_rdBurst,
--       plbv46_master_wrapper_1_wrapped_M_wrBurst_pin => plbv46_pci9056_M_wrBurst,
--       plbv46_master_wrapper_1_wrapped_M_wrDBus_pin => plbv46_pci9056_M_wrDBus,
--       plbv46_master_wrapper_1_wrapped_M_ABus_pin => plbv46_pci9056_M_ABus,
--       plbv46_master_wrapper_1_wrapped_M_UABus_pin => plbv46_pci9056_M_UABus,
--       plbv46_master_wrapper_1_wrapped_M_abort_pin => plbv46_pci9056_M_abort,
--       plbv46_master_wrapper_1_wrapped_M_lockErr_pin => plbv46_pci9056_M_lockErr,
--       plbv46_master_wrapper_1_wrapped_M_TAttribute_pin => plbv46_pci9056_M_TAttribute,
--       plbv46_master_wrapper_1_wrapped_M_type_pin => plbv46_pci9056_M_type,
--       plbv46_master_wrapper_1_wrapped_M_size_pin => plbv46_pci9056_M_size,
--       plbv46_master_wrapper_1_wrapped_M_MSize_pin => plbv46_pci9056_M_MSize,
--       plbv46_master_wrapper_1_wrapped_M_BE_pin => plbv46_pci9056_M_BE,
--       plbv46_master_wrapper_1_wrapped_M_RNW_pin => plbv46_pci9056_M_RNW,
--       plbv46_master_wrapper_1_wrapped_M_busLock_pin => plbv46_pci9056_M_busLock,
--       plbv46_master_wrapper_1_wrapped_M_priority_pin => plbv46_pci9056_M_priority,
--       plbv46_master_wrapper_1_wrapped_M_request_pin => plbv46_pci9056_M_request,
--       plbv46_master_wrapper_1_wrapped_MPLB_Rst_pin => plbv46_pci9056_MPLB_Rst,
--       plbv46_master_wrapper_1_wrapped_MPLB_Clk_pin => plbv46_pci9056_MPLB_Clk,

      plbv46_ipif_slave_0_IP2Bus_Error_pin => plbv46_test_IP2Bus_Error,
      plbv46_ipif_slave_0_IP2Bus_WrAck_pin => plbv46_test_IP2Bus_WrAck,
      plbv46_ipif_slave_0_IP2Bus_RdAck_pin => plbv46_test_IP2Bus_RdAck,
      plbv46_ipif_slave_0_IP2Bus_Data_pin => plbv46_test_IP2Bus_Data,
      plbv46_ipif_slave_0_Bus2IP_BE_pin => plbv46_test_Bus2IP_BE,
      plbv46_ipif_slave_0_Bus2IP_Data_pin => plbv46_test_Bus2IP_Data,
      plbv46_ipif_slave_0_Bus2IP_RNW_pin => plbv46_test_Bus2IP_RNW,
      plbv46_ipif_slave_0_Bus2IP_CS_pin => plbv46_test_Bus2IP_CS,
      plbv46_ipif_slave_0_Bus2IP_Addr_pin => plbv46_test_Bus2IP_Addr,
      plbv46_ipif_slave_0_Bus2IP_Reset_pin => plbv46_test_Bus2IP_Reset,
      plbv46_ipif_slave_0_Bus2IP_Clk_pin => plbv46_test_Bus2IP_Clk
      );
  
  i_opb_evr : opb_evr
    generic map (
      C_BASEADDR => EVR_LOCAL_BASE,
      C_BASEMASK => LOCAL_BASE_MASK)
    port map (
      OPB_Clk => evr_OPB_Clk,
      OPB_Rst => evr_OPB_Rst,
      Sl_DBus => evr_Sl_DBus,
      Sl_errAck => evr_Sl_errAck,
      Sl_retry => evr_Sl_retry,
      Sl_toutSup => evr_Sl_toutSup,
      Sl_xferAck => evr_Sl_xferAck,
      OPB_ABus => evr_OPB_ABus,
      OPB_BE => evr_OPB_BE,
      OPB_DBus => evr_OPB_DBus,
      OPB_RNW => evr_OPB_RNW,
      OPB_select => evr_OPB_select,
      OPB_seqAddr => evr_OPB_seqAddr,

      ip_irq_out => evr_irq,
    
      UNIVIN => UNIVIN,

      GPIO => GPIO,
      
      BREFCLKN => REFCLKN,
      BREFCLKP => REFCLKP,
      BREFCLK2N => RXREFCLKN,
      BREFCLK2P => RXREFCLKP,

      event_clk_out => event_clk,
      
      LED_fail => evr_LED_fail,
      LED_rx_link => evr_LED_rx_link,
      LED_ena => evr_LED_ena,
      LED_event_in => evr_LED_event_in,
      LED_event_out => evr_LED_event_out,
      LED_rx_fail => evr_LED_rx_fail,
      LED_error => evr_LED_error,
    
      EVRXN => RDUPN,
      EVRXP => RDUPP,
      EVTXN => TDUPN,
      EVTXP => TDUPP,
      FBRXN => RDRESN,
      FBRXP => RDRESP,
      FBTXN => TDRESN,
      FBTXP => TDRESP,
      CMLRXN => CMLRXN,
      CMLRXP => CMLRXP,
      CMLTXN => CMLTXN,
      CMLTXP => CMLTXP,

      CGSK => PROGSK,
      CGDI => PROGDI,
      CGCS => PROGCS,
      CGLOCK => LOCKED,

      CONTROL0 => control0);

  CMLRXN <= RD8N & RD7N & RD6N & RD5N & RD4N & RD3N & RD2N & RD1N;
  CMLRXP <= RD8P & RD7P & RD6P & RD5P & RD4P & RD3P & RD2P & RD1P;
  TD1N <= CMLTXN(7);
  TD2N <= CMLTXN(6);
  TD3N <= CMLTXN(5);
  TD4N <= CMLTXN(4);
  TD5N <= CMLTXN(3);
  TD6N <= CMLTXN(2);
  TD7N <= CMLTXN(1);
  TD8N <= CMLTXN(0);
  TD1P <= CMLTXP(7);
  TD2P <= CMLTXP(6);
  TD3P <= CMLTXP(5);
  TD4P <= CMLTXP(4);
  TD5P <= CMLTXP(3);
  TD6P <= CMLTXP(2);
  TD7P <= CMLTXP(1);
  TD8P <= CMLTXP(0);

  usec_enable_generation : process (clk)
    variable counter : std_logic_vector(7 downto 0) := (others => '0');
  begin
    if rising_edge(clk) then
      usec_ena <= '0';
      if counter(counter'high) = '1' then
        counter := X"42";
        usec_ena <= '1';
      end if;
      counter := counter - 1;
    end if;
  end process;

  local_reset : process (clk)
    variable counter : unsigned(4 downto 0) := "01111";
  begin
    reset <= not counter(counter'high);
    if rising_edge(clk) then
      if counter(counter'high) = '0' then
        counter := counter - 1;
      end if;
    end if;
  end process;

  DIV_SEL <= '1';
  EN <= '1';
  nMR <= '1';

  LA <= (others => 'Z');
  LD <= (others => 'Z');
  nWAIT <= 'Z';
  nBI <= '1';
  
--  nUACK <= '1';
--  nUBERR <= '1';
--  UD <= (others => 'Z');
--  nUACK <= recclk(0);
--  UD <= recclk(8 downto 1);

  DDRAA <= (others => '0');
  DDRABA <= (others => '0');
  DDRACASn <= '1';
  DDRARASn <= '1';
  DDRAWEn <= '1';
  DDRACK <= '0';
  DDRACKn <= '1';
  DDRACKE <= '0';
  DDRACSn <= '1';
  DDRADQ <= (others => '0');
  DDRALDM <= '0';
  DDRAUDM <= '0';
  DDRALDQS <= '0';
  DDRALDQSn <= '1';
  DDRAUDQS <= '0';
  DDRAUDQSn <= '1';
  DDRAODT <= '0';
  
  DDRBA <= (others => '0');
  DDRBBA <= (others => '0');
  DDRBCASn <= '1';
  DDRBRASn <= '1';
  DDRBWEn <= '1';
  DDRBCK <= '0';
  DDRBCKn <= '1';
  DDRBCKE <= '0';
  DDRBCSn <= '1';
  DDRBDQ <= (others => '0');
  DDRBLDM <= '0';
  DDRBUDM <= '0';
  DDRBLDQS <= '0';
  DDRBLDQSn <= '1';
  DDRBUDQS <= '0';
  DDRBUDQSn <= '1';
  DDRBODT <= '0';

  PHY_GTXCLK <= '0';
  PHY_MDC <= '0';
  PHY_MDIO <= 'Z';
  PHY_TXD <= (others => '0');
  PHY_TX_EN <= '0';
  PHY_TX_ER <= '0';

  TXD <= RXD;

  SYNC_1V0 <= '0';
  SYNC_1V8 <= '0';

  MOD_TXD <= (others => '0');
  nMOD_SEL <= (others => '1');

  gnd <= '0';
  vcc <= '1';

  LEN <= (others => '1');
  DLY <= (others => '0');
--  GPIO(6 downto 1) <= (others => '0');

--  CLK_OUT <= event_clk;
  
  i_ddr : FDDRRSE
    port map (
      Q => CLK_OUT,
      C0 => event_clk,
      C1 => not event_clk,
      CE => vcc,
      D0 => gnd,
      D1 => vcc,
      R => gnd,
      S => gnd);
  
  OUTEN <= '0';

  evr_OPB_Clk <= plbv46_test_Bus2IP_Clk;
  evr_OPB_Rst <= plbv46_test_Bus2IP_Reset;
  plbv46_test_IP2Bus_Data <= evr_Sl_DBus;
  plbv46_test_IP2Bus_Error <= evr_Sl_errAck;

  process (evr_OPB_RNW, evr_Sl_xferAck)
  begin
    plbv46_test_IP2Bus_WrAck <= '0';
    plbv46_test_IP2Bus_RdAck <= '0';
    if evr_Sl_xferAck = '1' then
      if evr_OPB_RNW = '1' then
	plbv46_test_IP2Bus_RdAck <= '1';
      else
	plbv46_test_IP2Bus_WrAck <= '1';
      end if;
    end if;
  end process;
  
  evr_OPB_ABus <= plbv46_test_Bus2IP_Addr;
  evr_OPB_BE <= plbv46_test_Bus2IP_BE;
  evr_OPB_DBus <= plbv46_test_Bus2IP_Data;
  evr_OPB_RNW <= plbv46_test_Bus2IP_RNW;
  evr_OPB_select <= plbv46_test_Bus2IP_CS(0);
  evr_OPB_seqAddr <= '0';
  
--  led_test: process (clk)
--    variable down_div : natural range 0 to 19999999 := 0;
--    variable led_pat  : std_logic_vector(63 downto 0) := X"8000000080000000";
--  begin
--    if rising_edge(clk) then
--      led_on <= led_pat;
--      down_div := down_div + 1;
--      if down_div = 19999999 then
--        down_div := 0;
--        led_pat := led_pat(62 downto 0) & led_pat(63);
--      end if;
--    end if;
--  end process;

  fp_leds: process (clk, led_on)
    variable ce        : std_logic;
    variable ce_cnt    : std_logic_vector(3 downto 0) := X"0";
    variable cycle_cnt : std_logic_vector(11 downto 0) := X"000"; 
    variable blank     : std_logic := '1';
    variable xlat      : std_logic := '0';
    variable led_pt    : integer range 0 to 63;
  begin
    LEDBLANK <= blank;
    LEDSCLK <= ce_cnt(ce_cnt'high - 1);               
    LEDXLAT <= (not ce_cnt(ce_cnt'high - 1)) and xlat;

    if rising_edge(clk) then
      led_link_rx(1) <= not evr_LED_rx_link;
      led_link_tx(1) <= not evr_LED_ena;
      led_b(0) <= not evr_LED_event_in;
      led_c(0) <= not evr_LED_event_out;
      led_link_rx(2) <= not evr_LED_rx_fail;
      led_link_tx(2) <= not evr_LED_error;
      
      led_on <= (others => '0');
      -- IC38 OUT0 through OUT14
      led_on(62) <= led_link_rx(2); -- R
      led_on(61) <= led_link_rx(1); -- G
      led_on(60) <= led_link_rx(0); -- B
      led_on(59) <= led_link_tx(2);
      led_on(58) <= led_link_tx(1);
      led_on(57) <= led_link_tx(0);
      led_on(56) <= led_ch2_rx(2); 
      led_on(55) <= led_ch2_rx(1); 
      led_on(54) <= led_ch2_rx(0); 
      led_on(53) <= led_ch2_tx(2); 
      led_on(52) <= led_ch2_tx(1); 
      led_on(51) <= led_ch2_tx(0); 
      led_on(50) <= led_ch1_rx(2); 
      led_on(49) <= led_ch1_rx(1); 
      led_on(48) <= led_ch1_rx(0); 
      -- IC39 OUT0 through OUT14
      led_on(46) <= led_ch1_tx(2); 
      led_on(45) <= led_ch1_tx(1); 
      led_on(44) <= led_ch1_tx(0); 
      led_on(43) <= led_d(2); 
      led_on(42) <= led_d(1); 
      led_on(41) <= led_d(0); 
      led_on(40) <= led_c(2); 
      led_on(39) <= led_c(1); 
      led_on(38) <= led_c(0); 
      led_on(37) <= led_b(2); 
      led_on(36) <= led_b(1); 
      led_on(35) <= led_b(0); 
      led_on(34) <= led_a(2); 
      led_on(33) <= led_a(1); 
      led_on(32) <= led_a(0); 
      -- IC41 OUT9 through OUT14
--      led_on(21) <= led_com_tx(2);
--      led_on(20) <= led_com_tx(1);
--      led_on(19) <= led_com_tx(0);
--      led_on(18) <= led_com_rx(2);
--      led_on(17) <= led_com_rx(1);
--      led_on(16) <= led_com_rx(0);
--      led_on(40 downto 33) <= GPIO;
      led_on(23 downto 16) <= GPIO;
      
      ce := ce_cnt(ce_cnt'high);
      ce_cnt := ce_cnt + 1;
      if ce = '1' then
        ce_cnt := X"0";

        if cycle_cnt(cycle_cnt'high) = '0' then
          -- shift in DC values
          LEDMODE <= '1';
          LEDSIN <= '1';
        else  
          LEDMODE <= '0';
          led_pt := conv_integer(cycle_cnt(5 downto 0));
          LEDSIN <= led_on(led_pt);
        end if;

        xlat := '0';
        if cycle_cnt = X"7FF" or cycle_cnt = X"83F" then
          xlat := '1';
        end if;

        cycle_cnt := cycle_cnt + 1;
        if cycle_cnt = X"840" then
          cycle_cnt := X"800";
          blank := '0';
        end if;
      end if;
    end if;
  end process;


end;
