---------------------------------------------------------------------------
--
--  File        : event_log.vhd
--
--  Title       : Event Receiver Event Log
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : Logs up to 512 Events
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity event_log is
  port (
    log_in      : in std_logic_vector(79 downto 0);
    log_wr_en   : in std_logic;
    log_wr_clk  : in std_logic;

    clk         : in std_logic;
    data_out    : out std_logic_vector(79 downto 0);
    addr        : in std_logic_vector(8 downto 0); 
    wr_pnt      : out std_logic_vector(8 downto 0);
    wr_overflow : out std_logic;
    reset       : in std_logic);
end event_log;

architecture rtl of event_log is

  signal bram_addr_A     : std_logic_vector(8 downto 0);
  signal bram_addr_B     : std_logic_vector(8 downto 0);
  signal bram_data_A_in  : std_logic_vector(79 downto 0);
  signal bram_data_B_out : std_logic_vector(79 downto 0);

  component event_log_ram is
  port (
      bram_data_B_out : out std_logic_vector(79 downto 0);
      bram_addr_A     : in  std_logic_vector(8 downto 0);
      bram_addr_B     : in  std_logic_vector(8 downto 0);
      CLKA            : in  std_logic;
      CLKB            : in  std_logic;
      bram_data_A_in  : in  std_logic_vector(79 downto 0);
      WEA             : in  std_logic);
  end component;

begin

  i_ram0: event_log_ram
    port map (
      bram_data_B_out => bram_data_B_out,
      bram_addr_A => bram_addr_A,
      bram_addr_B => bram_addr_B,
      CLKA => log_wr_clk,
      CLKB => clk,
      bram_data_A_in => bram_data_A_in,
      WEA => log_wr_en);

  bram_data_A_in <= log_in;
  bram_addr_B <= addr;
  data_out <= bram_data_B_out;
  
  process (log_wr_clk, log_wr_en, reset)
    variable overflow   : std_logic := '0';
    variable write_pos  : unsigned(9 downto 0) := (others => '0');
    variable reset_sync : std_logic_vector(1 downto 0); 
  begin
    bram_addr_A <= std_logic_vector(write_pos(8 downto 0));
    wr_pnt <= std_logic_vector(write_pos(8 downto 0));
    wr_overflow <= overflow;

    if rising_edge(log_wr_clk) then
      if log_wr_en = '1' then
        write_pos := write_pos + 1;
        if write_pos(write_pos'high) = '1' then
          overflow := '1';
        end if;
      end if;
      if reset_sync(reset_sync'high) = '1' then
        write_pos := (others => '0');
        overflow := '0';
      end if;
      reset_sync := reset_sync(reset_sync'high-1 downto 0) & reset;
    end if;
  end process;

end rtl;
