---------------------------------------------------------------------------
--
--  File        : gtx_cmlout_bram.vhd
--
--  Title       : Virtex 5 FX - GTX Transceiver
--		  to Generate CML outputs: RF and pulses
--                utilising BRAM blocks
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity gtx_cmlout_bram is
  port (
    clk          : in std_logic;
    data_in      : in std_logic_vector(31 downto 0);
    data_out     : out std_logic_vector(31 downto 0);
    addr         : in std_logic_vector(13 downto 2);
    we           : in std_logic_vector(3 downto 0);
    cs           : in std_logic;
    
    event_clk    : in std_logic;
    tx_data      : out std_logic_vector(39 downto 0);
    pulse        : in std_logic;
    enable       : in std_logic;
    freq_mode    : in std_logic;
    high_period  : in std_logic_vector(15 downto 0);
    low_period   : in std_logic_vector(15 downto 0);
    trig_pos     : in std_logic_vector(15 downto 0);
    trig_level   : in std_logic;
    pattern_mode : in std_logic;
    recycle      : in std_logic;
    samples      : in std_logic_vector(10 downto 0));
end gtx_cmlout_bram;

architecture STRUCT of gtx_cmlout_bram is

  signal gnd      : std_logic;
  signal vcc      : std_logic;
  signal gnd_vec  : std_logic_vector(31 downto 0);
  
  signal ADDRA    : std_logic_vector(10 downto 0);
  signal DOA      : std_logic_vector(39 downto 0);
  signal ADDRB    : std_logic_vector(10 downto 0);
  signal DOB      : std_logic_vector(39 downto 0);
  signal we_i     : std_logic_vector(4 downto 0);
  
begin

  bram_0 : RAMB18
    generic map (
      DOA_REG => 1,
      DOB_REG => 1,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 9,
      READ_WIDTH_B => 9,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 9,
      WRITE_WIDTH_B => 9,
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA(15 downto 8) => open,
      DOA(7 downto 0) => DOA(7 downto 0),
      DOB(15 downto 8) => open,
      DOB(7 downto 0) => DOB(7 downto 0),
      DOPA => open,
      DOPB => open,
      ADDRA(13 downto 3) => ADDRA,
      ADDRA(2 downto 0) => gnd_vec(2 downto 0),
      ADDRB(13 downto 3) => ADDRB,
      ADDRB(2 downto 0) => gnd_vec(2 downto 0),
      CLKA => clk,
      CLKB => event_clk,
      DIA(15 downto 8) => gnd_vec(15 downto 8),
      DIA(7 downto 0) => data_in(7 downto 0),
      DIB => gnd_vec(15 downto 0),
      DIPA => gnd_vec(1 downto 0),
      DIPB => gnd_vec(1 downto 0),
      ENA => vcc,
      ENB => vcc,
      REGCEA => vcc,
      REGCEB => vcc,
      SSRA => gnd,
      SSRB => gnd,
      WEA(0) => we_i(0),
      WEA(1) => we_i(0),
      WEB => gnd_vec(1 downto 0));

  bram_1 : RAMB18
    generic map (
      DOA_REG => 1,
      DOB_REG => 1,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 9,
      READ_WIDTH_B => 9,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 9,
      WRITE_WIDTH_B => 9,
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA(15 downto 8) => open,
      DOA(7 downto 0) => DOA(15 downto 8),
      DOB(15 downto 8) => open,
      DOB(7 downto 0) => DOB(15 downto 8),
      DOPA => open,
      DOPB => open,
      ADDRA(13 downto 3) => ADDRA,
      ADDRA(2 downto 0) => gnd_vec(2 downto 0),
      ADDRB(13 downto 3) => ADDRB,
      ADDRB(2 downto 0) => gnd_vec(2 downto 0),
      CLKA => clk,
      CLKB => event_clk,
      DIA(15 downto 8) => gnd_vec(15 downto 8),
      DIA(7 downto 0) => data_in(15 downto 8),
      DIB(15 downto 0) => gnd_vec(15 downto 0),
      DIPA => gnd_vec(1 downto 0),
      DIPB => gnd_vec(1 downto 0),
      ENA => vcc,
      ENB => vcc,
      REGCEA => vcc,
      REGCEB => vcc,
      SSRA => gnd,
      SSRB => gnd,
      WEA(0) => we_i(1),
      WEA(1) => we_i(1),
      WEB => gnd_vec(1 downto 0));

  bram_2 : RAMB18
    generic map (
      DOA_REG => 1,
      DOB_REG => 1,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 9,
      READ_WIDTH_B => 9,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 9,
      WRITE_WIDTH_B => 9,
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA(15 downto 8) => open,
      DOA(7 downto 0) => DOA(23 downto 16),
      DOB(15 downto 8) => open,
      DOB(7 downto 0) => DOB(23 downto 16),
      DOPA => open,
      DOPB => open,
      ADDRA(13 downto 3) => ADDRA,
      ADDRA(2 downto 0) => gnd_vec(2 downto 0),
      ADDRB(13 downto 3) => ADDRB,
      ADDRB(2 downto 0) => gnd_vec(2 downto 0),
      CLKA => clk,
      CLKB => event_clk,
      DIA(15 downto 8) => gnd_vec(15 downto 8),
      DIA(7 downto 0) => data_in(23 downto 16),
      DIB(15 downto 0) => gnd_vec(15 downto 0),
      DIPA => gnd_vec(1 downto 0),
      DIPB => gnd_vec(1 downto 0),
      ENA => vcc,
      ENB => vcc,
      REGCEA => vcc,
      REGCEB => vcc,
      SSRA => gnd,
      SSRB => gnd,
      WEA(0) => we_i(2),
      WEA(1) => we_i(2),
      WEB => gnd_vec(1 downto 0));

  bram_3 : RAMB18
    generic map (
      DOA_REG => 1,
      DOB_REG => 1,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 9,
      READ_WIDTH_B => 9,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 9,
      WRITE_WIDTH_B => 9,
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA(15 downto 8) => open,
      DOA(7 downto 0) => DOA(31 downto 24),
      DOB(15 downto 8) => open,
      DOB(7 downto 0) => DOB(31 downto 24),
      DOPA => open,
      DOPB => open,
      ADDRA(13 downto 3) => ADDRA,
      ADDRA(2 downto 0) => gnd_vec(2 downto 0),
      ADDRB(13 downto 3) => ADDRB,
      ADDRB(2 downto 0) => gnd_vec(2 downto 0),
      CLKA => clk,
      CLKB => event_clk,
      DIA(15 downto 8) => gnd_vec(15 downto 8),
      DIA(7 downto 0) => data_in(31 downto 24),
      DIB(15 downto 0) => gnd_vec(15 downto 0),
      DIPA => gnd_vec(1 downto 0),
      DIPB => gnd_vec(1 downto 0),
      ENA => vcc,
      ENB => vcc,
      REGCEA => vcc,
      REGCEB => vcc,
      SSRA => gnd,
      SSRB => gnd,
      WEA(0) => we_i(3),
      WEA(1) => we_i(3),
      WEB => gnd_vec(1 downto 0));

  bram_4 : RAMB18
    generic map (
      DOA_REG => 1,
      DOB_REG => 1,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 9,
      READ_WIDTH_B => 9,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000000",
      SRVAL_B => X"00000000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 9,
      WRITE_WIDTH_B => 9,
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA(15 downto 8) => open,
      DOA(7 downto 0) => DOA(39 downto 32),
      DOB(15 downto 8) => open,
      DOB(7 downto 0) => DOB(39 downto 32),
      DOPA => open,
      DOPB => open,
      ADDRA(13 downto 3) => ADDRA,
      ADDRA(2 downto 0) => gnd_vec(2 downto 0),
      ADDRB(13 downto 3) => ADDRB,
      ADDRB(2 downto 0) => gnd_vec(2 downto 0),
      CLKA => clk,
      CLKB => event_clk,
      DIA(15 downto 8) => gnd_vec(15 downto 8),
      DIA(7 downto 0) => data_in(7 downto 0),
      DIB(15 downto 0) => gnd_vec(15 downto 0),
      DIPA => gnd_vec(1 downto 0),
      DIPB => gnd_vec(1 downto 0),
      ENA => vcc,
      ENB => vcc,
      REGCEA => vcc,
      REGCEB => vcc,
      SSRA => gnd,
      SSRB => gnd,
      WEA(0) => we_i(4),
      WEA(1) => we_i(4),
      WEB => gnd_vec(1 downto 0));
  
  gnd <= '0';
  vcc <= '1';
  gnd_vec <= (others => '0');
  
  ADDRA <= addr(13 downto 3);

  process (addr, we, cs, DOA)
  begin
    data_out <= DOA(31 downto 0);
    if addr(2) = '0' then
      data_out <= (others => '0');
      data_out(7 downto 0) <= DOA(39 downto 32);
    end if;
    we_i <= (others => '0');

    if cs = '1' then
      if addr(2) = '0' then
	we_i(4) <= we(0);
      else
	we_i(3 downto 0) <= we;
      end if;
    end if;
  end process;
    
  process (event_clk)
  begin
    if rising_edge(event_clk) then
      tx_data <= (others => '0');
      if enable = '1' then
	tx_data <= DOB;
      end if;
    end if;
  end process;
  
  process (event_clk, pulse)
    variable pulse_in  : std_logic_vector(1 downto 0) := "00";
    variable running   : std_logic := '0';
    variable cycle_cnt : std_logic_vector(10 downto 0) := (others => '1');
  begin
    if rising_edge(event_clk) then

      if running = '1' then
	cycle_cnt := cycle_cnt + 1;
	if cycle_cnt = samples then
	  if recycle = '1' then
	    cycle_cnt := (others => '0');
	  else
	    cycle_cnt := (others => '1');
	    running := '0';
	  end if;
	end if;
      end if;

      if pulse_in = "01" then
	cycle_cnt := (others => '0');
	running := '1';
      end if;

      if enable = '0' then
	running := '0';
	cycle_cnt := (others => '1');
      end if;
      
      ADDRB <= cycle_cnt;

      -- Check if we are running in CML compatibility mode
      if pattern_mode = '0' then
	ADDRB <= (others => '0');
	pulse_in := pulse_in(0) & pulse;
	ADDRB(1 downto 0) <= pulse_in;
      end if;
    end if;
  end process;
  
end struct;

