---------------------------------------------------------------------------
--                                                                       --
--  File        : local_clock.vhd                                        --
--                                                                       --
--  Title       : Local bus clock generator
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--                                                                       --
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
-- LIBRARY UNISIM;
-- use UNISIM.VCOMPONENTS.ALL;

entity local_clock is
  port (
    clkin      : in std_logic;
    clkfxout   : out std_logic
    );
end local_clock;

architecture rtl of local_clock is

  signal clk0_out      : std_logic;
  signal clkfx         : std_logic;
  signal user_CLK0     : std_logic;
  signal user_CLK180   : std_logic;
  signal user_CLK270   : std_logic;
  signal user_CLK2X    : std_logic;
  signal user_CLK2X180 : std_logic;
  signal user_CLK90    : std_logic;
  signal user_CLKDV    : std_logic;
  signal user_CLKFX    : std_logic;
  signal user_CLKFX180 : std_logic;
  signal user_LOCKED   : std_logic;
  signal user_PSDONE   : std_logic;
  signal user_STATUS   : std_logic_vector(7 downto 0);
  signal user_CLKFB    : std_logic;
  signal user_CLKIN    : std_logic;
  signal user_DSSEN    : std_logic;
  signal user_PSCLK    : std_logic;
  signal user_PSEN     : std_logic;
  signal user_PSINCDEC : std_logic;
  signal user_RST      : std_logic;

  component bufg
    port (
      O : out std_logic;
      I : in  std_logic
      );
  end component;
  
  component DCM
    generic (
      CLKDV_DIVIDE : real := 2.0;
      CLKFX_DIVIDE : integer := 1;
      CLKFX_MULTIPLY : integer := 4;
      CLKIN_DIVIDE_BY_2 : boolean := false;
      CLKIN_PERIOD : real := 0.0;
      CLKOUT_PHASE_SHIFT : string := "NONE";
      CLK_FEEDBACK : string := "1X";
      DESKEW_ADJUST : string := "SYSTEM_SYNCHRONOUS";
      DFS_FREQUENCY_MODE : string := "LOW";
      DLL_FREQUENCY_MODE : string := "LOW";
      DSS_MODE : string := "NONE";
      DUTY_CYCLE_CORRECTION : boolean := true;
      FACTORY_JF : bit_vector := X"C080";                 
      MAXPERCLKIN : time := 1000000 ps;                   
      MAXPERPSCLK : time := 100000000 ps;                 
      PHASE_SHIFT : integer := 0;
      SIM_CLKIN_CYCLE_JITTER : time := 300 ps;            
      SIM_CLKIN_PERIOD_JITTER : time := 1000 ps;          
      STARTUP_WAIT : boolean := false);
    port (
      CLKIN : in STD_LOGIC;
      CLKFB : in STD_LOGIC;
      RST : in STD_LOGIC;
      PSEN : in STD_LOGIC;
      PSINCDEC : in STD_LOGIC;
      PSCLK : in STD_LOGIC;
      DSSEN : in STD_LOGIC;
      CLK0 : out STD_LOGIC;
      CLK90 : out STD_LOGIC;
      CLK180 : out STD_LOGIC;
      CLK270 : out STD_LOGIC;
      CLKDV : out STD_LOGIC;
      CLK2X : out STD_LOGIC;
      CLK2X180 : out STD_LOGIC;
      CLKFX : out STD_LOGIC;
      CLKFX180 : out STD_LOGIC;
      STATUS : out STD_LOGIC_VECTOR (7 downto 0);
      LOCKED : out STD_LOGIC;
      PSDONE : out STD_LOGIC);
  end component;

begin

  BUFG_clkfx : bufg
    port map (
      O => clkfx,
      I => user_clkfx);

  DCM_local : DCM
    generic map(
      CLK_FEEDBACK => "NONE",
      CLKDV_DIVIDE => 2.0,
      CLKFX_DIVIDE => 1,
      CLKFX_MULTIPLY => 2,
      CLKIN_DIVIDE_BY_2 => FALSE,
      CLKIN_PERIOD => 30.0,
      CLKOUT_PHASE_SHIFT => "NONE",
      DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
      DFS_FREQUENCY_MODE => "LOW",
      DLL_FREQUENCY_MODE => "LOW",
      DUTY_CYCLE_CORRECTION => FALSE,
      PHASE_SHIFT => 0,
      STARTUP_WAIT => TRUE)
    port map (
      CLKIN => user_CLKIN,
      CLKFB => user_CLKFB,
      RST => user_RST,
      PSEN => user_PSEN,
      PSINCDEC => user_PSINCDEC,
      PSCLK => user_PSCLK,
      DSSEN => user_DSSEN,
      CLKFX => user_CLKFX,
      LOCKED => user_LOCKED);

  user_CLKIN <= clkin;
  clkfxout <= clkfx;

  user_CLKFB <= '0';
  user_DSSEN <= '0';
  user_PSCLK <= '0';
  user_PSEN <= '0';
  user_PSINCDEC <= '0';
  user_RST <= '0';

end rtl;
  
