---------------------------------------------------------------------------
--
--  File        : log_ram_v2p.vhd
--
--  Title       : Event Receiver Event Log RAM for ECP2M
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : 80 bits x 512 memory
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity event_log_ram is
  port (
      bram_data_B_out : out std_logic_vector(79 downto 0);
      bram_addr_A     : in  std_logic_vector(8 downto 0);
      bram_addr_B     : in  std_logic_vector(8 downto 0);
      CLKA            : in  std_logic;
      CLKB            : in  std_logic;
      bram_data_A_in  : in  std_logic_vector(79 downto 0);
      WEA             : in  std_logic);
end event_log_ram;

architecture rtl of event_log_ram is

  signal gnd1            : std_logic;
  signal vcc1            : std_logic;
  signal gnd32           : std_logic_vector(31 downto 0);

  -- internal signal declarations
  signal scuba_vlo: std_logic;
  signal scuba_vhi: std_logic;
  
  -- local component declarations
  component VHI
    port (Z: out  std_logic);
  end component;
  component VLO
    port (Z: out  std_logic);
  end component;
  
  component DP16KB
    -- synopsys translate_off
    generic (INITVAL_3F : in String;
	     INITVAL_3E : in String; 
	     INITVAL_3D : in String;
	     INITVAL_3C : in String; 
	     INITVAL_3B : in String;
	     INITVAL_3A : in String; 
                INITVAL_39 : in String;
		INITVAL_38 : in String; 
                INITVAL_37 : in String;
		INITVAL_36 : in String; 
                INITVAL_35 : in String;
		INITVAL_34 : in String; 
                INITVAL_33 : in String;
		INITVAL_32 : in String; 
                INITVAL_31 : in String;
		INITVAL_30 : in String; 
                INITVAL_2F : in String;
		INITVAL_2E : in String; 
                INITVAL_2D : in String;
		INITVAL_2C : in String; 
                INITVAL_2B : in String;
		INITVAL_2A : in String; 
                INITVAL_29 : in String;
		INITVAL_28 : in String; 
                INITVAL_27 : in String;
		INITVAL_26 : in String; 
                INITVAL_25 : in String;
		INITVAL_24 : in String; 
                INITVAL_23 : in String;
		INITVAL_22 : in String; 
                INITVAL_21 : in String;
		INITVAL_20 : in String; 
                INITVAL_1F : in String;
		INITVAL_1E : in String; 
                INITVAL_1D : in String;
		INITVAL_1C : in String; 
                INITVAL_1B : in String;
		INITVAL_1A : in String; 
                INITVAL_19 : in String;
		INITVAL_18 : in String; 
                INITVAL_17 : in String;
		INITVAL_16 : in String; 
                INITVAL_15 : in String;
		INITVAL_14 : in String; 
                INITVAL_13 : in String;
		INITVAL_12 : in String; 
                INITVAL_11 : in String;
		INITVAL_10 : in String; 
                INITVAL_0F : in String;
		INITVAL_0E : in String; 
                INITVAL_0D : in String;
		INITVAL_0C : in String; 
                INITVAL_0B : in String;
		INITVAL_0A : in String; 
                INITVAL_09 : in String;
		INITVAL_08 : in String; 
                INITVAL_07 : in String;
		INITVAL_06 : in String; 
                INITVAL_05 : in String;
		INITVAL_04 : in String; 
                INITVAL_03 : in String;
		INITVAL_02 : in String; 
                INITVAL_01 : in String;
		INITVAL_00 : in String; 
                GSR : in String;
		WRITEMODE_B : in String; 
                CSDECODE_B : in std_logic_vector(2 downto 0); 
                CSDECODE_A : in std_logic_vector(2 downto 0); 
                WRITEMODE_A : in String;
		RESETMODE : in String; 
                REGMODE_B : in String;
		REGMODE_A : in String; 
                DATA_WIDTH_B : in Integer;
		DATA_WIDTH_A : in Integer);
    -- synopsys translate_on
        port (DIA0: in  std_logic;
	      DIA1: in  std_logic; 
            DIA2: in  std_logic;
	    DIA3: in  std_logic; 
            DIA4: in  std_logic;
	    DIA5: in  std_logic; 
            DIA6: in  std_logic;
	    DIA7: in  std_logic; 
            DIA8: in  std_logic;
	    DIA9: in  std_logic; 
            DIA10: in  std_logic;
	    DIA11: in  std_logic; 
            DIA12: in  std_logic;
	    DIA13: in  std_logic; 
            DIA14: in  std_logic;
	    DIA15: in  std_logic; 
            DIA16: in  std_logic;
	    DIA17: in  std_logic; 
            ADA0: in  std_logic;
	    ADA1: in  std_logic; 
            ADA2: in  std_logic;
	    ADA3: in  std_logic; 
            ADA4: in  std_logic;
	    ADA5: in  std_logic; 
            ADA6: in  std_logic;
	    ADA7: in  std_logic; 
            ADA8: in  std_logic;
	    ADA9: in  std_logic; 
            ADA10: in  std_logic;
	    ADA11: in  std_logic; 
            ADA12: in  std_logic;
	    ADA13: in  std_logic; 
            CEA: in  std_logic;
	    CLKA: in  std_logic;
	    WEA: in  std_logic; 
            CSA0: in  std_logic;
	    CSA1: in  std_logic; 
            CSA2: in  std_logic;
	    RSTA: in  std_logic; 
            DIB0: in  std_logic;
	    DIB1: in  std_logic; 
            DIB2: in  std_logic;
	    DIB3: in  std_logic; 
            DIB4: in  std_logic;
	    DIB5: in  std_logic; 
            DIB6: in  std_logic;
	    DIB7: in  std_logic; 
            DIB8: in  std_logic;
	    DIB9: in  std_logic; 
            DIB10: in  std_logic;
	    DIB11: in  std_logic; 
            DIB12: in  std_logic;
	    DIB13: in  std_logic; 
            DIB14: in  std_logic;
	    DIB15: in  std_logic; 
            DIB16: in  std_logic;
	    DIB17: in  std_logic; 
            ADB0: in  std_logic;
	    ADB1: in  std_logic; 
            ADB2: in  std_logic;
	    ADB3: in  std_logic; 
            ADB4: in  std_logic;
	    ADB5: in  std_logic; 
            ADB6: in  std_logic;
	    ADB7: in  std_logic; 
            ADB8: in  std_logic;
	    ADB9: in  std_logic; 
            ADB10: in  std_logic;
	    ADB11: in  std_logic; 
            ADB12: in  std_logic;
	    ADB13: in  std_logic; 
            CEB: in  std_logic;
	    CLKB: in  std_logic;
	    WEB: in  std_logic; 
            CSB0: in  std_logic;
	    CSB1: in  std_logic; 
            CSB2: in  std_logic;
	    RSTB: in  std_logic; 
            DOA0: out  std_logic;
	    DOA1: out  std_logic; 
            DOA2: out  std_logic;
	    DOA3: out  std_logic; 
            DOA4: out  std_logic;
	    DOA5: out  std_logic; 
            DOA6: out  std_logic;
	    DOA7: out  std_logic; 
            DOA8: out  std_logic;
	    DOA9: out  std_logic; 
            DOA10: out  std_logic;
	    DOA11: out  std_logic; 
            DOA12: out  std_logic;
	    DOA13: out  std_logic; 
            DOA14: out  std_logic;
	    DOA15: out  std_logic; 
            DOA16: out  std_logic;
	    DOA17: out  std_logic; 
            DOB0: out  std_logic;
	    DOB1: out  std_logic; 
            DOB2: out  std_logic;
	    DOB3: out  std_logic; 
            DOB4: out  std_logic;
	    DOB5: out  std_logic; 
            DOB6: out  std_logic;
	    DOB7: out  std_logic; 
            DOB8: out  std_logic;
	    DOB9: out  std_logic; 
            DOB10: out  std_logic;
	    DOB11: out  std_logic; 
            DOB12: out  std_logic;
	    DOB13: out  std_logic; 
            DOB14: out  std_logic;
	    DOB15: out  std_logic; 
            DOB16: out  std_logic;
	    DOB17: out  std_logic);
    end component;
  
begin

  DP16KB_4: DP16KB
    -- synopsys translate_off
    generic map (
      INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_07=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      CSDECODE_B=> "000",
      CSDECODE_A=> "000",
      WRITEMODE_B=> "NORMAL", 
      WRITEMODE_A=> "NORMAL",
      GSR=> "DISABLED",
      RESETMODE=> "SYNC", 
      REGMODE_B=> "OUTREG",
      REGMODE_A=> "OUTREG",
      DATA_WIDTH_B=>  18, 
      DATA_WIDTH_A=>  18)
    -- synopsys translate_on
    port map (
      DIA0 => bram_data_A_in(64),
      DIA1 => bram_data_A_in(65),
      DIA2 => bram_data_A_in(66),
      DIA3 => bram_data_A_in(67),
      DIA4 => bram_data_A_in(68),
      DIA5 => bram_data_A_in(69),
      DIA6 => bram_data_A_in(70),
      DIA7 => bram_data_A_in(71),
      DIA8 => bram_data_A_in(72),
      DIA9 => bram_data_A_in(73),
      DIA10 => bram_data_A_in(74),
      DIA11 => bram_data_A_in(75),
      DIA12 => bram_data_A_in(76),
      DIA13 => bram_data_A_in(77),
      DIA14 => bram_data_A_in(78),
      DIA15 => bram_data_A_in(79),
      DIA16 => gnd1,
      DIA17 => gnd1,
      ADA0 => scuba_vhi,
      ADA1 => scuba_vhi,
      ADA2 => scuba_vlo, 
      ADA3 => scuba_vlo,
      ADA4 => bram_addr_A(0),
      ADA5 => bram_addr_A(1),
      ADA6 => bram_addr_A(2),
      ADA7 => bram_addr_A(3),
      ADA8 => bram_addr_A(4),
      ADA9 => bram_addr_A(5),
      ADA10 => bram_addr_A(6),
      ADA11 => bram_addr_A(7),
      ADA12 => bram_addr_A(8),
      ADA13 => scuba_vlo,
      CEA => vcc1,
      CLKA => CLKA,
      WEA => WEA,
      CSA0 => scuba_vlo,
      CSA1 => scuba_vlo, 
      CSA2 => scuba_vlo,
      RSTA => gnd1,
      DIB0 => gnd1,
      DIB1 => gnd1,
      DIB2 => gnd1,
      DIB3 => gnd1,
      DIB4 => gnd1,
      DIB5 => gnd1,
      DIB6 => gnd1,
      DIB7 => gnd1,
      DIB8 => gnd1,
      DIB9 => gnd1,
      DIB10 => gnd1,
      DIB11 => gnd1,
      DIB12 => gnd1,
      DIB13 => gnd1,
      DIB14 => gnd1,
      DIB15 => gnd1,
      DIB16 => gnd1,
      DIB17 => gnd1,
      ADB0 => scuba_vhi, 
      ADB1 => scuba_vhi,
      ADB2 => scuba_vlo, 
      ADB3 => scuba_vlo,
      ADB4 => bram_addr_B(0),
      ADB5 => bram_addr_B(1),
      ADB6 => bram_addr_B(2),
      ADB7 => bram_addr_B(3),
      ADB8 => bram_addr_B(4),
      ADB9 => bram_addr_B(5),
      ADB10 => bram_addr_B(6),
      ADB11 => bram_addr_B(7),
      ADB12 => bram_addr_B(8),
      ADB13 => scuba_vlo,
      CEB => vcc1,
      CLKB => CLKB,
      WEB => gnd1,
      CSB0 => scuba_vlo,
      CSB1 => scuba_vlo,
      CSB2 => scuba_vlo, 
      RSTB => gnd1,
      DOA0 => open,
      DOA1 => open,
      DOA2 => open,
      DOA3 => open,
      DOA4 => open,
      DOA5 => open,
      DOA6 => open,
      DOA7 => open,
      DOA8 => open,
      DOA9 => open,
      DOA10 => open,
      DOA11 => open,
      DOA12 => open,
      DOA13 => open,
      DOA14 => open,
      DOA15 => open,
      DOA16 => open,
      DOA17 => open,
      DOB0 => bram_data_B_out(64), 
      DOB1 => bram_data_B_out(65), 
      DOB2 => bram_data_B_out(66), 
      DOB3 => bram_data_B_out(67), 
      DOB4 => bram_data_B_out(68), 
      DOB5 => bram_data_B_out(69), 
      DOB6 => bram_data_B_out(70), 
      DOB7 => bram_data_B_out(71), 
      DOB8 => bram_data_B_out(72), 
      DOB9 => bram_data_B_out(73), 
      DOB10 => bram_data_B_out(74), 
      DOB11 => bram_data_B_out(75), 
      DOB12 => bram_data_B_out(76), 
      DOB13 => bram_data_B_out(77), 
      DOB14 => bram_data_B_out(78), 
      DOB15 => bram_data_B_out(79), 
      DOB16 => open, 
      DOB17 => open);
  
  DP16KB_3: DP16KB
    -- synopsys translate_off
    generic map (
      INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_07=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      CSDECODE_B=> "000",
      CSDECODE_A=> "000",
      WRITEMODE_B=> "NORMAL", 
      WRITEMODE_A=> "NORMAL",
      GSR=> "DISABLED",
      RESETMODE=> "SYNC", 
      REGMODE_B=> "OUTREG",
      REGMODE_A=> "OUTREG",
      DATA_WIDTH_B=>  18, 
      DATA_WIDTH_A=>  18)
    -- synopsys translate_on
    port map (
      DIA0 => bram_data_A_in(48),
      DIA1 => bram_data_A_in(49),
      DIA2 => bram_data_A_in(50),
      DIA3 => bram_data_A_in(51),
      DIA4 => bram_data_A_in(52),
      DIA5 => bram_data_A_in(53),
      DIA6 => bram_data_A_in(54),
      DIA7 => bram_data_A_in(55),
      DIA8 => bram_data_A_in(56),
      DIA9 => bram_data_A_in(57),
      DIA10 => bram_data_A_in(58),
      DIA11 => bram_data_A_in(59),
      DIA12 => bram_data_A_in(60),
      DIA13 => bram_data_A_in(61),
      DIA14 => bram_data_A_in(62),
      DIA15 => bram_data_A_in(63),
      DIA16 => gnd1,
      DIA17 => gnd1,
      ADA0 => scuba_vhi,
      ADA1 => scuba_vhi,
      ADA2 => scuba_vlo, 
      ADA3 => scuba_vlo,
      ADA4 => bram_addr_A(0),
      ADA5 => bram_addr_A(1),
      ADA6 => bram_addr_A(2),
      ADA7 => bram_addr_A(3),
      ADA8 => bram_addr_A(4),
      ADA9 => bram_addr_A(5),
      ADA10 => bram_addr_A(6),
      ADA11 => bram_addr_A(7),
      ADA12 => bram_addr_A(8),
      ADA13 => scuba_vlo,
      CEA => vcc1,
      CLKA => CLKA,
      WEA => WEA,
      CSA0 => scuba_vlo,
      CSA1 => scuba_vlo, 
      CSA2 => scuba_vlo,
      RSTA => gnd1,
      DIB0 => gnd1,
      DIB1 => gnd1,
      DIB2 => gnd1,
      DIB3 => gnd1,
      DIB4 => gnd1,
      DIB5 => gnd1,
      DIB6 => gnd1,
      DIB7 => gnd1,
      DIB8 => gnd1,
      DIB9 => gnd1,
      DIB10 => gnd1,
      DIB11 => gnd1,
      DIB12 => gnd1,
      DIB13 => gnd1,
      DIB14 => gnd1,
      DIB15 => gnd1,
      DIB16 => gnd1,
      DIB17 => gnd1,
      ADB0 => scuba_vhi, 
      ADB1 => scuba_vhi,
      ADB2 => scuba_vlo, 
      ADB3 => scuba_vlo,
      ADB4 => bram_addr_B(0),
      ADB5 => bram_addr_B(1),
      ADB6 => bram_addr_B(2),
      ADB7 => bram_addr_B(3),
      ADB8 => bram_addr_B(4),
      ADB9 => bram_addr_B(5),
      ADB10 => bram_addr_B(6),
      ADB11 => bram_addr_B(7),
      ADB12 => bram_addr_B(8),
      ADB13 => scuba_vlo,
      CEB => vcc1,
      CLKB => CLKB,
      WEB => gnd1,
      CSB0 => scuba_vlo,
      CSB1 => scuba_vlo,
      CSB2 => scuba_vlo, 
      RSTB => gnd1,
      DOA0 => open,
      DOA1 => open,
      DOA2 => open,
      DOA3 => open,
      DOA4 => open,
      DOA5 => open,
      DOA6 => open,
      DOA7 => open,
      DOA8 => open,
      DOA9 => open,
      DOA10 => open,
      DOA11 => open,
      DOA12 => open,
      DOA13 => open,
      DOA14 => open,
      DOA15 => open,
      DOA16 => open,
      DOA17 => open,
      DOB0 => bram_data_B_out(48), 
      DOB1 => bram_data_B_out(49), 
      DOB2 => bram_data_B_out(50), 
      DOB3 => bram_data_B_out(51), 
      DOB4 => bram_data_B_out(52), 
      DOB5 => bram_data_B_out(53), 
      DOB6 => bram_data_B_out(54), 
      DOB7 => bram_data_B_out(55), 
      DOB8 => bram_data_B_out(56), 
      DOB9 => bram_data_B_out(57), 
      DOB10 => bram_data_B_out(58), 
      DOB11 => bram_data_B_out(59), 
      DOB12 => bram_data_B_out(60), 
      DOB13 => bram_data_B_out(61), 
      DOB14 => bram_data_B_out(62), 
      DOB15 => bram_data_B_out(63), 
      DOB16 => open, 
      DOB17 => open);
  
  DP16KB_2: DP16KB
    -- synopsys translate_off
    generic map (
      INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_07=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      CSDECODE_B=> "000",
      CSDECODE_A=> "000",
      WRITEMODE_B=> "NORMAL", 
      WRITEMODE_A=> "NORMAL",
      GSR=> "DISABLED",
      RESETMODE=> "SYNC", 
      REGMODE_B=> "OUTREG",
      REGMODE_A=> "OUTREG",
      DATA_WIDTH_B=>  18, 
      DATA_WIDTH_A=>  18)
    -- synopsys translate_on
    port map (
      DIA0 => bram_data_A_in(32),
      DIA1 => bram_data_A_in(33),
      DIA2 => bram_data_A_in(34),
      DIA3 => bram_data_A_in(35),
      DIA4 => bram_data_A_in(36),
      DIA5 => bram_data_A_in(37),
      DIA6 => bram_data_A_in(38),
      DIA7 => bram_data_A_in(39),
      DIA8 => bram_data_A_in(40),
      DIA9 => bram_data_A_in(41),
      DIA10 => bram_data_A_in(42),
      DIA11 => bram_data_A_in(43),
      DIA12 => bram_data_A_in(44),
      DIA13 => bram_data_A_in(45),
      DIA14 => bram_data_A_in(46),
      DIA15 => bram_data_A_in(47),
      DIA16 => gnd1,
      DIA17 => gnd1,
      ADA0 => scuba_vhi,
      ADA1 => scuba_vhi,
      ADA2 => scuba_vlo, 
      ADA3 => scuba_vlo,
      ADA4 => bram_addr_A(0),
      ADA5 => bram_addr_A(1),
      ADA6 => bram_addr_A(2),
      ADA7 => bram_addr_A(3),
      ADA8 => bram_addr_A(4),
      ADA9 => bram_addr_A(5),
      ADA10 => bram_addr_A(6),
      ADA11 => bram_addr_A(7),
      ADA12 => bram_addr_A(8),
      ADA13 => scuba_vlo,
      CEA => vcc1,
      CLKA => CLKA,
      WEA => WEA,
      CSA0 => scuba_vlo,
      CSA1 => scuba_vlo, 
      CSA2 => scuba_vlo,
      RSTA => gnd1,
      DIB0 => gnd1,
      DIB1 => gnd1,
      DIB2 => gnd1,
      DIB3 => gnd1,
      DIB4 => gnd1,
      DIB5 => gnd1,
      DIB6 => gnd1,
      DIB7 => gnd1,
      DIB8 => gnd1,
      DIB9 => gnd1,
      DIB10 => gnd1,
      DIB11 => gnd1,
      DIB12 => gnd1,
      DIB13 => gnd1,
      DIB14 => gnd1,
      DIB15 => gnd1,
      DIB16 => gnd1,
      DIB17 => gnd1,
      ADB0 => scuba_vhi, 
      ADB1 => scuba_vhi,
      ADB2 => scuba_vlo, 
      ADB3 => scuba_vlo,
      ADB4 => bram_addr_B(0),
      ADB5 => bram_addr_B(1),
      ADB6 => bram_addr_B(2),
      ADB7 => bram_addr_B(3),
      ADB8 => bram_addr_B(4),
      ADB9 => bram_addr_B(5),
      ADB10 => bram_addr_B(6),
      ADB11 => bram_addr_B(7),
      ADB12 => bram_addr_B(8),
      ADB13 => scuba_vlo,
      CEB => vcc1,
      CLKB => CLKB,
      WEB => gnd1,
      CSB0 => scuba_vlo,
      CSB1 => scuba_vlo,
      CSB2 => scuba_vlo, 
      RSTB => gnd1,
      DOA0 => open,
      DOA1 => open,
      DOA2 => open,
      DOA3 => open,
      DOA4 => open,
      DOA5 => open,
      DOA6 => open,
      DOA7 => open,
      DOA8 => open,
      DOA9 => open,
      DOA10 => open,
      DOA11 => open,
      DOA12 => open,
      DOA13 => open,
      DOA14 => open,
      DOA15 => open,
      DOA16 => open,
      DOA17 => open,
      DOB0 => bram_data_B_out(32), 
      DOB1 => bram_data_B_out(33), 
      DOB2 => bram_data_B_out(34), 
      DOB3 => bram_data_B_out(35), 
      DOB4 => bram_data_B_out(36), 
      DOB5 => bram_data_B_out(37), 
      DOB6 => bram_data_B_out(38), 
      DOB7 => bram_data_B_out(39), 
      DOB8 => bram_data_B_out(40), 
      DOB9 => bram_data_B_out(41), 
      DOB10 => bram_data_B_out(42), 
      DOB11 => bram_data_B_out(43), 
      DOB12 => bram_data_B_out(44), 
      DOB13 => bram_data_B_out(45), 
      DOB14 => bram_data_B_out(46), 
      DOB15 => bram_data_B_out(47), 
      DOB16 => open, 
      DOB17 => open);
  
  DP16KB_1: DP16KB
    -- synopsys translate_off
    generic map (
      INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_07=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      CSDECODE_B=> "000",
      CSDECODE_A=> "000",
      WRITEMODE_B=> "NORMAL", 
      WRITEMODE_A=> "NORMAL",
      GSR=> "DISABLED",
      RESETMODE=> "SYNC", 
      REGMODE_B=> "OUTREG",
      REGMODE_A=> "OUTREG",
      DATA_WIDTH_B=>  18, 
      DATA_WIDTH_A=>  18)
    -- synopsys translate_on
    port map (
      DIA0 => bram_data_A_in(16),
      DIA1 => bram_data_A_in(17),
      DIA2 => bram_data_A_in(18),
      DIA3 => bram_data_A_in(19),
      DIA4 => bram_data_A_in(20),
      DIA5 => bram_data_A_in(21),
      DIA6 => bram_data_A_in(22),
      DIA7 => bram_data_A_in(23),
      DIA8 => bram_data_A_in(24),
      DIA9 => bram_data_A_in(25),
      DIA10 => bram_data_A_in(26),
      DIA11 => bram_data_A_in(27),
      DIA12 => bram_data_A_in(28),
      DIA13 => bram_data_A_in(29),
      DIA14 => bram_data_A_in(30),
      DIA15 => bram_data_A_in(31),
      DIA16 => gnd1,
      DIA17 => gnd1,
      ADA0 => scuba_vhi,
      ADA1 => scuba_vhi,
      ADA2 => scuba_vlo, 
      ADA3 => scuba_vlo,
      ADA4 => bram_addr_A(0),
      ADA5 => bram_addr_A(1),
      ADA6 => bram_addr_A(2),
      ADA7 => bram_addr_A(3),
      ADA8 => bram_addr_A(4),
      ADA9 => bram_addr_A(5),
      ADA10 => bram_addr_A(6),
      ADA11 => bram_addr_A(7),
      ADA12 => bram_addr_A(8),
      ADA13 => scuba_vlo,
      CEA => vcc1,
      CLKA => CLKA,
      WEA => WEA,
      CSA0 => scuba_vlo,
      CSA1 => scuba_vlo, 
      CSA2 => scuba_vlo,
      RSTA => gnd1,
      DIB0 => gnd1,
      DIB1 => gnd1,
      DIB2 => gnd1,
      DIB3 => gnd1,
      DIB4 => gnd1,
      DIB5 => gnd1,
      DIB6 => gnd1,
      DIB7 => gnd1,
      DIB8 => gnd1,
      DIB9 => gnd1,
      DIB10 => gnd1,
      DIB11 => gnd1,
      DIB12 => gnd1,
      DIB13 => gnd1,
      DIB14 => gnd1,
      DIB15 => gnd1,
      DIB16 => gnd1,
      DIB17 => gnd1,
      ADB0 => scuba_vhi, 
      ADB1 => scuba_vhi,
      ADB2 => scuba_vlo, 
      ADB3 => scuba_vlo,
      ADB4 => bram_addr_B(0),
      ADB5 => bram_addr_B(1),
      ADB6 => bram_addr_B(2),
      ADB7 => bram_addr_B(3),
      ADB8 => bram_addr_B(4),
      ADB9 => bram_addr_B(5),
      ADB10 => bram_addr_B(6),
      ADB11 => bram_addr_B(7),
      ADB12 => bram_addr_B(8),
      ADB13 => scuba_vlo,
      CEB => vcc1,
      CLKB => CLKB,
      WEB => gnd1,
      CSB0 => scuba_vlo,
      CSB1 => scuba_vlo,
      CSB2 => scuba_vlo, 
      RSTB => gnd1,
      DOA0 => open,
      DOA1 => open,
      DOA2 => open,
      DOA3 => open,
      DOA4 => open,
      DOA5 => open,
      DOA6 => open,
      DOA7 => open,
      DOA8 => open,
      DOA9 => open,
      DOA10 => open,
      DOA11 => open,
      DOA12 => open,
      DOA13 => open,
      DOA14 => open,
      DOA15 => open,
      DOA16 => open,
      DOA17 => open,
      DOB0 => bram_data_B_out(16), 
      DOB1 => bram_data_B_out(17), 
      DOB2 => bram_data_B_out(18), 
      DOB3 => bram_data_B_out(19), 
      DOB4 => bram_data_B_out(20), 
      DOB5 => bram_data_B_out(21), 
      DOB6 => bram_data_B_out(22), 
      DOB7 => bram_data_B_out(23), 
      DOB8 => bram_data_B_out(24), 
      DOB9 => bram_data_B_out(25), 
      DOB10 => bram_data_B_out(26), 
      DOB11 => bram_data_B_out(27), 
      DOB12 => bram_data_B_out(28), 
      DOB13 => bram_data_B_out(29), 
      DOB14 => bram_data_B_out(30), 
      DOB15 => bram_data_B_out(31), 
      DOB16 => open, 
      DOB17 => open);
  
  DP16KB_0: DP16KB
    -- synopsys translate_off
    generic map (
      INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_07=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      CSDECODE_B=> "000",
      CSDECODE_A=> "000",
      WRITEMODE_B=> "NORMAL", 
      WRITEMODE_A=> "NORMAL",
      GSR=> "DISABLED",
      RESETMODE=> "SYNC", 
      REGMODE_B=> "OUTREG",
      REGMODE_A=> "OUTREG",
      DATA_WIDTH_B=>  18, 
      DATA_WIDTH_A=>  18)
    -- synopsys translate_on
    port map (
      DIA0 => bram_data_A_in(0),
      DIA1 => bram_data_A_in(1),
      DIA2 => bram_data_A_in(2),
      DIA3 => bram_data_A_in(3),
      DIA4 => bram_data_A_in(4),
      DIA5 => bram_data_A_in(5),
      DIA6 => bram_data_A_in(6),
      DIA7 => bram_data_A_in(7),
      DIA8 => bram_data_A_in(8),
      DIA9 => bram_data_A_in(9),
      DIA10 => bram_data_A_in(10),
      DIA11 => bram_data_A_in(11),
      DIA12 => bram_data_A_in(12),
      DIA13 => bram_data_A_in(13),
      DIA14 => bram_data_A_in(14),
      DIA15 => bram_data_A_in(15),
      DIA16 => gnd1,
      DIA17 => gnd1,
      ADA0 => scuba_vhi,
      ADA1 => scuba_vhi,
      ADA2 => scuba_vlo, 
      ADA3 => scuba_vlo,
      ADA4 => bram_addr_A(0),
      ADA5 => bram_addr_A(1),
      ADA6 => bram_addr_A(2),
      ADA7 => bram_addr_A(3),
      ADA8 => bram_addr_A(4),
      ADA9 => bram_addr_A(5),
      ADA10 => bram_addr_A(6),
      ADA11 => bram_addr_A(7),
      ADA12 => bram_addr_A(8),
      ADA13 => scuba_vlo,
      CEA => vcc1,
      CLKA => CLKA,
      WEA => WEA,
      CSA0 => scuba_vlo,
      CSA1 => scuba_vlo, 
      CSA2 => scuba_vlo,
      RSTA => gnd1,
      DIB0 => gnd1,
      DIB1 => gnd1,
      DIB2 => gnd1,
      DIB3 => gnd1,
      DIB4 => gnd1,
      DIB5 => gnd1,
      DIB6 => gnd1,
      DIB7 => gnd1,
      DIB8 => gnd1,
      DIB9 => gnd1,
      DIB10 => gnd1,
      DIB11 => gnd1,
      DIB12 => gnd1,
      DIB13 => gnd1,
      DIB14 => gnd1,
      DIB15 => gnd1,
      DIB16 => gnd1,
      DIB17 => gnd1,
      ADB0 => scuba_vhi, 
      ADB1 => scuba_vhi,
      ADB2 => scuba_vlo, 
      ADB3 => scuba_vlo,
      ADB4 => bram_addr_B(0),
      ADB5 => bram_addr_B(1),
      ADB6 => bram_addr_B(2),
      ADB7 => bram_addr_B(3),
      ADB8 => bram_addr_B(4),
      ADB9 => bram_addr_B(5),
      ADB10 => bram_addr_B(6),
      ADB11 => bram_addr_B(7),
      ADB12 => bram_addr_B(8),
      ADB13 => scuba_vlo,
      CEB => vcc1,
      CLKB => CLKB,
      WEB => gnd1,
      CSB0 => scuba_vlo,
      CSB1 => scuba_vlo,
      CSB2 => scuba_vlo, 
      RSTB => gnd1,
      DOA0 => open,
      DOA1 => open,
      DOA2 => open,
      DOA3 => open,
      DOA4 => open,
      DOA5 => open,
      DOA6 => open,
      DOA7 => open,
      DOA8 => open,
      DOA9 => open,
      DOA10 => open,
      DOA11 => open,
      DOA12 => open,
      DOA13 => open,
      DOA14 => open,
      DOA15 => open,
      DOA16 => open,
      DOA17 => open,
      DOB0 => bram_data_B_out(0), 
      DOB1 => bram_data_B_out(1), 
      DOB2 => bram_data_B_out(2), 
      DOB3 => bram_data_B_out(3), 
      DOB4 => bram_data_B_out(4), 
      DOB5 => bram_data_B_out(5), 
      DOB6 => bram_data_B_out(6), 
      DOB7 => bram_data_B_out(7), 
      DOB8 => bram_data_B_out(8), 
      DOB9 => bram_data_B_out(9), 
      DOB10 => bram_data_B_out(10), 
      DOB11 => bram_data_B_out(11), 
      DOB12 => bram_data_B_out(12), 
      DOB13 => bram_data_B_out(13), 
      DOB14 => bram_data_B_out(14), 
      DOB15 => bram_data_B_out(15), 
      DOB16 => open, 
      DOB17 => open);
  
  scuba_vlo_inst: VLO
    port map (Z=>scuba_vlo);

  scuba_vhi_inst: VHI
    port map (Z=>scuba_vhi);


  gnd1 <= '0';
  vcc1 <= '1';
  gnd32 <= (others => '0');

end rtl;

-- synopsys translate_off
library ecp2m;
configuration Structure_CON of RAMB16_S36_S36 is
    for Structure
        for all:VHI use entity ecp2m.VHI(V); end for;
        for all:VLO use entity ecp2m.VLO(V); end for;
        for all:DP16KB use entity ecp2m.DP16KB(V); end for;
    end for;
end Structure_CON;

-- synopsys translate_on
