---------------------------------------------------------------------------
--
--  File        : log_ram_v5.vhd
--
--  Title       : Event Receiver Event Log RAM for Virtex 5
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : 80 bits x 512 memory
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity event_log_ram is
  port (
      bram_data_B_out : out std_logic_vector(79 downto 0);
      bram_addr_A     : in  std_logic_vector(8 downto 0);
      bram_addr_B     : in  std_logic_vector(8 downto 0);
      CLKA            : in  std_logic;
      CLKB            : in  std_logic;
      bram_data_A_in  : in  std_logic_vector(79 downto 0);
      WEA             : in  std_logic);
end event_log_ram;

architecture rtl of event_log_ram is

  signal gnd1            : std_logic;
  signal vcc1            : std_logic;
  signal gnd32           : std_logic_vector(31 downto 0);

begin

  i_ram0: RAMB36
    generic map (
      DOA_REG => 0,
      DOB_REG => 0,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 36,
      READ_WIDTH_B => 36,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 36,
      WRITE_WIDTH_B => 36)
    port map (
      DOA => open,
      DOB => bram_data_B_out(79 downto 48),
      DOPA => open,
      DOPB => open,
      ADDRA(15 downto 14) => gnd32(1 downto 0),
      ADDRA(13 downto 5) => bram_addr_A,
      ADDRA(4 downto 0) => gnd32(4 downto 0),
      ADDRB(15 downto 14) => gnd32(1 downto 0),
      ADDRB(13 downto 5) => bram_addr_B,
      ADDRB(4 downto 0) => gnd32(4 downto 0),
      CLKA => CLKA,
      CLKB => CLKB,
      DIA => bram_data_A_in(79 downto 48),
      DIB => gnd32,
      DIPA => gnd32(3 downto 0),
      DIPB => gnd32(3 downto 0),
      ENA => vcc1,
      ENB => vcc1,
      SSRA => gnd1,
      SSRB => gnd1,
      WEA(0) => WEA,
      WEA(1) => WEA,
      WEA(2) => WEA,
      WEA(3) => WEA,
      WEB => gnd32(3 downto 0));

  i_ram1: RAMB36
    generic map (
      DOA_REG => 0,
      DOB_REG => 0,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 36,
      READ_WIDTH_B => 36,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 36,
      WRITE_WIDTH_B => 36)
    port map (
      DOA => open,
      DOB => bram_data_B_out(47 downto 16),
      DOPA => open,
      DOPB => open,
      ADDRA(15 downto 14) => gnd32(1 downto 0),
      ADDRA(13 downto 5) => bram_addr_A,
      ADDRA(4 downto 0) => gnd32(4 downto 0),
      ADDRB(15 downto 14) => gnd32(1 downto 0),
      ADDRB(13 downto 5) => bram_addr_B,
      ADDRB(4 downto 0) => gnd32(4 downto 0),
      CLKA => CLKA,
      CLKB => CLKB,
      DIA => bram_data_A_in(47 downto 16),
      DIB => gnd32,
      DIPA => gnd32(3 downto 0),
      DIPB => gnd32(3 downto 0),
      ENA => vcc1,
      ENB => vcc1,
      SSRA => gnd1,
      SSRB => gnd1,
      WEA(0) => WEA,
      WEA(1) => WEA,
      WEA(2) => WEA,
      WEA(3) => WEA,
      WEB => gnd32(3 downto 0));

  i_ram2: RAMB36
    generic map (
      DOA_REG => 0,
      DOB_REG => 0,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 36,
      READ_WIDTH_B => 36,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 36,
      WRITE_WIDTH_B => 36)
    port map (
      DOA => open,
      DOB(31 downto 16) => open,
      DOB(15 downto 0) => bram_data_B_out(15 downto 0),
      DOPA => open,
      DOPB => open,
      ADDRA(15 downto 14) => gnd32(1 downto 0),
      ADDRA(13 downto 5) => bram_addr_A,
      ADDRA(4 downto 0) => gnd32(4 downto 0),
      ADDRB(15 downto 14) => gnd32(1 downto 0),
      ADDRB(13 downto 5) => bram_addr_B,
      ADDRB(4 downto 0) => gnd32(4 downto 0),
      CLKA => CLKA,
      CLKB => CLKB,
      DIA(31 downto 16) => gnd32(15 downto 0),
      DIA(15 downto 0) => bram_data_A_in(15 downto 0),
      DIB => gnd32,
      DIPA => gnd32(3 downto 0),
      DIPB => gnd32(3 downto 0),
      ENA => vcc1,
      ENB => vcc1,
      SSRA => gnd1,
      SSRB => gnd1,
      WEA(0) => WEA,
      WEA(1) => WEA,
      WEA(2) => WEA,
      WEA(3) => WEA,
      WEB => gnd32(3 downto 0));

  gnd1 <= '0';
  vcc1 <= '1';
  gnd32 <= (others => '0');

end rtl;
