---------------------------------------------------------------------------
--
--  File        : map_ram_ecp2m.vhd
--
--  Title       : Event receiver mapping RAM
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity map_ram is
  generic (
    map_code_abits : integer);
  port (
    bram_addr_A     : in    std_logic_vector(8 downto 0);
    bram_addr_B     : in    std_logic_vector(8 downto 0);    
    clk             : in    std_logic;
    event_clk       : in    std_logic;
    bram_data_A_in  : in    std_logic_vector(31 downto 0); 
    bram_data_A_out : out   std_logic_vector(8*(2**map_code_abits) - 1 downto 0);
    bram_we         : in    std_logic_vector(2**(map_code_abits-2) - 1 downto 0);
    event_map       : out   std_logic_vector(8*(2**map_code_abits) - 1 downto 0));
end map_ram;

architecture structure of map_ram is

  signal gnd1            : std_logic;
  signal vcc1            : std_logic;
  signal gnd32           : std_logic_vector(31 downto 0);

  -- internal signal declarations
  signal scuba_vlo: std_logic;
  signal scuba_vhi: std_logic;
  
  -- local component declarations
  component VHI
    port (Z: out  std_logic);
  end component;
  component VLO
    port (Z: out  std_logic);
  end component;
  
  component DP16KB
    -- synopsys translate_off
    generic (
      INITVAL_3F : in String;
      INITVAL_3E : in String; 
      INITVAL_3D : in String;
      INITVAL_3C : in String; 
      INITVAL_3B : in String;
      INITVAL_3A : in String; 
      INITVAL_39 : in String;
      INITVAL_38 : in String; 
      INITVAL_37 : in String;
      INITVAL_36 : in String; 
      INITVAL_35 : in String;
      INITVAL_34 : in String; 
      INITVAL_33 : in String;
      INITVAL_32 : in String; 
      INITVAL_31 : in String;
      INITVAL_30 : in String; 
      INITVAL_2F : in String;
      INITVAL_2E : in String; 
      INITVAL_2D : in String;
      INITVAL_2C : in String; 
      INITVAL_2B : in String;
      INITVAL_2A : in String; 
      INITVAL_29 : in String;
      INITVAL_28 : in String; 
      INITVAL_27 : in String;
      INITVAL_26 : in String; 
      INITVAL_25 : in String;
      INITVAL_24 : in String; 
      INITVAL_23 : in String;
      INITVAL_22 : in String; 
      INITVAL_21 : in String;
      INITVAL_20 : in String; 
      INITVAL_1F : in String;
      INITVAL_1E : in String; 
      INITVAL_1D : in String;
      INITVAL_1C : in String; 
      INITVAL_1B : in String;
      INITVAL_1A : in String; 
      INITVAL_19 : in String;
      INITVAL_18 : in String; 
      INITVAL_17 : in String;
      INITVAL_16 : in String; 
      INITVAL_15 : in String;
      INITVAL_14 : in String; 
      INITVAL_13 : in String;
      INITVAL_12 : in String; 
      INITVAL_11 : in String;
      INITVAL_10 : in String; 
      INITVAL_0F : in String;
      INITVAL_0E : in String; 
      INITVAL_0D : in String;
      INITVAL_0C : in String; 
      INITVAL_0B : in String;
      INITVAL_0A : in String; 
      INITVAL_09 : in String;
      INITVAL_08 : in String; 
      INITVAL_07 : in String;
      INITVAL_06 : in String; 
      INITVAL_05 : in String;
      INITVAL_04 : in String; 
      INITVAL_03 : in String;
      INITVAL_02 : in String; 
      INITVAL_01 : in String;
      INITVAL_00 : in String; 
      GSR : in String;
      WRITEMODE_B : in String; 
      CSDECODE_B : in std_logic_vector(2 downto 0); 
      CSDECODE_A : in std_logic_vector(2 downto 0); 
      WRITEMODE_A : in String;
      RESETMODE : in String; 
      REGMODE_B : in String;
      REGMODE_A : in String; 
      DATA_WIDTH_B : in Integer;
      DATA_WIDTH_A : in Integer);
    -- synopsys translate_on
    port (
      DIA0: in  std_logic;
      DIA1: in  std_logic; 
      DIA2: in  std_logic;
      DIA3: in  std_logic; 
      DIA4: in  std_logic;
      DIA5: in  std_logic; 
      DIA6: in  std_logic;
      DIA7: in  std_logic; 
      DIA8: in  std_logic;
      DIA9: in  std_logic; 
      DIA10: in  std_logic;
      DIA11: in  std_logic; 
      DIA12: in  std_logic;
      DIA13: in  std_logic; 
      DIA14: in  std_logic;
      DIA15: in  std_logic; 
      DIA16: in  std_logic;
      DIA17: in  std_logic; 
      ADA0: in  std_logic;
      ADA1: in  std_logic; 
      ADA2: in  std_logic;
      ADA3: in  std_logic; 
      ADA4: in  std_logic;
      ADA5: in  std_logic; 
      ADA6: in  std_logic;
      ADA7: in  std_logic; 
      ADA8: in  std_logic;
      ADA9: in  std_logic; 
      ADA10: in  std_logic;
      ADA11: in  std_logic; 
      ADA12: in  std_logic;
      ADA13: in  std_logic; 
      CEA: in  std_logic;
      CLKA: in  std_logic;
      WEA: in  std_logic; 
      CSA0: in  std_logic;
      CSA1: in  std_logic; 
      CSA2: in  std_logic;
      RSTA: in  std_logic; 
      DIB0: in  std_logic;
      DIB1: in  std_logic; 
      DIB2: in  std_logic;
      DIB3: in  std_logic; 
      DIB4: in  std_logic;
      DIB5: in  std_logic; 
      DIB6: in  std_logic;
      DIB7: in  std_logic; 
      DIB8: in  std_logic;
      DIB9: in  std_logic; 
      DIB10: in  std_logic;
      DIB11: in  std_logic; 
      DIB12: in  std_logic;
      DIB13: in  std_logic; 
      DIB14: in  std_logic;
      DIB15: in  std_logic; 
      DIB16: in  std_logic;
      DIB17: in  std_logic; 
      ADB0: in  std_logic;
      ADB1: in  std_logic; 
      ADB2: in  std_logic;
      ADB3: in  std_logic; 
      ADB4: in  std_logic;
      ADB5: in  std_logic; 
      ADB6: in  std_logic;
      ADB7: in  std_logic; 
      ADB8: in  std_logic;
      ADB9: in  std_logic; 
      ADB10: in  std_logic;
      ADB11: in  std_logic; 
      ADB12: in  std_logic;
      ADB13: in  std_logic; 
      CEB: in  std_logic;
      CLKB: in  std_logic;
      WEB: in  std_logic; 
      CSB0: in  std_logic;
      CSB1: in  std_logic; 
      CSB2: in  std_logic;
      RSTB: in  std_logic; 
      DOA0: out  std_logic;
      DOA1: out  std_logic; 
      DOA2: out  std_logic;
      DOA3: out  std_logic; 
      DOA4: out  std_logic;
      DOA5: out  std_logic; 
      DOA6: out  std_logic;
      DOA7: out  std_logic; 
      DOA8: out  std_logic;
      DOA9: out  std_logic; 
      DOA10: out  std_logic;
      DOA11: out  std_logic; 
      DOA12: out  std_logic;
      DOA13: out  std_logic; 
      DOA14: out  std_logic;
      DOA15: out  std_logic; 
      DOA16: out  std_logic;
      DOA17: out  std_logic; 
      DOB0: out  std_logic;
      DOB1: out  std_logic; 
      DOB2: out  std_logic;
      DOB3: out  std_logic; 
      DOB4: out  std_logic;
      DOB5: out  std_logic; 
      DOB6: out  std_logic;
      DOB7: out  std_logic; 
      DOB8: out  std_logic;
      DOB9: out  std_logic; 
      DOB10: out  std_logic;
      DOB11: out  std_logic; 
      DOB12: out  std_logic;
      DOB13: out  std_logic; 
      DOB14: out  std_logic;
      DOB15: out  std_logic; 
      DOB16: out  std_logic;
      DOB17: out  std_logic);
  end component;

begin

  gnd1 <= '0';
  vcc1 <= '1';
  gnd32 <= (others => '0');
  
  -- This RAM block maps internal events, default values are set here for
  -- the following event codes:
  -- code bit Event
  -- 0x70 0   Seconds '0'
  -- 0x71 1   Seconds '1'
  -- 0x7A 5   Heartbeat
  -- 0x7B 4   Reset prescalers
  -- 0x7C 2   Timestamp event counter clock
  -- 0x7D 3   Timestamp Event coutner reset
  DP16KB_0: DP16KB
    -- synopsys translate_off
    generic map (
      INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_07=> "0x00000000000002000010000040000800000000000000100002000000000000000000000000000000", 
      INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      CSDECODE_B=> "000",
      CSDECODE_A=> "000",
      WRITEMODE_B=> "NORMAL", 
      WRITEMODE_A=> "NORMAL",
      GSR=> "DISABLED",
      RESETMODE=> "SYNC", 
      REGMODE_B=> "OUTREG",
      REGMODE_A=> "OUTREG",
      DATA_WIDTH_B=>  18, 
      DATA_WIDTH_A=>  18)
    -- synopsys translate_on
    port map (
      DIA0 => bram_data_A_in(0),
      DIA1 => bram_data_A_in(1),
      DIA2 => bram_data_A_in(2),
      DIA3 => bram_data_A_in(3),
      DIA4 => bram_data_A_in(4),
      DIA5 => bram_data_A_in(5),
      DIA6 => bram_data_A_in(6),
      DIA7 => bram_data_A_in(7),
      DIA8 => bram_data_A_in(8),
      DIA9 => bram_data_A_in(9),
      DIA10 => bram_data_A_in(10),
      DIA11 => bram_data_A_in(11),
      DIA12 => bram_data_A_in(12),
      DIA13 => bram_data_A_in(13),
      DIA14 => bram_data_A_in(14),
      DIA15 => bram_data_A_in(15),
      DIA16 => gnd1,
      DIA17 => gnd1,
      ADA0 => scuba_vhi,
      ADA1 => scuba_vhi,
      ADA2 => scuba_vlo, 
      ADA3 => scuba_vlo,
      ADA4 => bram_addr_A(0),
      ADA5 => bram_addr_A(1),
      ADA6 => bram_addr_A(2),
      ADA7 => bram_addr_A(3),
      ADA8 => bram_addr_A(4),
      ADA9 => bram_addr_A(5),
      ADA10 => bram_addr_A(6),
      ADA11 => bram_addr_A(7),
      ADA12 => bram_addr_A(8),
      ADA13 => scuba_vlo,
      CEA => vcc1,
      CLKA => clk,
      WEA => bram_we(2**(map_code_abits-2)-1),
      CSA0 => scuba_vlo,
      CSA1 => scuba_vlo, 
      CSA2 => scuba_vlo,
      RSTA => gnd1,
      DIB0 => gnd1,
      DIB1 => gnd1,
      DIB2 => gnd1,
      DIB3 => gnd1,
      DIB4 => gnd1,
      DIB5 => gnd1,
      DIB6 => gnd1,
      DIB7 => gnd1,
      DIB8 => gnd1,
      DIB9 => gnd1,
      DIB10 => gnd1,
      DIB11 => gnd1,
      DIB12 => gnd1,
      DIB13 => gnd1,
      DIB14 => gnd1,
      DIB15 => gnd1,
      DIB16 => gnd1,
      DIB17 => gnd1,
      ADB0 => scuba_vhi, 
      ADB1 => scuba_vhi,
      ADB2 => scuba_vlo, 
      ADB3 => scuba_vlo,
      ADB4 => bram_addr_B(0),
      ADB5 => bram_addr_B(1),
      ADB6 => bram_addr_B(2),
      ADB7 => bram_addr_B(3),
      ADB8 => bram_addr_B(4),
      ADB9 => bram_addr_B(5),
      ADB10 => bram_addr_B(6),
      ADB11 => bram_addr_B(7),
      ADB12 => bram_addr_B(8),
      ADB13 => scuba_vlo,
      CEB => vcc1,
      CLKB => event_clk,
      WEB => gnd1,
      CSB0 => scuba_vlo,
      CSB1 => scuba_vlo,
      CSB2 => scuba_vlo, 
      RSTB => gnd1,
      DOA0 => bram_data_A_out(8*(2**map_code_abits)-32),
      DOA1 => bram_data_A_out(8*(2**map_code_abits)-31),
      DOA2 => bram_data_A_out(8*(2**map_code_abits)-30),
      DOA3 => bram_data_A_out(8*(2**map_code_abits)-29),
      DOA4 => bram_data_A_out(8*(2**map_code_abits)-28),
      DOA5 => bram_data_A_out(8*(2**map_code_abits)-27),
      DOA6 => bram_data_A_out(8*(2**map_code_abits)-26),
      DOA7 => bram_data_A_out(8*(2**map_code_abits)-25),
      DOA8 => bram_data_A_out(8*(2**map_code_abits)-24),
      DOA9 => bram_data_A_out(8*(2**map_code_abits)-23),
      DOA10 => bram_data_A_out(8*(2**map_code_abits)-22),
      DOA11 => bram_data_A_out(8*(2**map_code_abits)-21),
      DOA12 => bram_data_A_out(8*(2**map_code_abits)-20),
      DOA13 => bram_data_A_out(8*(2**map_code_abits)-19),
      DOA14 => bram_data_A_out(8*(2**map_code_abits)-18),
      DOA15 => bram_data_A_out(8*(2**map_code_abits)-17),
      DOA16 => open,
      DOA17 => open,
      DOB0 => event_map(8*(2**map_code_abits)-32),
      DOB1 => event_map(8*(2**map_code_abits)-31),
      DOB2 => event_map(8*(2**map_code_abits)-30),
      DOB3 => event_map(8*(2**map_code_abits)-29),
      DOB4 => event_map(8*(2**map_code_abits)-28),
      DOB5 => event_map(8*(2**map_code_abits)-27),
      DOB6 => event_map(8*(2**map_code_abits)-26),
      DOB7 => event_map(8*(2**map_code_abits)-25),
      DOB8 => event_map(8*(2**map_code_abits)-24),
      DOB9 => event_map(8*(2**map_code_abits)-23),
      DOB10 => event_map(8*(2**map_code_abits)-22),
      DOB11 => event_map(8*(2**map_code_abits)-21),
      DOB12 => event_map(8*(2**map_code_abits)-20),
      DOB13 => event_map(8*(2**map_code_abits)-19),
      DOB14 => event_map(8*(2**map_code_abits)-18),
      DOB15 => event_map(8*(2**map_code_abits)-17),
      DOB16 => open, 
      DOB17 => open);

  -- This RAM block maps internal events, default values are set here for
  -- the following event codes:
  -- code bit Event
  -- 0x79 27  Stop Event Log
  DP16KB_1: DP16KB
    -- synopsys translate_off
    generic map (
      INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_07=> "0x00000002000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
      CSDECODE_B=> "000",
      CSDECODE_A=> "000",
      WRITEMODE_B=> "NORMAL", 
      WRITEMODE_A=> "NORMAL",
      GSR=> "DISABLED",
      RESETMODE=> "SYNC", 
      REGMODE_B=> "OUTREG",
      REGMODE_A=> "OUTREG",
      DATA_WIDTH_B=>  18, 
      DATA_WIDTH_A=>  18)
    -- synopsys translate_on
    port map (
      DIA0 => bram_data_A_in(16),
      DIA1 => bram_data_A_in(17),
      DIA2 => bram_data_A_in(18),
      DIA3 => bram_data_A_in(19),
      DIA4 => bram_data_A_in(20),
      DIA5 => bram_data_A_in(21),
      DIA6 => bram_data_A_in(22),
      DIA7 => bram_data_A_in(23),
      DIA8 => bram_data_A_in(24),
      DIA9 => bram_data_A_in(25),
      DIA10 => bram_data_A_in(26),
      DIA11 => bram_data_A_in(27),
      DIA12 => bram_data_A_in(28),
      DIA13 => bram_data_A_in(29),
      DIA14 => bram_data_A_in(30),
      DIA15 => bram_data_A_in(31),
      DIA16 => gnd1,
      DIA17 => gnd1,
      ADA0 => scuba_vhi,
      ADA1 => scuba_vhi,
      ADA2 => scuba_vlo, 
      ADA3 => scuba_vlo,
      ADA4 => bram_addr_A(0),
      ADA5 => bram_addr_A(1),
      ADA6 => bram_addr_A(2),
      ADA7 => bram_addr_A(3),
      ADA8 => bram_addr_A(4),
      ADA9 => bram_addr_A(5),
      ADA10 => bram_addr_A(6),
      ADA11 => bram_addr_A(7),
      ADA12 => bram_addr_A(8),
      ADA13 => scuba_vlo,
      CEA => vcc1,
      CLKA => clk,
      WEA => bram_we(2**(map_code_abits-2)-1),
      CSA0 => scuba_vlo,
      CSA1 => scuba_vlo, 
      CSA2 => scuba_vlo,
      RSTA => gnd1,
      DIB0 => gnd1,
      DIB1 => gnd1,
      DIB2 => gnd1,
      DIB3 => gnd1,
      DIB4 => gnd1,
      DIB5 => gnd1,
      DIB6 => gnd1,
      DIB7 => gnd1,
      DIB8 => gnd1,
      DIB9 => gnd1,
      DIB10 => gnd1,
      DIB11 => gnd1,
      DIB12 => gnd1,
      DIB13 => gnd1,
      DIB14 => gnd1,
      DIB15 => gnd1,
      DIB16 => gnd1,
      DIB17 => gnd1,
      ADB0 => scuba_vhi, 
      ADB1 => scuba_vhi,
      ADB2 => scuba_vlo, 
      ADB3 => scuba_vlo,
      ADB4 => bram_addr_B(0),
      ADB5 => bram_addr_B(1),
      ADB6 => bram_addr_B(2),
      ADB7 => bram_addr_B(3),
      ADB8 => bram_addr_B(4),
      ADB9 => bram_addr_B(5),
      ADB10 => bram_addr_B(6),
      ADB11 => bram_addr_B(7),
      ADB12 => bram_addr_B(8),
      ADB13 => scuba_vlo,
      CEB => vcc1,
      CLKB => event_clk,
      WEB => gnd1,
      CSB0 => scuba_vlo,
      CSB1 => scuba_vlo,
      CSB2 => scuba_vlo, 
      RSTB => gnd1,
      DOA0 => bram_data_A_out(8*(2**map_code_abits)-16),
      DOA1 => bram_data_A_out(8*(2**map_code_abits)-15),
      DOA2 => bram_data_A_out(8*(2**map_code_abits)-14),
      DOA3 => bram_data_A_out(8*(2**map_code_abits)-13),
      DOA4 => bram_data_A_out(8*(2**map_code_abits)-12),
      DOA5 => bram_data_A_out(8*(2**map_code_abits)-11),
      DOA6 => bram_data_A_out(8*(2**map_code_abits)-10),
      DOA7 => bram_data_A_out(8*(2**map_code_abits)-9),
      DOA8 => bram_data_A_out(8*(2**map_code_abits)-8),
      DOA9 => bram_data_A_out(8*(2**map_code_abits)-7),
      DOA10 => bram_data_A_out(8*(2**map_code_abits)-6),
      DOA11 => bram_data_A_out(8*(2**map_code_abits)-5),
      DOA12 => bram_data_A_out(8*(2**map_code_abits)-4),
      DOA13 => bram_data_A_out(8*(2**map_code_abits)-3),
      DOA14 => bram_data_A_out(8*(2**map_code_abits)-2),
      DOA15 => bram_data_A_out(8*(2**map_code_abits)-1),
      DOA16 => open,
      DOA17 => open,
      DOB0 => event_map(8*(2**map_code_abits)-16),
      DOB1 => event_map(8*(2**map_code_abits)-15),
      DOB2 => event_map(8*(2**map_code_abits)-14),
      DOB3 => event_map(8*(2**map_code_abits)-13),
      DOB4 => event_map(8*(2**map_code_abits)-12),
      DOB5 => event_map(8*(2**map_code_abits)-11),
      DOB6 => event_map(8*(2**map_code_abits)-10),
      DOB7 => event_map(8*(2**map_code_abits)-9),
      DOB8 => event_map(8*(2**map_code_abits)-8),
      DOB9 => event_map(8*(2**map_code_abits)-7),
      DOB10 => event_map(8*(2**map_code_abits)-6),
      DOB11 => event_map(8*(2**map_code_abits)-5),
      DOB12 => event_map(8*(2**map_code_abits)-4),
      DOB13 => event_map(8*(2**map_code_abits)-3),
      DOB14 => event_map(8*(2**map_code_abits)-2),
      DOB15 => event_map(8*(2**map_code_abits)-1),
      DOB16 => open, 
      DOB17 => open);

  ram_blocks:
    for i in 1 to (2**(map_code_abits-2))-1 generate
      i_ram_0: DP16KB
	-- synopsys translate_off
	generic map (
	  INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_07=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  CSDECODE_B=> "000",
	  CSDECODE_A=> "000",
	  WRITEMODE_B=> "NORMAL", 
	  WRITEMODE_A=> "NORMAL",
	  GSR=> "DISABLED",
	  RESETMODE=> "SYNC", 
	  REGMODE_B=> "OUTREG",
	  REGMODE_A=> "OUTREG",
	  DATA_WIDTH_B=>  18, 
	  DATA_WIDTH_A=>  18)
	-- synopsys translate_on
	port map (
	  DIA0 => bram_data_A_in(0),
	  DIA1 => bram_data_A_in(1),
	  DIA2 => bram_data_A_in(2),
	  DIA3 => bram_data_A_in(3),
	  DIA4 => bram_data_A_in(4),
	  DIA5 => bram_data_A_in(5),
	  DIA6 => bram_data_A_in(6),
	  DIA7 => bram_data_A_in(7),
	  DIA8 => bram_data_A_in(8),
	  DIA9 => bram_data_A_in(9),
	  DIA10 => bram_data_A_in(10),
	  DIA11 => bram_data_A_in(11),
	  DIA12 => bram_data_A_in(12),
	  DIA13 => bram_data_A_in(13),
	  DIA14 => bram_data_A_in(14),
	  DIA15 => bram_data_A_in(15),
	  DIA16 => gnd1,
	  DIA17 => gnd1,
	  ADA0 => scuba_vhi,
	  ADA1 => scuba_vhi,
	  ADA2 => scuba_vlo, 
	  ADA3 => scuba_vlo,
	  ADA4 => bram_addr_A(0),
	  ADA5 => bram_addr_A(1),
	  ADA6 => bram_addr_A(2),
	  ADA7 => bram_addr_A(3),
	  ADA8 => bram_addr_A(4),
	  ADA9 => bram_addr_A(5),
	  ADA10 => bram_addr_A(6),
	  ADA11 => bram_addr_A(7),
	  ADA12 => bram_addr_A(8),
	  ADA13 => scuba_vlo,
	  CEA => vcc1,
	  CLKA => clk,
	  WEA => bram_we(2**(map_code_abits-2)-1-i),
	  CSA0 => scuba_vlo,
	  CSA1 => scuba_vlo, 
	  CSA2 => scuba_vlo,
	  RSTA => gnd1,
	  DIB0 => gnd1,
	  DIB1 => gnd1,
	  DIB2 => gnd1,
	  DIB3 => gnd1,
	  DIB4 => gnd1,
	  DIB5 => gnd1,
	  DIB6 => gnd1,
	  DIB7 => gnd1,
	  DIB8 => gnd1,
	  DIB9 => gnd1,
	  DIB10 => gnd1,
	  DIB11 => gnd1,
	  DIB12 => gnd1,
	  DIB13 => gnd1,
	  DIB14 => gnd1,
	  DIB15 => gnd1,
	  DIB16 => gnd1,
	  DIB17 => gnd1,
	  ADB0 => scuba_vhi, 
	  ADB1 => scuba_vhi,
	  ADB2 => scuba_vlo, 
	  ADB3 => scuba_vlo,
	  ADB4 => bram_addr_B(0),
	  ADB5 => bram_addr_B(1),
	  ADB6 => bram_addr_B(2),
	  ADB7 => bram_addr_B(3),
	  ADB8 => bram_addr_B(4),
	  ADB9 => bram_addr_B(5),
	  ADB10 => bram_addr_B(6),
	  ADB11 => bram_addr_B(7),
	  ADB12 => bram_addr_B(8),
	  ADB13 => scuba_vlo,
	  CEB => vcc1,
	  CLKB => event_clk,
	  WEB => gnd1,
	  CSB0 => scuba_vlo,
	  CSB1 => scuba_vlo,
	  CSB2 => scuba_vlo, 
	  RSTB => gnd1,
	  DOA0 => bram_data_A_out(8*(2**map_code_abits)-32-i*32),
	  DOA1 => bram_data_A_out(8*(2**map_code_abits)-31-i*32),
	  DOA2 => bram_data_A_out(8*(2**map_code_abits)-30-i*32),
	  DOA3 => bram_data_A_out(8*(2**map_code_abits)-29-i*32),
	  DOA4 => bram_data_A_out(8*(2**map_code_abits)-28-i*32),
	  DOA5 => bram_data_A_out(8*(2**map_code_abits)-27-i*32),
	  DOA6 => bram_data_A_out(8*(2**map_code_abits)-26-i*32),
	  DOA7 => bram_data_A_out(8*(2**map_code_abits)-25-i*32),
	  DOA8 => bram_data_A_out(8*(2**map_code_abits)-24-i*32),
	  DOA9 => bram_data_A_out(8*(2**map_code_abits)-23-i*32),
	  DOA10 => bram_data_A_out(8*(2**map_code_abits)-22-i*32),
	  DOA11 => bram_data_A_out(8*(2**map_code_abits)-21-i*32),
	  DOA12 => bram_data_A_out(8*(2**map_code_abits)-20-i*32),
	  DOA13 => bram_data_A_out(8*(2**map_code_abits)-19-i*32),
	  DOA14 => bram_data_A_out(8*(2**map_code_abits)-18-i*32),
	  DOA15 => bram_data_A_out(8*(2**map_code_abits)-17-i*32),
	  DOA16 => open,
	  DOA17 => open,
	  DOB0 => event_map(8*(2**map_code_abits)-32-i*32),
	  DOB1 => event_map(8*(2**map_code_abits)-31-i*32),
	  DOB2 => event_map(8*(2**map_code_abits)-30-i*32),
	  DOB3 => event_map(8*(2**map_code_abits)-29-i*32),
	  DOB4 => event_map(8*(2**map_code_abits)-28-i*32),
	  DOB5 => event_map(8*(2**map_code_abits)-27-i*32),
	  DOB6 => event_map(8*(2**map_code_abits)-26-i*32),
	  DOB7 => event_map(8*(2**map_code_abits)-25-i*32),
	  DOB8 => event_map(8*(2**map_code_abits)-24-i*32),
	  DOB9 => event_map(8*(2**map_code_abits)-23-i*32),
	  DOB10 => event_map(8*(2**map_code_abits)-22-i*32),
	  DOB11 => event_map(8*(2**map_code_abits)-21-i*32),
	  DOB12 => event_map(8*(2**map_code_abits)-20-i*32),
	  DOB13 => event_map(8*(2**map_code_abits)-19-i*32),
	  DOB14 => event_map(8*(2**map_code_abits)-18-i*32),
	  DOB15 => event_map(8*(2**map_code_abits)-17-i*32),
	  DOB16 => open, 
	  DOB17 => open);
      i_ram_1: DP16KB
	-- synopsys translate_off
	generic map (
	  INITVAL_3F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_3A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_39=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_38=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_37=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_36=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_35=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_34=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_33=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_32=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_31=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_30=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_2A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_29=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_28=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_27=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_26=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_25=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_24=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_23=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_22=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_21=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_20=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_1A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_19=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_18=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_17=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_16=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_15=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_14=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_13=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_12=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_11=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_10=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0F=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0E=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0D=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0C=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0B=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_0A=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_09=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_08=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_07=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_06=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_05=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_04=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_03=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_02=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_01=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  INITVAL_00=> "0x00000000000000000000000000000000000000000000000000000000000000000000000000000000", 
	  CSDECODE_B=> "000",
	  CSDECODE_A=> "000",
	  WRITEMODE_B=> "NORMAL", 
	  WRITEMODE_A=> "NORMAL",
	  GSR=> "DISABLED",
	  RESETMODE=> "SYNC", 
	  REGMODE_B=> "OUTREG",
	  REGMODE_A=> "OUTREG",
	  DATA_WIDTH_B=>  18, 
	  DATA_WIDTH_A=>  18)
	-- synopsys translate_on
	port map (
	  DIA0 => bram_data_A_in(16),
	  DIA1 => bram_data_A_in(17),
	  DIA2 => bram_data_A_in(18),
	  DIA3 => bram_data_A_in(19),
	  DIA4 => bram_data_A_in(20),
	  DIA5 => bram_data_A_in(21),
	  DIA6 => bram_data_A_in(22),
	  DIA7 => bram_data_A_in(23),
	  DIA8 => bram_data_A_in(24),
	  DIA9 => bram_data_A_in(25),
	  DIA10 => bram_data_A_in(26),
	  DIA11 => bram_data_A_in(27),
	  DIA12 => bram_data_A_in(28),
	  DIA13 => bram_data_A_in(29),
	  DIA14 => bram_data_A_in(30),
	  DIA15 => bram_data_A_in(31),
	  DIA16 => gnd1,
	  DIA17 => gnd1,
	  ADA0 => scuba_vhi,
	  ADA1 => scuba_vhi,
	  ADA2 => scuba_vlo, 
	  ADA3 => scuba_vlo,
	  ADA4 => bram_addr_A(0),
	  ADA5 => bram_addr_A(1),
	  ADA6 => bram_addr_A(2),
	  ADA7 => bram_addr_A(3),
	  ADA8 => bram_addr_A(4),
	  ADA9 => bram_addr_A(5),
	  ADA10 => bram_addr_A(6),
	  ADA11 => bram_addr_A(7),
	  ADA12 => bram_addr_A(8),
	  ADA13 => scuba_vlo,
	  CEA => vcc1,
	  CLKA => clk,
	  WEA => bram_we(2**(map_code_abits-2)-1-i),
	  CSA0 => scuba_vlo,
	  CSA1 => scuba_vlo, 
	  CSA2 => scuba_vlo,
	  RSTA => gnd1,
	  DIB0 => gnd1,
	  DIB1 => gnd1,
	  DIB2 => gnd1,
	  DIB3 => gnd1,
	  DIB4 => gnd1,
	  DIB5 => gnd1,
	  DIB6 => gnd1,
	  DIB7 => gnd1,
	  DIB8 => gnd1,
	  DIB9 => gnd1,
	  DIB10 => gnd1,
	  DIB11 => gnd1,
	  DIB12 => gnd1,
	  DIB13 => gnd1,
	  DIB14 => gnd1,
	  DIB15 => gnd1,
	  DIB16 => gnd1,
	  DIB17 => gnd1,
	  ADB0 => scuba_vhi, 
	  ADB1 => scuba_vhi,
	  ADB2 => scuba_vlo, 
	  ADB3 => scuba_vlo,
	  ADB4 => bram_addr_B(0),
	  ADB5 => bram_addr_B(1),
	  ADB6 => bram_addr_B(2),
	  ADB7 => bram_addr_B(3),
	  ADB8 => bram_addr_B(4),
	  ADB9 => bram_addr_B(5),
	  ADB10 => bram_addr_B(6),
	  ADB11 => bram_addr_B(7),
	  ADB12 => bram_addr_B(8),
	  ADB13 => scuba_vlo,
	  CEB => vcc1,
	  CLKB => event_clk,
	  WEB => gnd1,
	  CSB0 => scuba_vlo,
	  CSB1 => scuba_vlo,
	  CSB2 => scuba_vlo, 
	  RSTB => gnd1,
	  DOA0 => bram_data_A_out(8*(2**map_code_abits)-16-i*32),
	  DOA1 => bram_data_A_out(8*(2**map_code_abits)-15-i*32),
	  DOA2 => bram_data_A_out(8*(2**map_code_abits)-14-i*32),
	  DOA3 => bram_data_A_out(8*(2**map_code_abits)-13-i*32),
	  DOA4 => bram_data_A_out(8*(2**map_code_abits)-12-i*32),
	  DOA5 => bram_data_A_out(8*(2**map_code_abits)-11-i*32),
	  DOA6 => bram_data_A_out(8*(2**map_code_abits)-10-i*32),
	  DOA7 => bram_data_A_out(8*(2**map_code_abits)-9-i*32),
	  DOA8 => bram_data_A_out(8*(2**map_code_abits)-8-i*32),
	  DOA9 => bram_data_A_out(8*(2**map_code_abits)-7-i*32),
	  DOA10 => bram_data_A_out(8*(2**map_code_abits)-6-i*32),
	  DOA11 => bram_data_A_out(8*(2**map_code_abits)-5-i*32),
	  DOA12 => bram_data_A_out(8*(2**map_code_abits)-4-i*32),
	  DOA13 => bram_data_A_out(8*(2**map_code_abits)-3-i*32),
	  DOA14 => bram_data_A_out(8*(2**map_code_abits)-2-i*32),
	  DOA15 => bram_data_A_out(8*(2**map_code_abits)-1-i*32),
	  DOA16 => open,
	  DOA17 => open,
	  DOB0 => event_map(8*(2**map_code_abits)-16-i*32),
	  DOB1 => event_map(8*(2**map_code_abits)-15-i*32),
	  DOB2 => event_map(8*(2**map_code_abits)-14-i*32),
	  DOB3 => event_map(8*(2**map_code_abits)-13-i*32),
	  DOB4 => event_map(8*(2**map_code_abits)-12-i*32),
	  DOB5 => event_map(8*(2**map_code_abits)-11-i*32),
	  DOB6 => event_map(8*(2**map_code_abits)-10-i*32),
	  DOB7 => event_map(8*(2**map_code_abits)-9-i*32),
	  DOB8 => event_map(8*(2**map_code_abits)-8-i*32),
	  DOB9 => event_map(8*(2**map_code_abits)-7-i*32),
	  DOB10 => event_map(8*(2**map_code_abits)-6-i*32),
	  DOB11 => event_map(8*(2**map_code_abits)-5-i*32),
	  DOB12 => event_map(8*(2**map_code_abits)-4-i*32),
	  DOB13 => event_map(8*(2**map_code_abits)-3-i*32),
	  DOB14 => event_map(8*(2**map_code_abits)-2-i*32),
	  DOB15 => event_map(8*(2**map_code_abits)-1-i*32),
	  DOB16 => open, 
	  DOB17 => open);
    end generate;
  
end structure;
    
