---------------------------------------------------------------------------
--
--  File        : membuf_bsram_v5.vhd
--
--  Title       : Event Receiver memory buffer RAM for Virtex 5
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
library unisim;
use unisim.vcomponents.all;

entity membuf_bsram is
  port (
    addra: IN std_logic_VECTOR(10 downto 0);
    addrb: IN std_logic_VECTOR(8 downto 0);
    clka: IN std_logic;
    clkb: IN std_logic;
    douta: OUT std_logic_VECTOR(7 downto 0);
    dinb: IN std_logic_VECTOR(31 downto 0);
    doutb: OUT std_logic_VECTOR(31 downto 0);
    web: IN std_logic);
end membuf_bsram;

architecture Structure of membuf_bsram is

  signal gnd      : std_logic;
  signal vcc      : std_logic;
  signal gnd_vec  : std_logic_vector(31 downto 0);

begin

  bram_0 : RAMB36
    generic map (
      DOA_REG => 1,
      DOB_REG => 1,
      INIT_A => X"00000",
      INIT_B => X"00000",
      READ_WIDTH_A => 9,
      READ_WIDTH_B => 36,
      SIM_COLLISION_CHECK => "ALL",
      SRVAL_A => X"00000",
      SRVAL_B => X"00000",
      WRITE_MODE_A => "WRITE_FIRST",
      WRITE_MODE_B => "WRITE_FIRST",
      WRITE_WIDTH_A => 9,
      WRITE_WIDTH_B => 36)
    port map (
      DOA(31 downto 8) => open,
      DOA(7 downto 0) => douta,
      DOB => doutb,
      DOPA => open,
      DOPB => open,
      ADDRA(15 downto 12) => gnd_vec(3 downto 0),
      ADDRA(11 downto 1) => addra,
      ADDRA(0) => gnd,
      ADDRB(15 downto 12) => gnd_vec(3 downto 0),
      ADDRB(11 downto 3) => addrb,
      ADDRB(2 downto 0) => gnd_vec(2 downto 0),
      CLKA => clka,
      CLKB => clkb,
      DIA => gnd_vec(31 downto 0),
      DIB => dinb,
      DIPA => gnd_vec(3 downto 0),
      DIPB => gnd_vec(3 downto 0),
      ENA => vcc,
      ENB => vcc,
      REGCEA => vcc,
      REGCEB => vcc,
      SSRA => gnd,
      SSRB => gnd,
      WEA => gnd_vec(3 downto 0),
      WEB(0) => web,
      WEB(1) => web,
      WEB(2) => web,
      WEB(3) => web);

  gnd <= '0';
  vcc <= '1';
  gnd_vec <= (others => '0');

end Structure;
