---------------------------------------------------------------------------
--
--  File        : mgt_cmlout_bram.vhd
--
--  Title       : Multi Gigabit Transceiver
--		  to Generate CML outputs: RF and pulses
--                utilising BRAM blocks
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity mgt_cmlout_bram is
  port (
    clk       : in std_logic;
    data_in   : in std_logic_vector(23 downto 0);
    data_out  : out std_logic_vector(31 downto 0);
    addr      : in std_logic_vector(1 downto 0);
    we        : in std_logic_vector(2 downto 0);
    event_clk : in std_logic;
    brefclk   : in std_logic;
    brefclk2  : in std_logic;
    refclksel : in std_logic;
    pulse     : in std_logic;
    enable : in std_logic;
    powerdown : in std_logic;
    freq_mode : in std_logic;
    high_period : in std_logic_vector(15 downto 0);
    low_period  : in std_logic_vector(15 downto 0);
    trig_pos    : in std_logic_vector(15 downto 0);
    trig_level  : in std_logic;
    RXN : in std_logic;
    RXP : in std_logic;
    txreset : in std_logic;
    TXN : out std_logic;
    TXP : out std_logic);
end mgt_cmlout_bram;

architecture STRUCT of mgt_cmlout_bram is
  signal GND1 : std_logic;
  signal GND2 : std_logic_vector (1 downto 0);
  signal GND4 : std_logic_vector (3 downto 0);
  signal GND32 : std_logic_vector (31 downto 0);
  signal VCC1 : std_logic;
  signal VCC4 : std_logic_vector (3 downto 0);
  signal RXCHARISCOMMA_float : std_logic_vector (3 downto 0);
  signal RXCHARISK_float : std_logic_vector (3 downto 0);
  signal RXDATA_float : std_logic_vector (15 downto 0);
  signal RXDISPERR_float : std_logic_vector (1 downto 0);
  signal RXNOTINTABLE_float : std_logic_vector (1 downto 0);
  signal RXRUNDISP_float : std_logic_vector (3 downto 0);
  signal TXBUFERR_float : std_logic;
  signal TXKERR_float : std_logic_vector (3 downto 0);
  signal TXRUNDISP_float : std_logic_vector (3 downto 0);
  signal RXRECCLK : std_logic;
  signal RXCLK_BUF : std_logic;
  signal txdata : std_logic_vector(19 downto 0);
  signal ram_txdata : std_logic_vector(23 downto 0);
  signal freq_txdata : std_logic_vector(19 downto 0);

  signal pulse_in : std_logic_vector(1 downto 0);
  
  component RAMB16_S9_S9 is
    generic (
      INIT_00 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_01 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_02 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_03 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_04 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_05 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_06 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_07 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_08 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_09 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0A : bit_vector :=
      X"00000A0000000000000000000000000000000000000000000000000000000000";
      INIT_0B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_0F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_10 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_11 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_12 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_13 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_14 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_15 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_16 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_17 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_18 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_19 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1A : bit_vector :=
      X"00000A0000000000000000000000000000000000000000000000000000000000";
      INIT_1B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_1F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_20 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_21 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_22 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_23 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_24 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_25 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_26 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_27 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_28 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_29 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2A : bit_vector :=
      X"00000A0000000000000000000000000000000000000000000000000000000000";
      INIT_2B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_2F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_30 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_31 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_32 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_33 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_34 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_35 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_36 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_37 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_38 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_39 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3A : bit_vector :=
      X"00000A0000000000000000000000000000000000000000000000000000000000";
      INIT_3B : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3C : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3D : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3E : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_3F : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INIT_A : bit_vector  := X"000";
      INIT_B : bit_vector  := X"000";
      INITP_00 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_01 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_02 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_03 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_04 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_05 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_06 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      INITP_07 : bit_vector :=
      X"0000000000000000000000000000000000000000000000000000000000000000";
      SRVAL_A : bit_vector  := X"000";
      SRVAL_B : bit_vector  := X"000";
      WRITE_MODE_A : string := "WRITE_FIRST";
      WRITE_MODE_B : string := "WRITE_FIRST"
      );

    port (
      DOA   : out std_logic_vector(7 downto 0);
      DOB   : out std_logic_vector(7 downto 0);
      DOPA  : out std_logic_vector(0 downto 0);
      DOPB  : out std_logic_vector(0 downto 0);
      ADDRA : in  std_logic_vector(10 downto 0);
      ADDRB : in  std_logic_vector(10 downto 0);
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      DIA   : in  std_logic_vector(7 downto 0);
      DIB   : in  std_logic_vector(7 downto 0);
      DIPA  : in  std_logic_vector(0 downto 0);
      DIPB  : in  std_logic_vector(0 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic);
  end component;

  component GT_CUSTOM
    generic( 
      ALIGN_COMMA_MSB : boolean := FALSE;
      CHAN_BOND_LIMIT : integer := 16;
      CHAN_BOND_MODE : string := "OFF";
      CHAN_BOND_OFFSET : integer := 8;
      CHAN_BOND_ONE_SHOT : boolean := FALSE;
      CHAN_BOND_SEQ_1_1 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_1_2 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_1_3 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_1_4 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_2_1 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_2_2 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_2_3 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_2_4 : bit_vector := "00000000000";
      CHAN_BOND_SEQ_2_USE : boolean := FALSE;
      CHAN_BOND_SEQ_LEN : integer := 1;
      CHAN_BOND_WAIT : integer := 8;
      CLK_COR_INSERT_IDLE_FLAG : boolean := FALSE;
      CLK_COR_KEEP_IDLE : boolean := FALSE;
      CLK_COR_REPEAT_WAIT : integer := 1;
      CLK_COR_SEQ_1_1 : bit_vector := "00000000000";
      CLK_COR_SEQ_1_2 : bit_vector := "00000000000";
      CLK_COR_SEQ_1_3 : bit_vector := "00000000000";
      CLK_COR_SEQ_1_4 : bit_vector := "00000000000";
      CLK_COR_SEQ_2_1 : bit_vector := "00000000000";
      CLK_COR_SEQ_2_2 : bit_vector := "00000000000";
      CLK_COR_SEQ_2_3 : bit_vector := "00000000000";
      CLK_COR_SEQ_2_4 : bit_vector := "00000000000";
      CLK_COR_SEQ_2_USE : boolean := FALSE;
      CLK_COR_SEQ_LEN : integer := 1;
      CLK_CORRECT_USE : boolean := TRUE;
      COMMA_10B_MASK : bit_vector := "1111111000";
      CRC_END_OF_PKT : string := "K29_7";
      CRC_FORMAT : string := "USER_MODE";
      CRC_START_OF_PKT : string := "K27_7";
      DEC_MCOMMA_DETECT : boolean := TRUE;
      DEC_PCOMMA_DETECT : boolean := TRUE;
      DEC_VALID_COMMA_ONLY : boolean := TRUE;
      MCOMMA_10B_VALUE : bit_vector := "1100000000";
      MCOMMA_DETECT : boolean := TRUE;
      PCOMMA_10B_VALUE : bit_vector := "0011111000";
      PCOMMA_DETECT : boolean := TRUE;
      REF_CLK_V_SEL : integer := 0;
      RX_BUFFER_USE : boolean := TRUE;
      RX_CRC_USE : boolean := FALSE;
      RX_DATA_WIDTH : integer := 2;
      RX_DECODE_USE : boolean := TRUE;
      RX_LOS_INVALID_INCR : integer := 1;
      RX_LOS_THRESHOLD : integer := 4;
      RX_LOSS_OF_SYNC_FSM : boolean := TRUE;
      SERDES_10B : boolean := FALSE;
      TERMINATION_IMP : integer := 50;
      TX_BUFFER_USE : boolean := TRUE;
      TX_CRC_FORCE_VALUE : bit_vector := "11010110";
      TX_CRC_USE : boolean := FALSE;
      TX_DATA_WIDTH : integer := 2;
      TX_DIFF_CTRL : integer := 500;
      TX_PREEMPHASIS : integer := 0
      );
    port (
      CHBONDI : in std_logic_vector (3 downto 0);
      CONFIGENABLE : in std_logic;
      CONFIGIN : in std_logic;
      ENMCOMMAALIGN : in std_logic;
      ENPCOMMAALIGN : in std_logic;
      ENCHANSYNC : in std_logic;
      LOOPBACK : in std_logic_vector (1 downto 0);
      POWERDOWN : in std_logic;
      REFCLK : in std_logic;
      REFCLK2 : in std_logic;
      REFCLKSEL : in std_logic;
      BREFCLK : in std_logic;
      BREFCLK2 : in std_logic;
      RXN : in std_logic;
      RXP : in std_logic;
      RXPOLARITY : in std_logic;
      RXRESET : in std_logic;
      RXUSRCLK : in std_logic;
      RXUSRCLK2 : in std_logic;
      TXBYPASS8B10B : in std_logic_vector (3 downto 0);
      TXCHARDISPMODE : in std_logic_vector (3 downto 0);
      TXCHARDISPVAL : in std_logic_vector (3 downto 0);
      TXCHARISK : in std_logic_vector (3 downto 0);
      TXDATA : in std_logic_vector (31 downto 0);
      TXFORCECRCERR : in std_logic;
      TXINHIBIT : in std_logic;
      TXPOLARITY : in std_logic;
      TXRESET : in std_logic;
      TXUSRCLK : in std_logic;
      TXUSRCLK2 : in std_logic;
      CHBONDDONE : out std_logic;
      CHBONDO : out std_logic_vector (3 downto 0);
      CONFIGOUT : out std_logic;
      RXBUFSTATUS : out std_logic_vector (1 downto 0);
      RXCHARISCOMMA : out std_logic_vector (3 downto 0);
      RXCHARISK : out std_logic_vector (3 downto 0);
      RXCHECKINGCRC : out std_logic;
      RXCLKCORCNT : out std_logic_vector (2 downto 0);
      RXCOMMADET : out std_logic;
      RXCRCERR : out std_logic;
      RXDATA : out std_logic_vector (31 downto 0);
      RXDISPERR : out std_logic_vector (3 downto 0);
      RXLOSSOFSYNC : out std_logic_vector (1 downto 0);
      RXNOTINTABLE : out std_logic_vector (3 downto 0);
      RXREALIGN : out std_logic;
      RXRECCLK : out std_logic;
      RXRUNDISP : out std_logic_vector (3 downto 0);
      TXBUFERR : out std_logic;
      TXKERR : out std_logic_vector (3 downto 0);
      TXN : out std_logic;
      TXP : out std_logic;
      TXRUNDISP : out std_logic_vector (3 downto 0)
      );
  end component;
  
  component BUFG
    port (O : out STD_ULOGIC;
	  I : in  STD_ULOGIC);
  end component;
   
begin

  bram_0 : RAMB16_S9_S9
    generic map (
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA => data_out(7 downto 0),
      DOB => ram_txdata(7 downto 0),
      ADDRA(10 downto 3) => GND32(10 downto 3),
      ADDRA(2) => vcc1,
      ADDRA(1 downto 0) => addr,
      ADDRB(10 downto 3) => GND32(10 downto 3),
      ADDRB(2) => enable,
      ADDRB(1 downto 0) => pulse_in,
      CLKA => clk,
      CLKB => event_clk,
      DIA => data_in(7 downto 0),
      DIB => GND32(7 downto 0),
      DIPA => GND32(0 downto 0),
      DIPB => GND32(0 downto 0),
      ENA => VCC1,
      ENB => VCC1,
      SSRA => GND1,
      SSRB => GND1,
      WEA => we(0),
      WEB => GND1);
  
  bram_1 : RAMB16_S9_S9
    generic map (
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA => data_out(15 downto 8),
      DOB => ram_txdata(15 downto 8),
      ADDRA(10 downto 3) => GND32(10 downto 3),
      ADDRA(2) => vcc1,
      ADDRA(1 downto 0) => addr,
      ADDRB(10 downto 3) => GND32(10 downto 3),
      ADDRB(2) => enable,
      ADDRB(1 downto 0) => pulse_in,
      CLKA => clk,
      CLKB => event_clk,
      DIA => data_in(15 downto 8),
      DIB => GND32(7 downto 0),
      DIPA => GND32(0 downto 0),
      DIPB => GND32(0 downto 0),
      ENA => VCC1,
      ENB => VCC1,
      SSRA => GND1,
      SSRB => GND1,
      WEA => we(1),
      WEB => GND1);

  bram_3 : RAMB16_S9_S9
    generic map (
      INIT_00 =>
      X"000000000000000000000000000000000000000000000000FF00FF0000000000")
    --                                                  ^^
    --                              enable = 1, pattern 11
    --                                                    ^^
    --                                enable = 1, pattern 10
    --                                                      ^^
    --                                  enable = 1, pattern 01
    --                                                        ^^
    --                                    enable = 1, pattern 00
    port map (
      DOA => data_out(23 downto 16),
      DOB => ram_txdata(23 downto 16),
      ADDRA(10 downto 3) => GND32(10 downto 3),
      ADDRA(2) => vcc1,
      ADDRA(1 downto 0) => addr,
      ADDRB(10 downto 3) => GND32(10 downto 3),
      ADDRB(2) => enable,
      ADDRB(1 downto 0) => pulse_in,
      CLKA => clk,
      CLKB => event_clk,
      DIA => data_in(23 downto 16),
      DIB => GND32(7 downto 0),
      DIPA => GND32(0 downto 0),
      DIPB => GND32(0 downto 0),
      ENA => VCC1,
      ENB => VCC1,
      SSRA => GND1,
      SSRB => GND1,
      WEA => we(2),
      WEB => GND1);

  GT_CUSTOM_INST : GT_CUSTOM
    Generic map (
      ALIGN_COMMA_MSB => FALSE,
      CHAN_BOND_LIMIT => 16,
      CHAN_BOND_MODE => "OFF",
      CHAN_BOND_OFFSET => 8,
      CHAN_BOND_ONE_SHOT => FALSE,
      CHAN_BOND_SEQ_1_1 => "00000000000",
      CHAN_BOND_SEQ_1_2 => "00000000000",
      CHAN_BOND_SEQ_1_3 => "00000000000",
      CHAN_BOND_SEQ_1_4 => "00000000000",
      CHAN_BOND_SEQ_2_1 => "00000000000",
      CHAN_BOND_SEQ_2_2 => "00000000000",
      CHAN_BOND_SEQ_2_3 => "00000000000",
      CHAN_BOND_SEQ_2_4 => "00000000000",
      CHAN_BOND_SEQ_2_USE => FALSE,
      CHAN_BOND_SEQ_LEN => 1,
      CHAN_BOND_WAIT => 8,
      CLK_CORRECT_USE => FALSE,
      CLK_COR_INSERT_IDLE_FLAG => FALSE,
      CLK_COR_KEEP_IDLE => FALSE,
      CLK_COR_REPEAT_WAIT => 1,
      CLK_COR_SEQ_1_1 => "00000000000",
      CLK_COR_SEQ_1_2 => "00000000000",
      CLK_COR_SEQ_1_3 => "00000000000",
      CLK_COR_SEQ_1_4 => "00000000000",
      CLK_COR_SEQ_2_1 => "00000000000",
      CLK_COR_SEQ_2_2 => "00000000000",
      CLK_COR_SEQ_2_3 => "00000000000",
      CLK_COR_SEQ_2_4 => "00000000000",
      CLK_COR_SEQ_2_USE => FALSE,
      CLK_COR_SEQ_LEN => 1,
      COMMA_10B_MASK => "1111111111",
      CRC_END_OF_PKT => "K29_7",
      CRC_FORMAT => "USER_MODE",
      CRC_START_OF_PKT => "K27_7",
--      DEC_MCOMMA_DETECT => TRUE,
--      DEC_PCOMMA_DETECT => TRUE,
--      DEC_VALID_COMMA_ONLY => TRUE,
      DEC_MCOMMA_DETECT => FALSE,
      DEC_PCOMMA_DETECT => FALSE,
      DEC_VALID_COMMA_ONLY => FALSE,
      MCOMMA_10B_VALUE => "1100000101",
--      MCOMMA_DETECT => TRUE,
      MCOMMA_DETECT => FALSE,
      PCOMMA_10B_VALUE => "0011111010",
--      PCOMMA_DETECT => TRUE,
      PCOMMA_DETECT => FALSE,
--      RX_BUFFER_USE => TRUE,
      RX_BUFFER_USE => FALSE,
      RX_CRC_USE => FALSE,
      RX_DATA_WIDTH => 2,
      RX_DECODE_USE => TRUE,
--      RX_DECODE_USE => FALSE,
      RX_LOSS_OF_SYNC_FSM => TRUE,
--      RX_LOSS_OF_SYNC_FSM => FALSE,
      RX_LOS_INVALID_INCR => 1,
      RX_LOS_THRESHOLD => 4,
      TERMINATION_IMP => 50,
      SERDES_10B => FALSE,
--      TX_BUFFER_USE => TRUE,
      TX_BUFFER_USE => FALSE,
      TX_CRC_FORCE_VALUE => "11010110",
      TX_CRC_USE => FALSE,
      TX_DATA_WIDTH => 2,
      TX_DIFF_CTRL => 800,
      TX_PREEMPHASIS => 0,
      REF_CLK_V_SEL => 1)
     port map (
      CHBONDI(3 downto 0) => GND4(3 downto 0),
      CONFIGENABLE => GND1,
      CONFIGIN => GND1,
      ENMCOMMAALIGN => GND1,
      ENPCOMMAALIGN => GND1,
      ENCHANSYNC => GND1,
--      LOOPBACK(1 downto 0) => GND4(1 downto 0),
      LOOPBACK(1) => GND1,
      LOOPBACK(0) => GND1,
      POWERDOWN => powerdown,
      REFCLK => GND1,
      REFCLK2 => GND1,
      REFCLKSEL => refclksel,
      BREFCLK => brefclk,
      BREFCLK2 => brefclk2,
      RXN => RXN,
      RXP => RXP,
      RXPOLARITY => VCC1,
--      RXPOLARITY => GND1,
      RXRESET => VCC1,
      RXUSRCLK => GND1,
      RXUSRCLK2 => GND1,
      TXBYPASS8B10B(3 downto 0) => VCC4(3 downto 0),
      TXCHARDISPMODE(1) => txdata(19),
      TXCHARDISPMODE(0) => txdata(9),
      TXCHARDISPMODE(3 downto 2) => GND2,
      TXCHARDISPVAL(1) => txdata(18),
      TXCHARDISPVAL(0) => txdata(8),
      TXCHARDISPVAL(3 downto 2) => GND2,
      TXCHARISK(1 downto 0) => GND4(1 downto 0),
      TXCHARISK(3 downto 2) => GND4(1 downto 0),
      TXDATA(7 downto 0) => txdata(7 downto 0),
      TXDATA(15 downto 8) => txdata(17 downto 10),
      TXDATA(31 downto 16) => GND32(15 downto 0),
      TXFORCECRCERR => GND1,
      TXINHIBIT => GND1,
--      TXPOLARITY => GND1,
      TXPOLARITY => VCC1,
      TXRESET => txreset,
      TXUSRCLK => event_clk,
      TXUSRCLK2 => event_clk,
--      CONFIGOUT => CONFIGOUT,
--      RXBUFSTATUS(1 downto 0) => RXBUFSTATUS(1 downto 0),
      RXCHARISCOMMA(1 downto 0) => RXCHARISCOMMA_float(3 downto 2),
      RXCHARISCOMMA(3 downto 2) => RXCHARISCOMMA_float(1 downto 0),
--      RXCHARISK(1 downto 0) => RXCHARISK(1 downto 0),
      RXCHARISK(3 downto 0) => RXCHARISK_float(3 downto 0),
--      RXCOMMADET => RXCOMMADET,
--      RXDATA(15 downto 0) => RXDATA(15 downto 0),
--      RXDATA(31 downto 16) => RXDATA_float(15 downto 0),
--      RXDISPERR(1 downto 0) => RXDISPERR(1 downto 0),
--      RXDISPERR(3 downto 2) => RXDISPERR_float(1 downto 0),
--      RXLOSSOFSYNC(1 downto 0) => RXLOSSOFSYNC(1 downto 0),
--      RXNOTINTABLE(1 downto 0) => RXNOTINTABLE(1 downto 0),
--      RXNOTINTABLE(3 downto 2) => RXNOTINTABLE_float(1 downto 0),
--      RXREALIGN => RXREALIGN,
      RXRECCLK => RXRECCLK,
--      RXRUNDISP(1 downto 0) => RXRUNDISP(1 downto 0),
--      RXRUNDISP(3 downto 2) => RXRUNDISP_float(3 downto 2),
      TXBUFERR => TXBUFERR_float,
      TXKERR(1 downto 0) => TXKERR_float(1 downto 0),
      TXKERR(3 downto 2) => TXKERR_float(3 downto 2),
      TXN => TXN,
      TXP => TXP,
      TXRUNDISP(1 downto 0) => TXRUNDISP_float(1 downto 0),
      TXRUNDISP(3 downto 2) => TXRUNDISP_float(3 downto 2));

   GND1 <= '0';
   GND2 <= "00";
   GND4 <= "0000";
   GND32 <= "00000000000000000000000000000000";
   VCC1 <= '1';
   VCC4 <= "1111";
   GND4 <= "0000";

  process (freq_mode, ram_txdata, freq_txdata)
  begin
    txdata <= ram_txdata(19 downto 0);
    if freq_mode = '1' then
      txdata <= freq_txdata;
    end if;
  end process;
  
  process (event_clk, high_period, low_period, trig_pos, trig_level, pulse)
    variable acc        : std_logic_vector(16 downto 0) := (others => '0');
    variable freq_level : std_logic := '0';
  begin
    if rising_edge(event_clk) then
      if freq_level = '1' then
	freq_txdata <= (others => '1');
	if acc(acc'high) = '1' then
	  case acc(4 downto 0) is
	    when "11111" => -- -1
	      freq_txdata <= "11111111111111111110";
	    when "11110" => -- -2
	      freq_txdata <= "11111111111111111100";
	    when "11101" => -- -3
	      freq_txdata <= "11111111111111111000";
	    when "11100" => -- -4
	      freq_txdata <= "11111111111111110000";
	    when "11011" => -- -5
	      freq_txdata <= "11111111111111100000";
	    when "11010" => -- -6
	      freq_txdata <= "11111111111111000000";
	    when "11001" => -- -7
	      freq_txdata <= "11111111111110000000";
	    when "11000" => -- -8
	      freq_txdata <= "11111111111100000000";
	    when "10111" => -- -9
	      freq_txdata <= "11111111111000000000";
	    when "10110" => -- -10
	      freq_txdata <= "11111111110000000000";
	    when "10101" => -- -11
	      freq_txdata <= "11111111100000000000";
	    when "10100" => -- -12
	      freq_txdata <= "11111111000000000000";
	    when "10011" => -- -13
	      freq_txdata <= "11111110000000000000";
	    when "10010" => -- -14
	      freq_txdata <= "11111100000000000000";
	    when "10001" => -- -15
	      freq_txdata <= "11111000000000000000";
	    when "10000" => -- -16
	      freq_txdata <= "11110000000000000000";
	    when "01111" => -- -17
	      freq_txdata <= "11100000000000000000";
	    when "01110" => -- -18
	      freq_txdata <= "11000000000000000000";
	    when "01101" => -- -19
	      freq_txdata <= "10000000000000000000";
	    when others  =>
	      freq_txdata <= "00000000000000000000";
	  end case;
	  acc := acc + '0' & high_period;
	  freq_level := '0';
	end if;
      else
	freq_txdata <= (others => '0');
	if acc(acc'high) = '1' then
	  case acc(4 downto 0) is
	    when "11111" => -- -1
	      freq_txdata <= "00000000000000000001";
	    when "11110" => -- -2
	      freq_txdata <= "00000000000000000011";
	    when "11101" => -- -3
	      freq_txdata <= "00000000000000000111";
	    when "11100" => -- -4
	      freq_txdata <= "00000000000000001111";
	    when "11011" => -- -5
	      freq_txdata <= "00000000000000011111";
	    when "11010" => -- -6
	      freq_txdata <= "00000000000000111111";
	    when "11001" => -- -7
	      freq_txdata <= "00000000000001111111";
	    when "11000" => -- -8
	      freq_txdata <= "00000000000011111111";
	    when "10111" => -- -9
	      freq_txdata <= "00000000000111111111";
	    when "10110" => -- -10
	      freq_txdata <= "00000000001111111111";
	    when "10101" => -- -11
	      freq_txdata <= "00000000011111111111";
	    when "10100" => -- -12
	      freq_txdata <= "00000000111111111111";
	    when "10011" => -- -13
	      freq_txdata <= "00000001111111111111";
	    when "10010" => -- -14
	      freq_txdata <= "00000011111111111111";
	    when "10001" => -- -15
               freq_txdata <= "00000111111111111111";
	    when "10000" => -- -16
	      freq_txdata <= "00001111111111111111";
	    when "01111" => -- -17
	      freq_txdata <= "00011111111111111111";
	    when "01110" => -- -18
	      freq_txdata <= "00111111111111111111";
	    when "01101" => -- -19
               freq_txdata <= "01111111111111111111";
	    when others  =>
	      freq_txdata <= "11111111111111111111";
	  end case;
	  acc := acc + '0' & low_period;
	  freq_level := '1';
	end if;
      end if;
      acc := acc - 20;       
      pulse_in <= pulse_in(0) & pulse;
      if pulse_in = "01" then
	acc := '0' & trig_pos;
	freq_level := trig_level;
      end if;
    end if;
  end process;

  data_out(31 downto 24) <= (others => '0');
  
end struct;

