---------------------------------------------------------------------------
--                                                                       --
--  File        : micrel_load.vhd                                        --
--                                                                       --
--  Title       : Micrel SY87729L/SY87739L MicroWire Serial
--		  Configuration Logic
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--  		
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity micrel_load is
  port (
    clk           : in std_logic;
    data          : in std_logic_vector(31 downto 0);
    reload        : in std_logic;
    reset         : in std_logic;
    
    -- Micrel SY87729L serial bus signals
    CGSK          : out std_logic;
    CGDI          : out std_logic;
    CGCS          : out std_logic);
end micrel_load;

architecture rtl of micrel_load is

begin

  -- SY87729L Programming via MicroWire
  refclk_programming: process (clk, data, reload, reset)
    variable prescaler : std_logic_vector(6 downto 0);
    variable bit_counter : std_logic_vector(5 downto 0);
  begin
    if clk'event and clk = '1' then
      CGDI <= data(conv_integer(bit_counter(4 downto 0)));
      if prescaler(prescaler'high) = '1' then
	prescaler := (others => '0');
	if bit_counter(5) = '0' then
	  bit_counter := bit_counter - 1;
	end if;
      else
	prescaler := prescaler + 1;
      end if;

      CGCS <= not bit_counter(5);

      if reset = '1' or reload = '1' then
	prescaler := (others => '0');
	bit_counter := "011111";
	CGCS <= '0';
      end if;

      CGSK <= '0';
      if prescaler(prescaler'high - 1) = '1' and bit_counter(5) = '0' then
	CGSK <= '1';
      end if;
    end if;
  end process;

end rtl;
