---------------------------------------------------------------------------
--                                                                       --
--  File        : mx_counter.vhd                                         --
--                                                                       --
--  Title       : Event generator multiplexed counter                    --
--                                                                       --
--  Author      : Jukka Pietarinen                                       --
--                Micro-Research Finland Oy                              --
--                <jukka.pietarinen@mrf.fi>                              --
--                                                                       --
--  Description :                                                        --
--                                                                       --
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity mx_counter is
  generic (
    counter_range  : integer);
  port (
    reg_width      : in    std_logic_vector(31 downto 0);
    pulse_out      : out   std_logic;
    ena_out        : out   std_logic; -- output single '1' at the same
                                      -- time there is the first '1' on the
                                      -- rising edge of pulse_out
    clk            : in    std_logic;
    sync           : in    std_logic;
    polarity       : in    std_logic;
    reset          : in    std_logic);
end mx_counter;

architecture rtl of mx_counter is

  signal pulse         : std_logic;
  
begin

  width: process (reset, clk, reg_width, sync, pulse)
    variable counter : std_logic_vector(counter_range-1 downto 0);
    variable odddiv  : std_logic;
    variable pulse_1 : std_logic;
  begin
    pulse_out <= pulse;
    ena_out <= '0';
    if pulse = '1' and pulse_1 = '0' then
      ena_out <= '1';
    end if;
    if rising_edge(clk) then
      pulse_1 := pulse;
      if sync = '1' then
        pulse <= polarity;
        counter := '0' & (reg_width(counter_range-1 downto 1) - 1);
        odddiv := '0';
      end if;

      if counter(counter_range-1) = '1' then
        -- extend cycle by one when ls bit of prescaler is 1
        if reg_width(0) = '1' and pulse = '0' and odddiv = '0' then
          odddiv := '1';
        else
          odddiv := '0';
          pulse <= not pulse;
          counter := '0' & (reg_width(counter_range-1 downto 1) - 1);
        end if;
      end if;
	
      counter := counter - 1;

      if reset = '1' then
	pulse <= polarity;
	counter := (others => '1');
	odddiv := '0';
      end if;
    end if;
  end process;
  
end rtl;
