---------------------------------------------------------------------------
--
--  File        : pcs_rx.vhd
--
--  Title       : ECP2M Transceiver Quad
--		  For cRIO EVR
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity pcs_rx is
  port (
    refclkp         : in std_logic;
    refclkn         : in std_logic;
    refclk_out      : out std_logic;
    
    RXPOLARITY      : in std_logic;

    RXPOWERDOWN     : in std_logic;
    RXRESET         : in std_logic;

    TXCHARISK       : in std_logic_vector (1 downto 0);
    TXDATA          : in std_logic_vector (15 downto 0);
    TXRESET         : in std_logic;
    TXPOLARITY      : in std_logic;
    TXPOWERDOWN     : in std_logic;

    RXLOSS          : out std_logic_vector (1 downto 0);
    RXSYNC          : out std_logic;
    
    RXCHARISK       : out std_logic_vector (1 downto 0);
    RXDATA          : out std_logic_vector (15 downto 0);
    RXDISPERR       : out std_logic_vector (1 downto 0);
    RXNOTINTABLE    : out std_logic_vector (1 downto 0);

    RXCLK           : out std_logic;
    TXCLK           : out std_logic;

    PCSRESET        : in std_logic;
    
    -- PCS I/O pins
    hdinp0          : in std_logic;
    hdinn0          : in std_logic;
    hdinp1          : in std_logic;
    hdinn1          : in std_logic;
    hdinp2          : in std_logic;
    hdinn2          : in std_logic;
    hdinp3          : in std_logic;
    hdinn3          : in std_logic;

    hdoutp0         : out std_logic;
    hdoutn0         : out std_logic;
    hdoutp1         : out std_logic;
    hdoutn1         : out std_logic;
    hdoutp2         : out std_logic;
    hdoutn2         : out std_logic;
    hdoutp3         : out std_logic;
    hdoutn3         : out std_logic
    );
end pcs_rx;

architecture STRUCT of pcs_rx is
  signal ff_rxiclk_ch0          : std_logic;
  signal ff_txiclk_ch0          : std_logic;
  signal ff_ebrd_clk_0          : std_logic;
  signal ff_txdata_ch0          : std_logic_vector(15 downto 0);
  signal ff_tx_k_cntrl_ch0      : std_logic_vector(1 downto 0);
  signal ff_force_disp_ch0      : std_logic_vector(1 downto 0);
  signal ff_disp_sel_ch0        : std_logic_vector(1 downto 0);
  signal ff_correct_disp_ch0    : std_logic_vector(1 downto 0);
  signal ffc_rrst_ch0           : std_logic;
  signal ffc_lane_tx_rst_ch0    : std_logic;
  signal ffc_lane_rx_rst_ch0    : std_logic;
  signal ffc_txpwdnb_ch0        : std_logic;
  signal ffc_rxpwdnb_ch0        : std_logic;
  signal ff_rxdata_ch0          : std_logic_vector(15 downto 0);
  signal ff_rx_k_cntrl_ch0      : std_logic_vector(1 downto 0);
  signal ff_rxfullclk_ch0       : std_logic;
  signal ff_rxhalfclk_ch0       : std_logic;
  signal ff_disp_err_ch0        : std_logic_vector(1 downto 0);
  signal ff_cv_ch0              : std_logic_vector(1 downto 0);
  signal ffs_rlos_lo_ch0        : std_logic;
  signal ffs_rlol_ch0           : std_logic;
  signal oob_out_ch0            : std_logic;

  signal ff_rxiclk_ch1          : std_logic;
  signal ff_txiclk_ch1          : std_logic;
  signal ff_ebrd_clk_1          : std_logic;
  signal ff_txdata_ch1          : std_logic_vector(15 downto 0);
  signal ff_tx_k_cntrl_ch1      : std_logic_vector(1 downto 0);
  signal ff_force_disp_ch1      : std_logic_vector(1 downto 0);
  signal ff_disp_sel_ch1        : std_logic_vector(1 downto 0);
  signal ff_correct_disp_ch1    : std_logic_vector(1 downto 0);
  signal ffc_rrst_ch1           : std_logic;
  signal ffc_lane_tx_rst_ch1    : std_logic;
  signal ffc_lane_rx_rst_ch1    : std_logic;
  signal ffc_txpwdnb_ch1        : std_logic;
  signal ffc_rxpwdnb_ch1        : std_logic;
  signal ff_rxdata_ch1          : std_logic_vector(15 downto 0);
  signal ff_rx_k_cntrl_ch1      : std_logic_vector(1 downto 0);
  signal ff_rxfullclk_ch1       : std_logic;
  signal ff_rxhalfclk_ch1       : std_logic;
  signal ff_disp_err_ch1        : std_logic_vector(1 downto 0);
  signal ff_cv_ch1              : std_logic_vector(1 downto 0);
  signal ffs_rlos_lo_ch1        : std_logic;
  signal ffs_rlol_ch1           : std_logic;
  signal oob_out_ch1            : std_logic;

  signal ff_rxiclk_ch2          : std_logic;
  signal ff_txiclk_ch2          : std_logic;
  signal ff_ebrd_clk_2          : std_logic;
  signal ff_txdata_ch2          : std_logic_vector(7 downto 0);
  signal ff_tx_k_cntrl_ch2      : std_logic;
  signal ff_force_disp_ch2      : std_logic;
  signal ff_disp_sel_ch2        : std_logic;
  signal ff_correct_disp_ch2    : std_logic;
  signal ffc_rrst_ch2           : std_logic;
  signal ffc_lane_tx_rst_ch2    : std_logic;
  signal ffc_lane_rx_rst_ch2    : std_logic;
  signal ffc_txpwdnb_ch2        : std_logic;
  signal ffc_rxpwdnb_ch2        : std_logic;
  signal ff_rxdata_ch2          : std_logic_vector(7 downto 0);
  signal ff_rx_k_cntrl_ch2      : std_logic;
  signal ff_rxfullclk_ch2       : std_logic;
  signal ff_disp_err_ch2        : std_logic;
  signal ff_cv_ch2              : std_logic;
  signal ffs_rlos_lo_ch2        : std_logic;
  signal ffs_rlol_ch2           : std_logic;
  signal oob_out_ch2            : std_logic;

  signal ff_rxiclk_ch3          : std_logic;
  signal ff_txiclk_ch3          : std_logic;
  signal ff_ebrd_clk_3          : std_logic;
  signal ff_txdata_ch3          : std_logic_vector(7 downto 0);
  signal ff_tx_k_cntrl_ch3      : std_logic;
  signal ff_force_disp_ch3      : std_logic;
  signal ff_disp_sel_ch3        : std_logic;
  signal ff_correct_disp_ch3    : std_logic;
  signal ffc_rrst_ch3           : std_logic;
  signal ffc_lane_tx_rst_ch3    : std_logic;
  signal ffc_lane_rx_rst_ch3    : std_logic;
  signal ffc_txpwdnb_ch3        : std_logic;
  signal ffc_rxpwdnb_ch3        : std_logic;
  signal ff_rxdata_ch3          : std_logic_vector(7 downto 0);
  signal ff_rx_k_cntrl_ch3      : std_logic;
  signal ff_rxfullclk_ch3       : std_logic;
  signal ff_disp_err_ch3        : std_logic;
  signal ff_cv_ch3              : std_logic;
  signal ffs_rlos_lo_ch3        : std_logic;
  signal ffs_rlol_ch3           : std_logic;
  signal oob_out_ch3            : std_logic;

  signal ffc_macro_rst          : std_logic;
  signal ffc_quad_rst           : std_logic;
  signal ffc_trst               : std_logic;
  signal ff_txfullclk           : std_logic;
  signal ff_txhalfclk           : std_logic;
  signal refck2core             : std_logic;
  signal ffs_plol               : std_logic;

  COMPONENT pcs
    PORT(
      refclkp : IN std_logic;
      refclkn : IN std_logic;
      hdinp0 : IN std_logic;
      hdinn0 : IN std_logic;
      ff_rxiclk_ch0 : IN std_logic;
      ff_txiclk_ch0 : IN std_logic;
      ff_ebrd_clk_0 : IN std_logic;
      ff_txdata_ch0 : IN std_logic_vector(15 downto 0);
      ff_tx_k_cntrl_ch0 : IN std_logic_vector(1 downto 0);
      ff_force_disp_ch0 : IN std_logic_vector(1 downto 0);
      ff_disp_sel_ch0 : IN std_logic_vector(1 downto 0);
      ff_correct_disp_ch0 : IN std_logic_vector(1 downto 0);
      ffc_rrst_ch0 : IN std_logic;
      ffc_lane_tx_rst_ch0 : IN std_logic;
      ffc_lane_rx_rst_ch0 : IN std_logic;
      ffc_txpwdnb_ch0 : IN std_logic;
      ffc_rxpwdnb_ch0 : IN std_logic;
      hdinp1 : IN std_logic;
      hdinn1 : IN std_logic;
      ff_rxiclk_ch1 : IN std_logic;
      ff_txiclk_ch1 : IN std_logic;
      ff_ebrd_clk_1 : IN std_logic;
      ff_txdata_ch1 : IN std_logic_vector(15 downto 0);
      ff_tx_k_cntrl_ch1 : IN std_logic_vector(1 downto 0);
      ff_force_disp_ch1 : IN std_logic_vector(1 downto 0);
      ff_disp_sel_ch1 : IN std_logic_vector(1 downto 0);
      ff_correct_disp_ch1 : IN std_logic_vector(1 downto 0);
      ffc_rrst_ch1 : IN std_logic;
      ffc_lane_tx_rst_ch1 : IN std_logic;
      ffc_lane_rx_rst_ch1 : IN std_logic;
      ffc_txpwdnb_ch1 : IN std_logic;
      ffc_rxpwdnb_ch1 : IN std_logic;
      hdinp2 : IN std_logic;
      hdinn2 : IN std_logic;
      ff_rxiclk_ch2 : IN std_logic;
      ff_txiclk_ch2 : IN std_logic;
      ff_ebrd_clk_2 : IN std_logic;
      ff_txdata_ch2 : IN std_logic_vector(7 downto 0);
      ff_tx_k_cntrl_ch2 : IN std_logic;
      ff_force_disp_ch2 : IN std_logic;
      ff_disp_sel_ch2 : IN std_logic;
      ff_correct_disp_ch2 : IN std_logic;
      ffc_rrst_ch2 : IN std_logic;
      ffc_lane_tx_rst_ch2 : IN std_logic;
      ffc_lane_rx_rst_ch2 : IN std_logic;
      ffc_txpwdnb_ch2 : IN std_logic;
      ffc_rxpwdnb_ch2 : IN std_logic;
      hdinp3 : IN std_logic;
      hdinn3 : IN std_logic;
      ff_rxiclk_ch3 : IN std_logic;
      ff_txiclk_ch3 : IN std_logic;
      ff_ebrd_clk_3 : IN std_logic;
      ff_txdata_ch3 : IN std_logic_vector(7 downto 0);
      ff_tx_k_cntrl_ch3 : IN std_logic;
      ff_force_disp_ch3 : IN std_logic;
      ff_disp_sel_ch3 : IN std_logic;
      ff_correct_disp_ch3 : IN std_logic;
      ffc_rrst_ch3 : IN std_logic;
      ffc_lane_tx_rst_ch3 : IN std_logic;
      ffc_lane_rx_rst_ch3 : IN std_logic;
      ffc_txpwdnb_ch3 : IN std_logic;
      ffc_rxpwdnb_ch3 : IN std_logic;
      ffc_macro_rst : IN std_logic;
      ffc_quad_rst : IN std_logic;
      ffc_trst : IN std_logic;
      hdoutp0 : OUT std_logic;
      hdoutn0 : OUT std_logic;
      ff_rxdata_ch0 : OUT std_logic_vector(15 downto 0);
      ff_rx_k_cntrl_ch0 : OUT std_logic_vector(1 downto 0);
      ff_rxfullclk_ch0 : OUT std_logic;
      ff_rxhalfclk_ch0 : OUT std_logic;
      ff_disp_err_ch0 : OUT std_logic_vector(1 downto 0);
      ff_cv_ch0 : OUT std_logic_vector(1 downto 0);
      ffs_rlos_lo_ch0 : OUT std_logic;
      ffs_rlol_ch0 : OUT std_logic;
      oob_out_ch0 : OUT std_logic;
      hdoutp1 : OUT std_logic;
      hdoutn1 : OUT std_logic;
      ff_rxdata_ch1 : OUT std_logic_vector(15 downto 0);
      ff_rx_k_cntrl_ch1 : OUT std_logic_vector(1 downto 0);
      ff_rxfullclk_ch1 : OUT std_logic;
      ff_rxhalfclk_ch1 : OUT std_logic;
      ff_disp_err_ch1 : OUT std_logic_vector(1 downto 0);
      ff_cv_ch1 : OUT std_logic_vector(1 downto 0);
      ffs_rlos_lo_ch1 : OUT std_logic;
      ffs_rlol_ch1 : OUT std_logic;
      oob_out_ch1 : OUT std_logic;
      hdoutp2 : OUT std_logic;
      hdoutn2 : OUT std_logic;
      ff_rxdata_ch2 : OUT std_logic_vector(7 downto 0);
      ff_rx_k_cntrl_ch2 : OUT std_logic;
      ff_rxfullclk_ch2 : OUT std_logic;
      ff_disp_err_ch2 : OUT std_logic;
      ff_cv_ch2 : OUT std_logic;
      ffs_rlos_lo_ch2 : OUT std_logic;
      ffs_rlol_ch2 : OUT std_logic;
      oob_out_ch2 : OUT std_logic;
      hdoutp3 : OUT std_logic;
      hdoutn3 : OUT std_logic;
      ff_rxdata_ch3 : OUT std_logic_vector(7 downto 0);
      ff_rx_k_cntrl_ch3 : OUT std_logic;
      ff_rxfullclk_ch3 : OUT std_logic;
      ff_disp_err_ch3 : OUT std_logic;
      ff_cv_ch3 : OUT std_logic;
      ffs_rlos_lo_ch3 : OUT std_logic;
      ffs_rlol_ch3 : OUT std_logic;
      oob_out_ch3 : OUT std_logic;
      ff_txfullclk : OUT std_logic;
      ff_txhalfclk : OUT std_logic;
      refck2core : OUT std_logic;
      ffs_plol : OUT std_logic
      );
  END COMPONENT;
   
begin
  uut: pcs PORT MAP(
    refclkp => refclkp,
    refclkn => refclkn,
    hdinp0 => hdinp0,
    hdinn0 => hdinn0,
    hdoutp0 => hdoutp0,
    hdoutn0 => hdoutn0,
    ff_rxiclk_ch0 => ff_rxiclk_ch0,
    ff_txiclk_ch0 => ff_txiclk_ch0,
    ff_ebrd_clk_0 => ff_ebrd_clk_0,
    ff_txdata_ch0 => ff_txdata_ch0,
    ff_rxdata_ch0 => ff_rxdata_ch0,
    ff_tx_k_cntrl_ch0 => ff_tx_k_cntrl_ch0,
    ff_rx_k_cntrl_ch0 => ff_rx_k_cntrl_ch0,
    ff_rxfullclk_ch0 => ff_rxfullclk_ch0,
    ff_rxhalfclk_ch0 => ff_rxhalfclk_ch0,
    ff_force_disp_ch0 => ff_force_disp_ch0,
    ff_disp_sel_ch0 => ff_disp_sel_ch0,
    ff_correct_disp_ch0 => ff_correct_disp_ch0,
    ff_disp_err_ch0 => ff_disp_err_ch0,
    ff_cv_ch0 => ff_cv_ch0,
    ffc_rrst_ch0 => ffc_rrst_ch0,
    ffc_lane_tx_rst_ch0 => ffc_lane_tx_rst_ch0,
    ffc_lane_rx_rst_ch0 => ffc_lane_rx_rst_ch0,
    ffc_txpwdnb_ch0 => ffc_txpwdnb_ch0,
    ffc_rxpwdnb_ch0 => ffc_rxpwdnb_ch0,
    ffs_rlos_lo_ch0 => ffs_rlos_lo_ch0,
    ffs_rlol_ch0 => ffs_rlol_ch0,
    oob_out_ch0 => oob_out_ch0,
    hdinp1 => hdinp1,
    hdinn1 => hdinn1,
    hdoutp1 => hdoutp1,
    hdoutn1 => hdoutn1,
    ff_rxiclk_ch1 => ff_rxiclk_ch1,
    ff_txiclk_ch1 => ff_txiclk_ch1,
    ff_ebrd_clk_1 => ff_ebrd_clk_1,
    ff_txdata_ch1 => ff_txdata_ch1,
    ff_rxdata_ch1 => ff_rxdata_ch1,
    ff_tx_k_cntrl_ch1 => ff_tx_k_cntrl_ch1,
    ff_rx_k_cntrl_ch1 => ff_rx_k_cntrl_ch1,
    ff_rxfullclk_ch1 => ff_rxfullclk_ch1,
    ff_rxhalfclk_ch1 => ff_rxhalfclk_ch1,
    ff_force_disp_ch1 => ff_force_disp_ch1,
    ff_disp_sel_ch1 => ff_disp_sel_ch1,
    ff_correct_disp_ch1 => ff_correct_disp_ch1,
    ff_disp_err_ch1 => ff_disp_err_ch1,
    ff_cv_ch1 => ff_cv_ch1,
    ffc_rrst_ch1 => ffc_rrst_ch1,
    ffc_lane_tx_rst_ch1 => ffc_lane_tx_rst_ch1,
    ffc_lane_rx_rst_ch1 => ffc_lane_rx_rst_ch1,
    ffc_txpwdnb_ch1 => ffc_txpwdnb_ch1,
    ffc_rxpwdnb_ch1 => ffc_rxpwdnb_ch1,
    ffs_rlos_lo_ch1 => ffs_rlos_lo_ch1,
    ffs_rlol_ch1 => ffs_rlol_ch1,
    oob_out_ch1 => oob_out_ch1,
    hdinp2 => hdinp2,
    hdinn2 => hdinn2,
    hdoutp2 => hdoutp2,
    hdoutn2 => hdoutn2,
    ff_rxiclk_ch2 => ff_rxiclk_ch2,
    ff_txiclk_ch2 => ff_txiclk_ch2,
    ff_ebrd_clk_2 => ff_ebrd_clk_2,
    ff_txdata_ch2 => ff_txdata_ch2,
    ff_rxdata_ch2 => ff_rxdata_ch2,
    ff_tx_k_cntrl_ch2 => ff_tx_k_cntrl_ch2,
    ff_rx_k_cntrl_ch2 => ff_rx_k_cntrl_ch2,
    ff_rxfullclk_ch2 => ff_rxfullclk_ch2,
    ff_force_disp_ch2 => ff_force_disp_ch2,
    ff_disp_sel_ch2 => ff_disp_sel_ch2,
    ff_correct_disp_ch2 => ff_correct_disp_ch2,
    ff_disp_err_ch2 => ff_disp_err_ch2,
    ff_cv_ch2 => ff_cv_ch2,
    ffc_rrst_ch2 => ffc_rrst_ch2,
    ffc_lane_tx_rst_ch2 => ffc_lane_tx_rst_ch2,
    ffc_lane_rx_rst_ch2 => ffc_lane_rx_rst_ch2,
    ffc_txpwdnb_ch2 => ffc_txpwdnb_ch2,
    ffc_rxpwdnb_ch2 => ffc_rxpwdnb_ch2,
    ffs_rlos_lo_ch2 => ffs_rlos_lo_ch2,
    ffs_rlol_ch2 => ffs_rlol_ch2,
    oob_out_ch2 => oob_out_ch2,
    hdinp3 => hdinp3,
    hdinn3 => hdinn3,
    hdoutp3 => hdoutp3,
    hdoutn3 => hdoutn3,
    ff_rxiclk_ch3 => ff_rxiclk_ch3,
    ff_txiclk_ch3 => ff_txiclk_ch3,
    ff_ebrd_clk_3 => ff_ebrd_clk_3,
    ff_txdata_ch3 => ff_txdata_ch3,
    ff_rxdata_ch3 => ff_rxdata_ch3,
    ff_tx_k_cntrl_ch3 => ff_tx_k_cntrl_ch3,
    ff_rx_k_cntrl_ch3 => ff_rx_k_cntrl_ch3,
    ff_rxfullclk_ch3 => ff_rxfullclk_ch3,
    ff_force_disp_ch3 => ff_force_disp_ch3,
    ff_disp_sel_ch3 => ff_disp_sel_ch3,
    ff_correct_disp_ch3 => ff_correct_disp_ch3,
    ff_disp_err_ch3 => ff_disp_err_ch3,
    ff_cv_ch3 => ff_cv_ch3,
    ffc_rrst_ch3 => ffc_rrst_ch3,
    ffc_lane_tx_rst_ch3 => ffc_lane_tx_rst_ch3,
    ffc_lane_rx_rst_ch3 => ffc_lane_rx_rst_ch3,
    ffc_txpwdnb_ch3 => ffc_txpwdnb_ch3,
    ffc_rxpwdnb_ch3 => ffc_rxpwdnb_ch3,
    ffs_rlos_lo_ch3 => ffs_rlos_lo_ch3,
    ffs_rlol_ch3 => ffs_rlol_ch3,
    oob_out_ch3 => oob_out_ch3,
    ffc_macro_rst => ffc_macro_rst,
    ffc_quad_rst => ffc_quad_rst,
    ffc_trst => ffc_trst,
    ff_txfullclk => ff_txfullclk,
    ff_txhalfclk => ff_txhalfclk,
    refck2core => refck2core,
    ffs_plol => ffs_plol
    );
  
  refclk_out <= refck2core;

  RXLOSS(1) <= ffs_rlos_lo_ch0;
  RXLOSS(0) <= ffs_rlol_ch0;
  RXCHARISK <= ff_rx_k_cntrl_ch0;
  RXDATA <= ff_rxdata_ch0(15 downto 0);
  RXDISPERR <= ff_disp_err_ch0;
  RXNOTINTABLE <= ff_cv_ch0;
    
  RXCLK <= ff_rxhalfclk_ch0;
  TXCLK <= ff_txhalfclk;
  
  ffc_macro_rst <= PCSRESET;
  ffc_quad_rst <= PCSRESET;
  ffc_trst <= PCSRESET;

  ffc_rxpwdnb_ch0 <= not RXPOWERDOWN;
  ffc_txpwdnb_ch1 <= not TXPOWERDOWN;

  ff_rxiclk_ch0 <= ff_rxhalfclk_ch0;
  ff_txiclk_ch0 <= refck2core;
  ff_ebrd_clk_0 <= ff_rxfullclk_ch0;
  ff_txdata_ch0 <= (others => '0');
  ff_tx_k_cntrl_ch0 <= "00";
  ff_force_disp_ch0 <= "00";
  ff_disp_sel_ch0 <= "00";
  ff_correct_disp_ch0 <= "00";
  ffc_rrst_ch0 <= RXRESET;
  ffc_lane_rx_rst_ch0 <= '0';
  ffc_lane_tx_rst_ch0 <= '0';
  ffc_txpwdnb_ch0 <= '0';

  ff_rxiclk_ch1 <= ff_rxhalfclk_ch1;
  ff_txiclk_ch1 <= refck2core;
  ff_ebrd_clk_1 <= ff_rxfullclk_ch1;
  ff_txdata_ch1 <= TXDATA;
  ff_tx_k_cntrl_ch1 <= TXCHARISK;
  ff_force_disp_ch1 <= "00";
  ff_disp_sel_ch1 <= "00";
  ff_correct_disp_ch1 <= "00";
  ffc_rrst_ch1 <= '1';
  ffc_lane_rx_rst_ch1 <= '0';
  ffc_rxpwdnb_ch1 <= '0';

  ff_rxiclk_ch2 <= '0';
  ff_txiclk_ch2 <= '0';
  ff_ebrd_clk_2 <= '0';
  ff_txdata_ch2 <= (others => '0');
  ff_tx_k_cntrl_ch2 <= '0';
  ff_force_disp_ch2 <= '0';
  ff_disp_sel_ch2 <= '0';
  ff_correct_disp_ch2 <= '0';
  ffc_rrst_ch2 <= '1';
  ffc_lane_rx_rst_ch2 <= '1';
  ffc_lane_tx_rst_ch2 <= '1';
  ffc_rxpwdnb_ch2 <= '0';
  ffc_txpwdnb_ch2 <= '0';

  ff_rxiclk_ch3 <= '0';
  ff_txiclk_ch3 <= '0';
  ff_ebrd_clk_3 <= '0';
  ff_txdata_ch3 <= (others => '0');
  ff_tx_k_cntrl_ch3 <= '0';
  ff_force_disp_ch3 <= '0';
  ff_disp_sel_ch3 <= '0';
  ff_correct_disp_ch3 <= '0';
  ffc_rrst_ch3 <= '1';
  ffc_lane_rx_rst_ch3 <= '1';
  ffc_lane_tx_rst_ch3 <= '1';
  ffc_rxpwdnb_ch3 <= '0';
  ffc_txpwdnb_ch3 <= '0';

end STRUCT;

