---------------------------------------------------------------------------
--
--  File        : pmcevr200_pkg.vhd
--
--  Title       : Event Receiver PMC-EVR-200 board specific package
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use work.evr_pkg.all;

package evr_bs_pkg is
  -- Local OPB address definitions
  constant LOCAL_BASE_MASK  : std_logic_vector := X"80000000";
  constant EVR_LOCAL_BASE   : std_logic_vector := X"00000000";

  -- Event Receiver configuration parameters
  -- C_EVR_PULSE_GENS sets the number of internal pulse generators
  constant C_EVR_PULSE_GENS    : integer := 16;
  constant C_EVR_TTL_INPUTS    : integer := 1;
  -- C_EVR_TTL_OUTPUTS defines the number of front panel TTL outputs
  constant C_EVR_TTL_OUTPUTS   : integer := 3;
  -- C_EVR_CML_OUTPUTS defines the number of front panel CML outputs
  -- note: the CML output mapping registers are appended after the
  -- TTL output mapping registers
  constant C_EVR_CML_OUTPUTS   : integer := 0;
  -- C_EVR_UNIV_OUTPUTS defines the number of Universal outputs
  -- = twice the number of Universal I/O slots
  constant C_EVR_UNIV_OUTPUTS  : integer := 0;
  -- C_EVR_TB_OUTPUTS defines the number of Transition Board/Rear I/O/
  -- PXI star trigger/trigger bus outputs
  constant C_EVR_TB_OUTPUTS    : integer := 32;
  -- C_EVR_USE_RESYNC defines if the board supports the event clock
  -- resynchronizing circuit to reduce jitter (this is a requirement
  -- for CML outputs)
  constant C_EVR_USE_RESYNC    : boolean := FALSE;
  -- C_EVR_REF_BREFCLK defines if reference clock from fractional
  -- synthesizer is on BREFCLKN/BREFCLKP (TRUE) or
  -- BREFCLK2N/BREFCLK2P (FALSE)
  constant C_EVR_REF_BREFCLK   : boolean := TRUE;
  -- C_EVR_PRESCALERS defines the number of prescalers
  constant C_EVR_PRESCALERS    : integer := 3;
  -- C_EVR_RXPOLARITY defines the event link receiver polarity
  -- '0' - normal polarity, '1' - inverted polarity
  constant C_EVR_RXPOLARITY    : std_logic := '1';
  constant C_EVR_PULSE_PRESC_RANGE : integer_array(0 to C_EVR_PULSE_GENS-1) :=
    (16, 16, 16, 16,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0);
  constant C_EVR_PULSE_DELAY_RANGE : integer_array(0 to C_EVR_PULSE_GENS-1) :=
    (32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32);
  constant C_EVR_PULSE_WIDTH_RANGE : integer_array(0 to C_EVR_PULSE_GENS-1) :=
    (32, 32, 32, 32, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16);
  constant C_EVR_PRESC_RANGE : integer_array(0 to C_EVR_PRESCALERS-1) :=
    (16, 16, 16);
end evr_bs_pkg;
