---------------------------------------------------------------------------
--
--  File        : prescaler.vhd
--
--  Title       : Event receiver programmable prescaler
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@iki.fi>
--
--  Description : 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity prescaler is
  generic (
    prescaler_range : integer := 16);      
  port (
    presc     : in std_logic_vector(prescaler_range-1 downto 0);
    pulse_out : out std_logic;
    event_clk : in std_logic;
    reset     : in std_logic);
end prescaler;

architecture structure of prescaler is

  signal pulse         : std_logic;

begin

  prescaling : process (reset, event_clk, presc, pulse)
    variable counter  : std_logic_vector(prescaler_range-1 downto 0);
    variable odddiv   : std_logic;
    variable polarity : std_logic := '1';
    variable sync     : std_logic := '0';
  begin
    pulse_out <= pulse;
    if event_clk'event and event_clk = '1' then
      if sync = '1' then
        pulse <= polarity;
        counter := '0' & (presc(prescaler_range-1 downto 1) - 1);
        odddiv := '0';
      end if;

      if counter(15) = '1' then
        -- extend cycle by one when ls bit of prescaler is 1
        if presc(0) = '1' and pulse = '0' and odddiv = '0' then
          odddiv := '1';
        else
          odddiv := '0';
          pulse <= not pulse;
          counter := '0' & (presc(prescaler_range-1 downto 1) - 1);
        end if;
      end if;

      counter := counter - 1;

      if reset = '1' then
        pulse <= polarity;
        counter := (others => '1');
        odddiv := '0';
      end if;
    end if;
  end process;
  
end structure;
