---------------------------------------------------------------------------
--
--  File        : sync_domains.vhd
--
--  Title       : Propagate sync/reset signal from one clock domain to
--                another
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
--  Description : The purpose of this block is to transfer a trigger or
--                sync/reset signal from one clock domain to another.
--                The input trigger (trig_in) does not need to be active
--                for more than a single cycle and the output (trig_out)
--                will be active for at least one destination (clk_out)
--                cycle.
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity sync_domain is
  port (
    clk_in   : in std_logic;
    trig_in  : in std_logic;
    clk_out  : in std_logic;
    trig_out : out std_logic);
end sync_domain;

architecture structure of sync_domain is

  signal propagate_trig : std_logic;

begin

  source_domain: process (clk_in, trig_in)
    variable ack_sync : std_logic;
    variable ext_trig : std_logic_vector(3 downto 0);
  begin
    if rising_edge(clk_in) then
      -- we make the trigger longer
      ext_trig := ext_trig(2 downto 0) & trig_in;
      propagate_trig <= '0';
      if ext_trig /= "0000" then
	propagate_trig <= '1';
      end if;
    end if;
  end process;

  destination_domain: process (clk_out, propagate_trig)
    variable trig_sync : std_logic_vector(3 downto 0);
  begin
    if rising_edge(clk_out) then
      -- sychronise
      trig_sync := trig_sync(2 downto 0) & propagate_trig;
      trig_out <= '0';
      if trig_sync(3 downto 2) = "01" then
	trig_out <= '1';
      end if;
    end if;
  end process;

end structure;
