---------------------------------------------------------------------------
--
--  File        : transmitter.vhd
--
--  Title       : Event Generator Multi-Gigabit Transmitter
--
--  Author      : Jukka Pietarinen
--                Micro-Research Finland Oy
--                <jukka.pietarinen@mrf.fi>
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.evr_pkg.all;
-- library unisim;
-- use unisim.vcomponents.all;

entity transmitter is
  generic (
    ref_on_brefclk  : boolean);
  port (
    brefclk_dcm     : in std_logic;
    brefclk2_dcm    : in std_logic;

    tx_clk          : out std_logic;
    
    -- Transmitter side connections
    tx_event        : in std_logic_vector(7 downto 0);
    tx_dbus         : in std_logic_vector(7 downto 0);
    tx_databuf      : in std_logic_vector(7 downto 0);
    k_databuf       : in std_logic;
    ena_databuf     : out std_logic;
    databuf_mode    : in std_logic;
    
    tx_charisk      : out std_logic_vector(1 downto 0);
    tx_data         : out std_logic_vector(15 downto 0));
end transmitter;

architecture structure of transmitter is

  signal refclksel     : std_logic;

  signal event_clk     : std_logic;
  
  signal tx_mgt_pwd    : std_logic;

begin

  tx_clk <= event_clk;
  
  refclock_source_1:
    if ref_on_brefclk generate
      refclksel <= '0';
      event_clk <= brefclk_dcm;
    end generate;

  refclock_source_2:
    if not ref_on_brefclk generate
      refclksel <= '1';
      event_clk <= brefclk2_dcm;
    end generate;

  transmit_data : process (event_clk, tx_event, tx_dbus, tx_databuf,
			   k_databuf, databuf_mode)
    variable even : std_logic := '0';
  begin
    if rising_edge(event_clk) then
      tx_charisk <= "00";
      tx_data(15 downto 8) <= tx_event;
      if tx_event = X"00" and even = '0' then
	tx_charisk <= "10";
	tx_data(15 downto 8) <= X"BC"; -- K28.5 character
      end if;
      tx_data(7 downto 0) <= tx_dbus;
      if even = '1' and databuf_mode = '1' then
	tx_data(7 downto 0) <= tx_databuf;
	tx_charisk(0) <= k_databuf;
      end if;
      ena_databuf <= even;
      even := not even;
    end if;
  end process;

end structure;
